@@ -14,6 +14,17 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+#
+# In addition, as a special exception, the copyright holders give
+# permission to link the code of portions of this program with the OpenSSL
+# library.
+#
+# You must obey the GNU General Public License in all respects for all of
+# the code used other than OpenSSL. If you modify file(s) with this
+# exception, you may extend this exception to your version of the file(s),
+# but you are not obligated to do so. If you do not wish to do so, delete
+# this exception statement from your version. If you delete this exception
+# statement from all source files in the program, then also delete it here.
 
 import app
 import pygtk
@@ -184,6 +195,7 @@ class MainFrame:
         UIBackendDelegate.dialogParent = self.widgetTree['main-window']
 
         self.widgetTree['main-window'].set_icon_from_file (resources.sharePath('pixmaps/miro-128x128.png'))
+        self.widgetTree['main-window'].set_title(config.get(prefs.LONG_APP_NAME))
 
         # create the buttonsDown attribute to the video time scale.  It will
         # track which mouse buttons are currently pressed.  This is usefull
@@ -219,8 +231,8 @@ class MainFrame:
         ui_string = "<ui>\n"
         ui_string += "  <menubar>\n"
         for menu in menubar.menubar:
-            if actions.has_key(menu.action):
-                ui_string += '  <menu action="%s">\n' % menu.action
+            if actions.has_key('toplevel-%s' % menu.action):
+                ui_string += '  <menu action="toplevel-%s">\n' % menu.action
                 for menuitem in menu.menuitems:
                     if isinstance(menuitem, menubar.Separator):
                         ui_string += '      <separator/>\n'
@@ -385,8 +397,10 @@ class MainFrame:
         platformutils.confirmMainThread()
         if (self.aboutWidget is None):
             self.aboutWidget = gtk.AboutDialog()
-            self.aboutWidget.set_name("Miro")
-            self.aboutWidget.set_version(config.get(prefs.APP_VERSION))
+            self.aboutWidget.set_name(config.get(prefs.SHORT_APP_NAME))
+            self.aboutWidget.set_version( "%s (r%s)" % \
+                                          (config.get(prefs.APP_VERSION), 
+                                           config.get(prefs.APP_REVISION_NUM)))
             self.aboutWidget.set_website("http://www.getmiro.com/")
             self.aboutWidget.set_copyright("Copyright (c) 2005-2007.  See LICENSE file for details.\nMiro and the Miro logo are trademarks of the Participatory Culture Foundation.")
             def delete_event_cb(widget, event):
