@@ -14,6 +14,17 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+#
+# In addition, as a special exception, the copyright holders give
+# permission to link the code of portions of this program with the OpenSSL
+# library.
+#
+# You must obey the GNU General Public License in all respects for all of
+# the code used other than OpenSSL. If you modify file(s) with this
+# exception, you may extend this exception to your version of the file(s),
+# but you are not obligated to do so. If you do not wish to do so, delete
+# this exception statement from your version. If you delete this exception
+# statement from all source files in the program, then also delete it here.
 
 import errno
 import os
@@ -110,13 +121,18 @@ def EscapeMessagePart(message_part):
 def BuildDialog (title, message, buttons, default):
     flags = gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT
     dialog = gtk.Dialog(title, dialogParent, flags, buttons)
+    dialog.myvbox = gtk.VBox()
+    dialog.myvbox.set_border_width(6)
+    dialog.vbox.add(dialog.myvbox)
+    dialog.myvbox.show()
+
     dialog.set_default_size(425, -1)
     label = gtk.Label()
     label.set_line_wrap(True)
     label.set_selectable(True)
     label.set_markup(message)
     label.set_padding (6, 6)
-    dialog.vbox.add(label)
+    dialog.myvbox.add(label)
     label.show()
     dialog.set_default_response (default)
     return dialog
@@ -125,7 +141,7 @@ def BuildTextEntryDialog(title, message, buttons, default, prefillCallback, fill
     dialog = BuildDialog(title, message, buttons, default)
     dialog.entry = gtk.Entry()
     dialog.entry.set_activates_default(True)
-    dialog.vbox.add(dialog.entry)
+    dialog.myvbox.add(dialog.entry)
     
     prefill = None
     if fillWithClipboardURL:
@@ -146,6 +162,33 @@ def BuildTextEntryDialog(title, message, buttons, default, prefillCallback, fill
     dialog.entry.show()
     return dialog
 
+def BuildCheckboxDialog(title, message, checkbox_text, buttons, default, checkbox_value):
+    dialog = BuildDialog(title, message, buttons, default)
+    alignment = gtk.Alignment(.5, 0, 0, 0)
+    dialog.myvbox.add(alignment)
+    alignment.show()
+    dialog.checkbox = gtk.CheckButton(checkbox_text)
+    dialog.checkbox.set_active (checkbox_value)
+    alignment.add(dialog.checkbox)
+    dialog.checkbox.show()
+    return dialog
+
+def BuildCheckboxTextboxDialog(title, message, checkbox_text, buttons, default, checkbox_value, textbox_value):
+    dialog = BuildCheckboxDialog(title, message, checkbox_text, buttons, default, checkbox_value)
+    dialog.textbox = gtk.TextView()
+    dialog.scrollWindow = gtk.ScrolledWindow()
+    dialog.scrollWindow.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_AUTOMATIC)
+    dialog.scrollWindow.add(dialog.textbox)
+    dialog.scrollWindow.set_shadow_type(gtk.SHADOW_IN)
+    
+    textbuffer = dialog.textbox.get_buffer();
+    textbuffer.set_text(textbox_value)
+    dialog.textbox.set_editable(True)
+    dialog.myvbox.add(dialog.scrollWindow)
+    dialog.textbox.show()
+    dialog.scrollWindow.show()
+    return dialog
+
 def BuildHTTPAuth(summary, message, prefillUser = None, prefillPassword = None):
     """Ask the user for HTTP login information for a location, identified
     to the user by its URL and the domain string provided by the
@@ -157,7 +200,7 @@ def BuildHTTPAuth(summary, message, prefillUser = None, prefillPassword = None):
     dialog = gtk.Dialog(summary, None, (), (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK))
     dialog.set_default_size(425, -1)
     table = gtk.Table()
-    dialog.vbox.add(table)
+    dialog.myvbox.add(table)
     
     label = gtk.Label()
     label.set_line_wrap(True)
@@ -310,6 +353,18 @@ def ShowTextEntryDialogAsync(title, description, buttons, default, prefillCallba
     gtkDialog.connect("response", callback)
     gtkDialog.show()
 
+@gtkAsyncMethod
+def ShowCheckboxDialogAsync(title, description, checkbox_text, buttons, default, checkbox_value, callback):
+    gtkDialog = BuildCheckboxDialog (title, description, checkbox_text, buttons, default, checkbox_value)
+    gtkDialog.connect("response", callback)
+    gtkDialog.show()
+
+@gtkAsyncMethod
+def ShowCheckboxTextboxDialogAsync(title, description, checkbox_text, buttons, default, checkbox_value, textbox_value, callback):
+    gtkDialog = BuildCheckboxTextboxDialog (title, description, checkbox_text, buttons, default, checkbox_value, textbox_value)
+    gtkDialog.connect("response", callback)
+    gtkDialog.show()
+
 def pidIsRunning(pid):
     if pid is None:
         return False
@@ -332,6 +387,9 @@ def init_clipboard ():
 
 class UIBackendDelegate:
 
+    def maximizeWindow(self):
+        logging.warn("UIBackendDelegate.maximizeWindow() not implemented")
+
     def performStartupTasks(self, terminationCallback):
         import startup
         startup.performStartupTasks(terminationCallback)
@@ -424,6 +482,35 @@ class UIBackendDelegate:
             ShowTextEntryDialogAsync (EscapeMessagePart(dialog.title), EscapeMessagePart(dialog.description), self.makeButtonTuple(dialog), default=0,
                                       prefillCallback=dialog.prefillCallback, fillWithClipboardURL=dialog.fillWithClipboardURL,
                                       callback = AsyncDialogResponse)
+        elif isinstance(dialog, dialogs.CheckboxTextboxDialog):
+            def AsyncDialogResponse(gtkDialog, response):
+                retval = None
+                if response == gtk.RESPONSE_DELETE_EVENT:
+                    dialog.runCallback (None)
+                elif response >= 0 and response < len(dialog.buttons):
+                    dialog.runCallback (dialog.buttons [response], gtkDialog.checkbox.get_active(), gtkDialog.textbox.get_buffer().get_text(gtkDialog.textbox.get_buffer().get_start_iter(),gtkDialog.textbox.get_buffer().get_end_iter()))
+                else:
+                    dialog.runCallback (None)
+                gtkDialog.destroy()
+
+            ShowCheckboxTextboxDialogAsync (EscapeMessagePart(dialog.title), EscapeMessagePart(dialog.description), EscapeMessagePart(dialog.checkbox_text), self.makeButtonTuple(dialog), default=0,
+                                     checkbox_value = dialog.checkbox_value,
+                                     textbox_value = EscapeMessagePart(dialog.textbox_value),
+                                     callback = AsyncDialogResponse)
+        elif isinstance(dialog, dialogs.CheckboxDialog):
+            def AsyncDialogResponse(gtkDialog, response):
+                retval = None
+                if response == gtk.RESPONSE_DELETE_EVENT:
+                    dialog.runCallback (None)
+                elif response >= 0 and response < len(dialog.buttons):
+                    dialog.runCallback (dialog.buttons [response], gtkDialog.checkbox.get_active())
+                else:
+                    dialog.runCallback (None)
+                gtkDialog.destroy()
+
+            ShowCheckboxDialogAsync (EscapeMessagePart(dialog.title), EscapeMessagePart(dialog.description), EscapeMessagePart(dialog.checkbox_text), self.makeButtonTuple(dialog), default=0,
+                                     checkbox_value = dialog.checkbox_value,
+                                     callback = AsyncDialogResponse)
         elif isinstance(dialog, dialogs.SearchChannelDialog):
             def AsyncDialogResponse(gtkDialog, response):
                 retval = None
@@ -459,6 +546,34 @@ class UIBackendDelegate:
         else:
             dialog.runCallback (None)
 
+
+    def askForOpenPathname(self, title, callback, defaultDirectory=None,
+            typeString=None, types=None):
+        dialog = gtk.FileSelection("File Selection")
+        def okButton(w):
+            print dialog.get_filename()
+            callback(dialog.get_filename())
+            dialog.destroy()
+        dialog.ok_button.connect("clicked", okButton)
+        dialog.cancel_button.connect("clicked", lambda w: dialog.destroy())
+        if defaultDirectory is not None:
+            dialog.set_filename(defaultDirectory)
+        dialog.show()
+
+    def askForSavePathname(self, title, callback, defaultDirectory=None, 
+            defaultFilename=None):
+        dialog = gtk.FileSelection("File Selection")
+        def okButton(w):
+            print dialog.get_filename()
+            callback(dialog.get_filename())
+            dialog.destroy()
+        dialog.ok_button.connect("clicked", okButton)
+        dialog.cancel_button.connect("clicked", lambda w: dialog.destroy())
+        if defaultFilename is not None:
+            dialog.set_filename(defaultFilename)
+        dialog.show()
+
+
     @gtkAsyncMethod
     def showContextMenu(self, menuItems):
         menu = gtk.Menu()
@@ -486,24 +601,24 @@ class UIBackendDelegate:
         clipboard.set_text(text)
         primary.set_text(text)
 
-    def killDownloadDaemon(self, oldpid):
-        if oldpid is None:
+    def killProcess(self, pid):
+        if pid is None:
             return
-        if pidIsRunning(oldpid):
+        if pidIsRunning(pid):
             try:
-                os.kill(oldpid, signal.SIGTERM)
+                os.kill(pid, signal.SIGTERM)
                 for i in xrange(100):
                     time.sleep(.01)
-                    if not pidIsRunning(oldpid):
+                    if not pidIsRunning(pid):
                         return
-                os.kill(oldpid, signal.SIGKILL)
+                os.kill(pid, signal.SIGKILL)
             except:
                 logging.exception ("error killing download daemon")
 
     def launchDownloadDaemon(self, oldpid, env):
         # Use UNIX style kill
         if oldpid is not None and pidIsRunning(oldpid):
-            self.killDownloadDaemon(oldpid)
+            self.killProcess(oldpid)
 
         environ = os.environ.copy()
         import miro
