@@ -14,6 +14,17 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+#
+# In addition, as a special exception, the copyright holders give
+# permission to link the code of portions of this program with the OpenSSL
+# library.
+#
+# You must obey the GNU General Public License in all respects for all of
+# the code used other than OpenSSL. If you modify file(s) with this
+# exception, you may extend this exception to your version of the file(s),
+# but you are not obligated to do so. If you do not wish to do so, delete
+# this exception statement from your version. If you delete this exception
+# statement from all source files in the program, then also delete it here.
 
 """Frontend callback handler.  Responsible for handling all GUI actions."""
 
@@ -147,7 +158,9 @@ def SetupDirList (widgetTree, toggleRenderer):
                                         gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
         response = dialog.run()
         if response == gtk.RESPONSE_ACCEPT:
-            addFeed (dialog.get_filename())
+            filename = dialog.get_filename()
+            if filename:
+                addFeed (filename)
         dialog.destroy()
 
     def toggled (renderer, path):
@@ -227,6 +240,15 @@ class Mediator:
         if isinstance (mapped.actualFeed, feed.DirectoryWatchFeedImpl):
             self.removeDirectory (id)
 
+def _buildAction(action):
+    impl = menubar.getImpl(action)
+    if impl:
+        return (action, None, menubar.getLabel(action), menubar.getShortcut(action).GTKString(), None, lambda event: eventloop.addIdle(impl, action))
+    else:
+        return (action, None, menubar.getLabel(action), menubar.getShortcut(action).GTKString(), None, None)
+        
+    
+
 class CallbackHandler(object):
     """Class to handle menu item activation, button presses, etc.  The method
     names for this class correspond to the event handler that they implement.
@@ -298,23 +320,27 @@ class CallbackHandler(object):
             ('RemovePlaylists', None, menubar.getLabel('RemovePlaylists'), menubar.getShortcut('RemovePlaylists').GTKString(), None, self.on_remove_playlist_activate),
             ])
         actionGroups["Ubiquitous"].add_actions ([
-            ('Video', None, menubar.getLabel('Video')),
-            ('Channels', None, menubar.getLabel('Channels')),
-            ('Playlists', None, menubar.getLabel('Playlists')),
-            ('Playback', None, menubar.getLabel('Playback')),
-            ('Open', gtk.STOCK_OPEN, menubar.getLabel('Open'), menubar.getShortcut('Open').GTKString(), None, self.on_open_video_activate),
+            ('toplevel-Video', None, menubar.getLabel('Video')),
+            ('toplevel-Channels', None, menubar.getLabel('Channels')),
+            ('toplevel-Playlists', None, menubar.getLabel('Playlists')),
+            ('toplevel-Playback', None, menubar.getLabel('Playback')),
+            ('toplevel-Help', None, menubar.getLabel('Help')),
 
+            ('Open', gtk.STOCK_OPEN, menubar.getLabel('Open'), menubar.getShortcut('Open').GTKString(), None, self.on_open_video_activate),
             ('NewPlaylist', None, menubar.getLabel('NewPlaylist'), menubar.getShortcut('NewPlaylist').GTKString(), None, self.on_new_playlist_activate),
             ('NewPlaylistFolder', None, menubar.getLabel('NewPlaylistFolder'), menubar.getShortcut('NewPlaylistFolder').GTKString(), None, self.on_new_playlist_folder_activate),
             ('NewChannelFolder', None, menubar.getLabel('NewChannelFolder'), menubar.getShortcut('NewChannelFolder').GTKString(), None, self.on_new_channel_folder_activate),
             ('NewChannel', None, menubar.getLabel('NewChannel'), menubar.getShortcut('NewChannel').GTKString(), None, self.on_add_channel_button_clicked),
             ('NewSearchChannel', None, menubar.getLabel('NewSearchChannel'), menubar.getShortcut('NewSearchChannel').GTKString(), None, self.on_add_search_channel_button_clicked),
             ('NewGuide', None, _("New Channel _Guide..."), None, None, self.on_add_guide_button_clicked),
+            _buildAction('NewDownload'),
+            _buildAction('ImportChannels'),
+            _buildAction('ExportChannels'),
 
             ('EditPreferences', gtk.STOCK_PREFERENCES, menubar.getLabel('EditPreferences'), menubar.getShortcut('EditPreferences').GTKString(), None, self.on_preference),
             ('Quit', gtk.STOCK_QUIT, menubar.getLabel('Quit'), menubar.getShortcut('Quit').GTKString(), None, self.on_quit_activate),
             ('UpdateAllChannels', None, menubar.getLabel('UpdateAllChannels'), menubar.getShortcut('UpdateAllChannels').GTKString(), None, self.on_update_all_channels_activate),
-            ('Help', None, menubar.getLabel('Help')),
+            ('Help', None, menubar.getLabel('Help'), menubar.getShortcut('Help').GTKString(), None, self.on_help_clicked),
             ('ReportBug', None, menubar.getLabel('ReportBug'), menubar.getShortcut('ReportBug').GTKString(), None, self.on_report_bug_clicked),
             ('About', gtk.STOCK_ABOUT, menubar.getLabel('About'), menubar.getShortcut('About').GTKString(), None, self.on_about_clicked),
             ('Donate', None, menubar.getLabel('Donate'), menubar.getShortcut('Donate').GTKString(), None, self.on_donate_clicked),
@@ -442,8 +468,7 @@ class CallbackHandler(object):
                 return os.path.splitext(path)[1]
             if getExt(savePath) == '':
                 savePath += getExt(videoPath)
-            logging.info ("saving %r to %r", videoPath, savePath)
-            shutil.copyfile(videoPath, savePath)
+            app.saveVideo(videoPath, savePath)
         CallbackHandler.current_folder = chooser.get_current_folder()
         chooser.destroy()
 
@@ -491,10 +516,10 @@ class CallbackHandler(object):
         eventloop.addIdle (app.controller.copyCurrentFeedURL, "Copy feed URL")
 
     def on_mail_channel_link_activate(self, event = None):
-        print "Mail Chanel Link unimplemented"
+        eventloop.addIdle (app.controller.recommendCurrentFeed, "Copy feed URL")
 
     def on_copy_video_link_activate(self, event = None):
-        print "Copy Video Link unimplemented"
+        eventloop.addIdle(lambda:app.controller.copyCurrentItemURL, "Copy Item URL")
 
     def on_add_channel_button_clicked(self, event = None):
         eventloop.addIdle(lambda:app.controller.addAndSelectFeed(), "Add Channel")
@@ -520,6 +545,7 @@ class CallbackHandler(object):
         movie_dir = config.get(prefs.MOVIES_DIRECTORY)
         widgetTree = MainFrame.WidgetTree(resources.path('miro.glade'), 'dialog-preferences', 'miro')
         dialog = widgetTree['dialog-preferences']
+        widgetTree['prefs-notebook'].set_property("homogeneous", True)
         dialog.integerWidgets = []
         dialog.floatWidgets = []
         mainWindow = self.mainFrame.widgetTree['main-window']
@@ -528,6 +554,8 @@ class CallbackHandler(object):
         AttachBoolean (dialog, widgetTree['checkbutton-padding'], prefs.PRESERVE_DISK_SPACE, widgetTree['spinbutton-padding'])
         AttachBoolean (dialog, widgetTree['checkbutton-autorun'], prefs.RUN_DTV_AT_STARTUP)
         AttachInteger (dialog, widgetTree['spinbutton-limit'], prefs.UPSTREAM_LIMIT_IN_KBS)
+        AttachBoolean (dialog, widgetTree['checkbutton-down-limit'], prefs.LIMIT_DOWNSTREAM_BT, widgetTree['spinbutton-down-limit'])
+        AttachInteger (dialog, widgetTree['spinbutton-down-limit'], prefs.DOWNSTREAM_BT_LIMIT_IN_KBS)
         AttachInteger (dialog, widgetTree['spinbutton-bt-min-port'], prefs.BT_MIN_PORT)
         AttachInteger (dialog, widgetTree['spinbutton-bt-max-port'], prefs.BT_MAX_PORT)
         AttachInteger (dialog, widgetTree['spinbutton-max-manual'], prefs.MAX_MANUAL_DOWNLOADS)
@@ -536,6 +564,9 @@ class CallbackHandler(object):
         AttachCombo   (dialog, widgetTree['combobox-expiration'], prefs.EXPIRE_AFTER_X_DAYS, (1, 3, 6, 10, 30, -1))
         AttachBooleanRadio (dialog, widgetTree['radiobutton-playback-one'], widgetTree['radiobutton-playback-all'], prefs.SINGLE_VIDEO_PLAYBACK_MODE)
         AttachBoolean (dialog, widgetTree['checkbutton-resumemode'], prefs.RESUME_VIDEOS_MODE)
+        AttachBoolean (dialog, widgetTree['checkbutton-warnonquit'], prefs.WARN_IF_DOWNLOADING_ON_QUIT)
+        AttachBoolean (dialog, widgetTree['checkbutton-bt-autoforward'], prefs.USE_UPNP)
+        AttachBoolean (dialog, widgetTree['checkbutton-bt-enc-req'], prefs.BT_ENC_REQ)
 
         treeview = widgetTree['treeview-collection-dirs']
         listmodel = gtk.TreeStore(gobject.TYPE_INT, gobject.TYPE_STRING, gobject.TYPE_BOOLEAN)
@@ -567,6 +598,7 @@ class CallbackHandler(object):
             pass
         chooser = widgetTree['filechooserbutton-movies-directory']
         chooser.set_filename (movie_dir + "/")
+        chooser.set_current_folder (movie_dir)
         # run the dialog
         response = dialog.run()
         for descriptor, widget in dialog.integerWidgets:
@@ -607,14 +639,17 @@ class CallbackHandler(object):
     def on_about_clicked(self, event = None):
         self.mainFrame.about()
 
+    def on_help_clicked(self, event=None):
+        app.delegate.openExternalURL(config.get(prefs.HELP_URL))
+
     def on_donate_clicked(self, event = None):
-        print "Donate unimplemented"
+        app.delegate.openExternalURL(config.get(prefs.DONATE_URL))
 
     def on_delete(self, event = None):
         eventloop.addUrgentCall(app.controller.removeCurrentSelection, 
                 "remove current selection")
 
-    def on_button_chrome_search_go_clicked (self, event=None):
+    def on_search_activate (self, event=None):
         widgetTree = self.mainFrame.widgetTree
         term = widgetTree["entry-chrome-search-term"].get_text()
         iter = widgetTree["combobox-chrome-search-engine"].get_active_iter()
