@@ -16,6 +16,41 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+# 
+# In addition, as a special exception, the copyright holders give
+# permission to link the code of portions of this program with the OpenSSL
+# library.
+#
+# You must obey the GNU General Public License in all respects for all of
+# the code used other than OpenSSL. If you modify file(s) with this
+# exception, you may extend this exception to your version of the file(s),
+# but you are not obligated to do so. If you do not wish to do so, delete
+# this exception statement from your version. If you delete this exception
+# statement from all source files in the program, then also delete it here.
+
+# This needs to be above Paths and configuration :(
+#
+# This isn't being used right now, as it doesn't detect all the cases
+# in which we need the xine hack
+def use_xine_hack_default():
+    try:
+        # Non-debian based system will throw an exception here
+        f = open('/etc/debian_version')
+        if f.read().strip() == '4.0': # Debian etch
+            f.close()
+            return True
+        else:
+            # Ubuntu Feisty is Debian based but lists testing/unstable
+            # in /etc/debian_version, so we check /etc/issue
+            f.close()
+            f = open('/etc/issue')
+            osname = f.read()
+            if (osname.find("Ubuntu") > -1) and (osname.find("7.04")>-1):
+                f.close()
+                return True
+    except:
+        pass
+    return False
 
 ##############################################################################
 ## Paths and configuration                                                   ##
@@ -23,6 +58,8 @@
 
 BOOST_LIB = 'boost_python'
 
+USE_XINE_HACK = True #use_xine_hack_default()
+
 ###############################################################################
 ## End of configuration. No user-servicable parts inside                     ##
 ###############################################################################
@@ -30,6 +67,7 @@ BOOST_LIB = 'boost_python'
 from distutils.cmd import Command
 from distutils.core import setup
 from distutils.extension import Extension
+from distutils.errors import DistutilsOptionError
 from distutils import dir_util
 from distutils import log
 from distutils.util import change_root
@@ -39,9 +77,12 @@ import distutils.command.build_py
 import distutils.command.build_py
 import distutils.command.install_data
 import os
+import pwd
 import subprocess
 import sys
 import re
+import time
+import shutil
 
 from Pyrex.Distutils import build_ext
 
@@ -61,13 +102,13 @@ while True:
         raise RuntimeError("Couldn't find Miro root directory")
     root_try = os.path.abspath(os.path.join(root_try, '..'))
 portable_dir = os.path.join(root_dir, 'portable')
-bittorrent_dir = os.path.join(portable_dir, 'BitTorrent')
 dl_daemon_dir = os.path.join(portable_dir, 'dl_daemon')
 test_dir = os.path.join(portable_dir, 'test')
 compiled_templates_dir = os.path.join(portable_dir, 'compiled_templates')
 compiled_templates_unittest_dir = os.path.join(compiled_templates_dir,'unittest')
 resource_dir = os.path.join(root_dir, 'resources')
 platform_dir = os.path.join(root_dir, 'platform', 'gtk-x11')
+glade_dir = os.path.join(platform_dir, 'glade')
 xine_dir = os.path.join(platform_dir, 'xine')
 frontend_implementation_dir = os.path.join(platform_dir,
         'frontend_implementation')
@@ -76,6 +117,7 @@ debian_package_dir = os.path.join(platform_dir, 'debian_package')
 sys.path[0:0] = ['%s/platform/%s' % (root_dir, 'gtk-x11'), '%s/platform' % root_dir, '%s/portable' % root_dir]
 
 import template_compiler
+import setup_portable
 template_compiler.compileAllTemplates(root_dir)
 
 # little hack get the version from the current app.config.template
@@ -88,6 +130,17 @@ if 'bdist_rpm' in sys.argv:
     appVersion = appVersion.replace('-', '_')
 
 #### utility functions ####
+def getlogin():
+    try:
+        return os.environ['LOGNAME']
+    except KeyError:
+        pass
+    try:
+        return os.environ['USER']
+    except KeyError:
+        pass
+    pwd.getpwuid(os.getuid())[0] 
+
 def read_file(path):
     f = open(path)
     try:
@@ -170,6 +223,9 @@ fasttypes_ext = \
         libraries = [BOOST_LIB],
     )
 
+##### The libtorrent extension ####
+libtorrent_ext = setup_portable.libtorrent_extension(portable_dir)
+
 #### MozillaBrowser Extension ####
 try:
     packages = getCommandOutput("pkg-config --list-all")
@@ -187,17 +243,19 @@ elif re.search("^firefox-xpcom", packages, re.MULTILINE):
 else:
     sys.exit("Can't find xulrunner-xpcom, mozilla-xpcom or firefox-xpcom")
 
-# build a miro script that wraps the miro.real script with an LD_LIBRARY_PATH
-# environment variable to pick up the xpcom we decided to use.
-try:
-    fflib = getCommandOutput("pkg-config --variable=libdir %s" % xpcom)
-    # pluck off the last \n
-    fflib = fflib[:-1]
-    f = open(os.path.join(platform_dir, "miro"), "w")
-    f.write("#!/bin/sh\nLD_LIBRARY_PATH=%s miro.real" % fflib)
-    f.close()
-except RuntimeError, error:
-    sys.exit("Package config error:\n%s" % (error,))
+# do this so that it doesn't execute when doing python setup.py clean
+if not "clean" in sys.argv:
+    # build a miro script that wraps the miro.real script with an LD_LIBRARY_PATH
+    # environment variable to pick up the xpcom we decided to use.
+    try:
+        fflib = getCommandOutput("pkg-config --variable=libdir %s" % xpcom)
+        # pluck off the last \n
+        fflib = fflib[:-1]
+        f = open(os.path.join(platform_dir, "miro"), "w")
+        f.write('#!/bin/sh\necho %s\nLD_LIBRARY_PATH=%s ADDON_PATH=/tmp/empty miro.real "$@"\n' % (fflib, fflib))
+        f.close()
+    except RuntimeError, error:
+        sys.exit("Package config error:\n%s" % (error,))
 
 mozilla_browser_options = parsePkgConfig("pkg-config" , 
         "gtk+-2.0 glib-2.0 pygtk-2.0 %s %s" % (gtkmozembed, xpcom))
@@ -220,7 +278,7 @@ for dir in xpcom_includes['include_dirs']:
 if mozIncludeBase is None:
     raise ValueError("Can't find mozilla include base directory")
 for subdir in ['dom', 'gfx', 'widget', 'commandhandler', 'uriloader',
-            'webbrwsr', 'necko']:
+            'webbrwsr', 'necko', 'windowwatcher']:
     path = os.path.join(mozIncludeBase, subdir)
     mozilla_browser_options['include_dirs'].append(path)
 
@@ -241,6 +299,7 @@ mozilla_browser_ext = Extension("miro.MozillaBrowser",
         [ os.path.join(frontend_implementation_dir,'MozillaBrowser.pyx'),
           os.path.join(frontend_implementation_dir,'MozillaBrowserXPCOM.cc'),
           os.path.join(frontend_implementation_dir,'HttpObserver.cc'),
+          os.path.join(frontend_implementation_dir,'PromptService.cc'),
           os.path.join(frontend_implementation_dir,'DragAndDrop.cc'),
           os.path.join(frontend_implementation_dir,'XPCOMUtil.cc'),
         ],
@@ -257,10 +316,22 @@ xlib_ext = \
 #### Xine Extension ####
 xine_options = parsePkgConfig('pkg-config', 
         'libxine pygtk-2.0 gtk+-2.0 glib-2.0 gthread-2.0')
+
+# If you get XINE crashes, uncommenting this might fix it. It's
+# necessary on Debian Etch and Ubuntu Feisty right now.
+#
+# We have a horrible workaround for buggy X drivers in xine_impl.c
+# controled by this variable
+if USE_XINE_HACK:
+    print "Using the Xine driver hack. If you experience trouble playing video,\n   try setting USE_XINE_HACK to False in setup.py."
+    if xine_options.has_key('define_macros'):
+        xine_options['define_macros'].append(('INCLUDE_XINE_DRIVER_HACK', '1'))
+    else:
+        xine_options['define_macros'] = [('INCLUDE_XINE_DRIVER_HACK', '1')]
 xine_ext = Extension('miro.xine', [
-        os.path.join(xine_dir, 'xine.pyx'),
-        os.path.join(xine_dir, 'xine_impl.c'),
-        ], **xine_options)
+                         os.path.join(xine_dir, 'xine.pyx'),
+                         os.path.join(xine_dir, 'xine_impl.c'),],
+                     **xine_options)
 
 #### Build the data_files list ####
 def listfiles(path):
@@ -270,14 +341,23 @@ data_files = []
 # filter out app.config.template (which is handled specially)
 files = [f for f in listfiles(resource_dir) \
         if os.path.basename(f) != 'app.config.template']
-files.append(os.path.join(platform_dir, 'glade', 'miro.glade'))
+files.extend(listfiles(os.path.join(glade_dir, 'pixmaps')))
+files.append(os.path.join(glade_dir, 'miro.glade'))
 data_files.append(('/usr/share/miro/resources/', files))
 # handle the sub directories.
-for dir in ('templates', 'css', 'images', 'testdata', os.path.join('templates','unittest'), 'searchengines'):
+for dir in ('templates', 'css', 'images', 'html', 'testdata', os.path.join('templates','unittest'), 'searchengines'):
     source_dir = os.path.join(resource_dir, dir)
     dest_dir = os.path.join('/usr/share/miro/resources/', dir)
     data_files.append((dest_dir, listfiles(source_dir)))
 # add the desktop file, icons, mime data, and man page.
+
+# do this so that it doesn't execute when doing python setup.py clean
+if not "clean" in sys.argv:
+    rv = os.system ("gcc %s -o %s `pkg-config --libs --cflags gdk-pixbuf-2.0 glib-2.0 libxine`" % (os.path.join(platform_dir, "xine/xine_extractor.c"), os.path.join(platform_dir, "xine/xine_extractor")))
+
+    if rv != 0:
+        raise RuntimeError("xine_extractor compilation failed.  Possibly missing libxine, gdk-pixbuf-2.0, or glib-2.0.")
+
 data_files += [
     ('/usr/share/pixmaps', 
      glob(os.path.join(platform_dir, 'miro-*.png'))),
@@ -287,6 +367,8 @@ data_files += [
      [os.path.join(platform_dir, 'miro.xml')]),
     ('/usr/share/man/man1',
      [os.path.join(platform_dir, 'miro.1.gz')]),
+    ('/usr/libexec/',
+     [os.path.join(platform_dir, 'xine/xine_extractor')]),
 ]
 
 os.system ("gzip -9 < " + os.path.join(platform_dir, 'miro.1') + " > " + os.path.join(platform_dir, 'miro.1.gz'))
@@ -312,11 +394,16 @@ class install_data (distutils.command.install_data.install_data):
         if self.root:
             dest = change_root(self.root, dest)
         self.mkpath(os.path.dirname(dest))
-        self.copy_file(source, dest)
+        # We don't use the dist utils copy_file() because it only copies
+        # the file if the timestamp is newer
+        shutil.copyfile(source,dest)
         expand_file_contents(dest, APP_REVISION=revision,
                              APP_REVISION_NUM=revisionnum,
                              APP_REVISION_URL=revisionurl,
                              APP_PLATFORM='gtk-x11',
+                             BUILD_MACHINE="%s@%s" % (getlogin(),
+                                                      os.uname()[1]),
+                             BUILD_TIME=str(time.time()),
                              MOZILLA_LIB_PATH=mozilla_lib_path[0])
         self.outfiles.append(dest)
 
@@ -397,7 +484,7 @@ class bdist_deb (Command):
         self.dist_dir = self.get_finalized_command('bdist').dist_dir
 
     def run (self):
-        # build all python modules/extensions
+        # buzild all python modules/extensions
         self.run_command('build')
         # copy the built files
         install = self.reinitialize_command('install', reinit_subcommands=1)
@@ -447,7 +534,62 @@ class bdist_deb (Command):
         os.system(dpkg_command)
         dir_util.remove_tree(self.bdist_dir)
 
-print "IS_FILE: %s" % repr(os.path.isfile(os.path.join(platform_dir, "miro.real")))
+#### install_theme installs a specifified theme .zip
+class install_theme(Command):
+
+    description = 'Install a provided theme to /usr/share/miro/themes'
+    user_options = [("theme=", None, 'ZIP file containing the theme')]
+
+    def initialize_options(self):
+        self.theme = None
+
+    def finalize_options(self):
+        if self.theme is None:
+            raise DistutilsOptionError, "must supply a theme ZIP file"
+        if not os.path.exists(self.theme):
+            raise DistutilsOptionError, "theme file does not exist"
+        import zipfile
+        if not zipfile.is_zipfile(self.theme):
+            raise DistutilsOptionError, "theme file is not a ZIP file"
+        zf = zipfile.ZipFile(self.theme)
+        appConfig = zf.read('app.config')
+        themeName = None
+        for line in appConfig.split('\n'):
+            if '=' in line:
+                name, value = line.split('=', 1)
+                name = name.strip()
+                value = value.lstrip()
+                if name == 'themeName':
+                    themeName = value
+        if themeName is None:
+            raise DistutilsOptionError, "invalid theme file"
+        self.zipfile = zf
+        self.theme_name = themeName
+        self.theme_dir = '/usr/share/miro/themes/%s' % themeName
+
+    def run(self):
+        if os.path.exists(self.theme_dir):
+            shutil.rmtree(self.theme_dir)
+        os.makedirs(self.theme_dir)
+        for name in self.zipfile.namelist():
+            if name.startswith('xul/'):
+                # ignore XUL stuff, we don't need it on linux
+                continue
+            print 'installing', os.path.join(self.theme_dir, name)
+            if name[-1] == '/':
+                os.makedirs(os.path.join(self.theme_dir, name))
+            else:
+                f = file(os.path.join(self.theme_dir, name), 'wb')
+                f.write(self.zipfile.read(name))
+                f.close()
+        print """%s theme installed.
+
+To use this theme, run:
+
+    miro --theme="%s"
+""" % (self.theme_name, self.theme_name)
+
+#print "IS_FILE: %s" % repr(os.path.isfile(os.path.join(platform_dir, "miro.real")))
 
 #### Run setup ####
 setup(name='miro', 
@@ -462,7 +604,7 @@ setup(name='miro',
     ],
     data_files=data_files,
     ext_modules = [
-        fasttypes_ext, mozilla_browser_ext, xine_ext, xlib_ext,
+        fasttypes_ext, mozilla_browser_ext, xine_ext, xlib_ext, libtorrent_ext,
         Extension("miro.database", 
                 [os.path.join(portable_dir, 'database.pyx')]),
         Extension("miro.sorts", 
@@ -472,7 +614,6 @@ setup(name='miro',
     ],
     packages = [
         'miro.frontend_implementation',
-        'miro.BitTorrent',
         'miro.dl_daemon',
         'miro.test',
         'miro.compiled_templates',
@@ -481,7 +622,6 @@ setup(name='miro',
     ],
     package_dir = {
         'miro.frontend_implementation' : frontend_implementation_dir,
-        'miro.BitTorrent' : bittorrent_dir,
         'miro.dl_daemon' : dl_daemon_dir,
         'miro.test' : test_dir,
         'miro.compiled_templates' : compiled_templates_dir,
@@ -492,5 +632,7 @@ setup(name='miro',
         'build_py': build_py,
         'bdist_deb': bdist_deb,
         'install_data': install_data,
+        'install_theme': install_theme,
     }
 )
+
