@@ -1,3 +1,32 @@
+# Miro - an RSS based video player application
+# Copyright (C) 2005-2007 Participatory Culture Foundation
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+#
+# In addition, as a special exception, the copyright holders give
+# permission to link the code of portions of this program with the OpenSSL
+# library.
+#
+# You must obey the GNU General Public License in all respects for all of
+# the code used other than OpenSSL. If you modify file(s) with this
+# exception, you may extend this exception to your version of the file(s),
+# but you are not obligated to do so. If you do not wish to do so, delete
+# this exception statement from your version. If you delete this exception
+# statement from all source files in the program, then also delete it here.
+
+import os
 import re
 import urllib
 import logging
@@ -6,6 +35,7 @@ from objc import YES, NO, nil
 from AppKit import *
 from WebKit import *
 from Foundation import *
+from PyObjCTools import AppHelper
 
 import app
 import prefs
@@ -57,14 +87,17 @@ class HTMLDisplay (app.Display):
         try:
             return self._web
         except AttributeError:
-            self._web = ManagedWebView.alloc().init(self.html, None, self.nowReadyToDisplay, lambda x:self.onURLLoad(unicode(x)), self.displaySizeHint, self.baseURL)
+            self._web = ManagedWebView.alloc().initWithInitialHTML_existinView_loadFinished_loadURL_sizeHint_baseURL_(
+                            self.html, None, self.nowReadyToDisplay, lambda x:self.onURLLoad(unicode(x)), self.displaySizeHint, self.baseURL)
             return self._web
     web = property(get_web)
 
     def getEventCookie(self):
         return ''
+
     def getDTVPlatformName(self):
         return 'webkit'
+
     def getBodyTagExtra(self):
         return 'ondragstart="handleDragStart(event);" ondragover="return handleDragOver(event);" ondragleave="return handleDragLeave(event);" ondrop="return handleDrop(event);" '
 
@@ -80,6 +113,9 @@ class HTMLDisplay (app.Display):
             print "Couldn't exec javascript! Web view not initialized"
         #print "DISP: %s with %s" % (self.view, js)
 
+    def navigateToFragment(self, fragment):
+        self.web.navigateToFragment(fragment)
+
     # DOM hooks used by the dynamic template code -- do they need a 
     # try..except wrapper like the above?
     def addItemAtEnd(self, xml, id):
@@ -148,7 +184,6 @@ class HTMLDisplay (app.Display):
         self.web.getView().stopLoading_(nil)
         self.readyToDisplay = False
         self.readyToDisplayHook = None
-                        
 
 ###############################################################################
 
@@ -174,11 +209,14 @@ class ManagedWebView (NSObject):
 
     WebView.registerViewClass_representationClass_forMIMEType_(ManagedWebHTMLView, WebHTMLRepresentation, u'text/html')
 
-    def init(self, initialHTML, existingView=nil, onInitialLoadFinished=None, onLoadURL=None, sizeHint=None, baseURL=None):
+    def initWithInitialHTML_existinView_loadFinished_loadURL_sizeHint_baseURL_(
+        self, initialHTML, existingView=nil, onInitialLoadFinished=None, onLoadURL=None, sizeHint=None, baseURL=None):
         self.onInitialLoadFinished = onInitialLoadFinished
         self.onLoadURL = onLoadURL
         self.initialLoadFinished = False
         self.view = existingView
+        self.openPanelContextID = 1
+        self.openPanelContext = dict()
         platformutils.callOnMainThreadAndWaitUntilDone(self.initWebView, initialHTML, sizeHint, baseURL)        
         return self
 
@@ -218,14 +256,10 @@ class ManagedWebView (NSObject):
     def isKeyExcludedFromWebScript_(self,key):
         return YES
 
-    def isSelectorExcludedFromWebScript_(self,sel):
-        if (str(sel) == 'eventURL'):
-            return NO
-        else:
-            return YES
+    def isSelectorExcludedFromWebScript_(self, sel):
+        return (str(sel) != 'eventURL')
 
     def eventURL(self,url):
-        url = urllib.unquote(str(url)).decode('utf8')
         self.onLoadURL(url)
 
     def webView_contextMenuItemsForElement_defaultMenuItems_(self,webView,contextMenu,defaultMenuItems):
@@ -254,18 +288,40 @@ class ManagedWebView (NSObject):
             scriptObj = self.view.windowScriptObject()
             scriptObj.setValue_forKey_(self, u'frontend')
 
+    def webView_didFailProvisionalLoadWithError_forFrame_(self, webview, error, frame):
+        urlError = (error.domain() == NSURLErrorDomain)
+        certError = (error.code() in (NSURLErrorServerCertificateHasBadDate, 
+                                      NSURLErrorServerCertificateHasUnknownRoot, 
+                                      NSURLErrorServerCertificateUntrusted))
+
+        if urlError and certError:
+            request = frame.provisionalDataSource().request()
+            if request is nil:
+                request = frame.dataSource().request()
+            url = request.URL()            
+            allowed = [config.get(prefs.CHANNEL_GUIDE_URL), 
+                       config.get(prefs.CHANNEL_GUIDE_FIRST_TIME_URL)]
+            if url.absoluteString() in allowed:
+                # The [NSURLRequest setAllowsAnyHTTPSCertificate:forHost:] selector is
+                # not documented anywhere, so I assume it is not public. It is however 
+                # a very clean and easy way to allow us to load our channel guide from
+                # https, so let's use it here anyway :)
+                NSURLRequest.setAllowsAnyHTTPSCertificate_forHost_(YES, url.host())
+                # Now reload
+                frame.loadRequest_(request)
+
     # Intercept navigation actions and give program a chance to respond
     def webView_decidePolicyForNavigationAction_request_frame_decisionListener_(self, webview, action, request, frame, listener):
         platformutils.warnIfNotOnMainThread('ManagedWebView.webView_decidePolicyForNavigationAction_request_frame_decisionListener_')
         method = request.HTTPMethod()
         url = unicode(request.URL())
         body = request.HTTPBody()
-        type = action['WebActionNavigationTypeKey']
-        #print "policy %d for url %s" % (type, url)
+        ntype = action['WebActionNavigationTypeKey']
+        #print "policy %d for url %s" % (ntype, url)
         # setting document.location.href in Javascript (our preferred
         # method of triggering an action) comes out as an
         # WebNavigationTypeOther.
-        if type == WebNavigationTypeLinkClicked or type == WebNavigationTypeFormSubmitted or type == WebNavigationTypeOther:
+        if ntype in (WebNavigationTypeLinkClicked, WebNavigationTypeFormSubmitted, WebNavigationTypeOther):
             # Make sure we have a real, bona fide Python string, not an
             # NSString. Unfortunately, == can tell the difference.
             if (not self.onLoadURL) or self.onLoadURL(url):
@@ -275,6 +331,12 @@ class ManagedWebView (NSObject):
         else:
             listener.use()
 
+    # Intercept external links requests
+    def webView_decidePolicyForNewWindowAction_request_newFrameName_decisionListener_(self, webView, info, request, name, listener):
+        url = info["WebActionOriginalURLKey"]
+        NSWorkspace.sharedWorkspace().openURL_(url)
+        listener.ignore()
+
     # Redirect resource: links to files in resource bundle
     def webView_resource_willSendRequest_redirectResponse_fromDataSource_(self, webview, resourceCookie, request, redirectResponse, dataSource):
         platformutils.warnIfNotOnMainThread('ManagedWebView.webView_resource_willSendRequest_redirectResponse_fromDataSource_')
@@ -287,10 +349,36 @@ class ManagedWebView (NSObject):
             return NSURLRequest.requestWithURL_cachePolicy_timeoutInterval_(urlObject, NSURLRequestReloadIgnoringCacheData, 60)
 
         if isinstance(request, NSMutableURLRequest):
+            language = os.environ['LANGUAGE'].split(':')[0].replace('_', '-')
+            request.setValue_forHTTPHeaderField_(language, u'Accept-Language')
             request.setValue_forHTTPHeaderField_(u'1', u'X-Miro')
         
         return request
 
+    def webView_runOpenPanelForFileButtonWithResultListener_(self, webview, listener):
+        self.openPanelContextID += 1
+        self.openPanelContext[self.openPanelContextID] = listener
+        panel = NSOpenPanel.openPanel()
+        panel.beginSheetForDirectory_file_types_modalForWindow_modalDelegate_didEndSelector_contextInfo_(
+            NSHomeDirectory(),
+            nil,
+            nil,
+            self.view.window(),
+            self,
+            'openPanelDidEnd:returnCode:contextInfo:',
+            self.openPanelContextID)
+
+    @AppHelper.endSheetMethod
+    def openPanelDidEnd_returnCode_contextInfo_(self, panel, result, contextID):
+        listener = self.openPanelContext[contextID]
+        del self.openPanelContext[contextID]
+        if result == NSOKButton:
+            filenames = panel.filenames()
+            listener.chooseFilename_(filenames[0])
+
+    def webView_runJavaScriptAlertPanelWithMessage_(self, webview, message):
+        logging.jsalert(message)
+
     # Return the actual WebView that we're managing
     def getView(self):
         return self.view
@@ -320,6 +408,10 @@ class ManagedWebView (NSObject):
     def execJS(self, js):
         self.view.stringByEvaluatingJavaScriptFromString_(js)
 
+    def navigateToFragment(self, fragment):
+        command = "var tab = document.getElementById(\"%s\"); tab.scrollIntoView(true);" % fragment
+        self.execJS(command)
+
     ## DOM mutators called, ultimately, by dynamic template system ##
 
     def findElt(self, id):
