@@ -1,10 +1,38 @@
+# Miro - an RSS based video player application
+# Copyright (C) 2005-2007 Participatory Culture Foundation
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+#
+# In addition, as a special exception, the copyright holders give
+# permission to link the code of portions of this program with the OpenSSL
+# library.
+#
+# You must obey the GNU General Public License in all respects for all of
+# the code used other than OpenSSL. If you modify file(s) with this
+# exception, you may extend this exception to your version of the file(s),
+# but you are not obligated to do so. If you do not wish to do so, delete
+# this exception statement from your version. If you delete this exception
+# statement from all source files in the program, then also delete it here.
+
+import os
 import math
 import logging
 
-from objc import YES, NO, nil
+from objc import YES, NO, nil, IBOutlet
 from AppKit import *
 from Foundation import *
-from PyObjCTools import NibClassBuilder
 
 import app
 import feed
@@ -22,8 +50,6 @@ import platformutils
 
 from gtcache import gettext as _
 
-NibClassBuilder.extractClasses(u"MainWindow")
-
 ###############################################################################
 
 class MainFrame:
@@ -35,7 +61,7 @@ class MainFrame:
         # Do this in two steps so that self.controller is set when self.controler.init
         # is called. That way, init can turn around and call selectDisplay.
         self.controller = MainController.alloc()
-        self.controller.init(self, appl)
+        self.controller.initWithFrame_application_(self, appl)
 
     def selectDisplay(self, display, area=None):
         """Install the provided 'display' in the requested area"""
@@ -56,9 +82,15 @@ class MainFrame:
 
 ###############################################################################
 
-class MainController (NibClassBuilder.AutoBaseClass):
+class MainController (NSWindowController):
 
-    def init(self, frame, appl):
+    channelsHostView        = IBOutlet('channelsHostView')
+    mainHostView            = IBOutlet('mainHostView')
+    splitView               = IBOutlet('splitView')
+    videoDisplayController  = IBOutlet('videoDisplayController')
+    videoInfoHostView       = IBOutlet('videoInfoHostView')
+
+    def initWithFrame_application_(self, frame, appl):
         super(MainController, self).init()
         self.frame = frame
         self.appl = appl
@@ -76,41 +108,43 @@ class MainController (NibClassBuilder.AutoBaseClass):
         return self
 
     def awakeFromNib(self):
-        from VideoDisplay import VideoDisplayController
-        self.videoDisplayController = VideoDisplayController.getInstance()
         self.frame.channelsDisplay = self.channelsHostView
         self.frame.mainDisplay = self.mainHostView
         self.frame.videoInfoDisplay = self.videoInfoHostView
         self.frame.videoInfoDisplay.backgroundColor = NSColor.blackColor()
         self.restoreLayout()
         self.updateWindowTexture()
+        self.window().setTitle_(config.get(prefs.LONG_APP_NAME))
         self.showWindow_(nil)
 
     def appWillTerminate_(self, notification):
         self.saveLayout()
 
     def restoreLayout(self):
-        windowFrame = config.get(prefs.MAIN_WINDOW_FRAME)
-        if windowFrame is None:
-            windowFrame = self.window().frame()
+        if app.delegate.maximizeMainFrameWhenAvailable:
+            app.delegate.doMaximizeWindow(self.window())
         else:
-            windowFrame = NSRectFromString(windowFrame)
-        screen = self.window().screen()
-        if screen is not None:
-            visibleFrame = screen.visibleFrame()
-            if not NSContainsRect(visibleFrame, windowFrame):
-                logging.debug("Fitting window to screen size")
-                windowFrame = visibleFrame
-        self.window().setFrame_display_(windowFrame, NO)
+            windowFrame = config.get(prefs.MAIN_WINDOW_FRAME)
+            if windowFrame is None:
+                windowFrame = self.window().frame()
+            else:
+                windowFrame = NSRectFromString(windowFrame)
+            screen = self.window().screen()
+            if screen is not None:
+                visibleFrame = screen.visibleFrame()
+                if not NSContainsRect(visibleFrame, windowFrame):
+                    logging.debug("Fitting window to screen size")
+                    windowFrame = visibleFrame
+            self.window().setFrame_display_(windowFrame, NO)
 
-        leftFrame = config.get(prefs.LEFT_VIEW_SIZE)
-        rightFrame = config.get(prefs.RIGHT_VIEW_SIZE)
-        if leftFrame is not None and rightFrame is not None:
-           leftFrame = NSRectFromString(leftFrame)
-           rightFrame = NSRectFromString(rightFrame)
-           self.splitView.subviews().objectAtIndex_(0).setFrame_(leftFrame)
-           self.splitView.subviews().objectAtIndex_(1).setFrame_(rightFrame)
-           self.splitView.adjustSubviews()
+            leftFrame = config.get(prefs.LEFT_VIEW_SIZE)
+            rightFrame = config.get(prefs.RIGHT_VIEW_SIZE)
+            if leftFrame is not None and rightFrame is not None:
+               leftFrame = NSRectFromString(leftFrame)
+               rightFrame = NSRectFromString(rightFrame)
+               self.splitView.subviews().objectAtIndex_(0).setFrame_(leftFrame)
+               self.splitView.subviews().objectAtIndex_(1).setFrame_(rightFrame)
+               self.splitView.adjustSubviews()
 
     def saveLayout(self):
         windowFrame = self.window().frame()
@@ -226,7 +260,10 @@ class MainController (NibClassBuilder.AutoBaseClass):
         contentBox.setFrameOrigin_((tabSize.width + dividerWidth, 0))
 
     def splitView_canCollapseSubview_(self, sender, subview):
-        return self.channelsHostView.isDescendantOf_(subview) and app.controller.videoDisplay.isSelected()
+        if hasattr(app.controller, 'videoDisplay'):
+            return self.channelsHostView.isDescendantOf_(subview) and app.controller.videoDisplay.isSelected()
+        else:
+            return NO
 
     ### Events ###
 
@@ -286,7 +323,7 @@ class MainController (NibClassBuilder.AutoBaseClass):
         eventloop.addIdle(app.controller.updateAllFeeds, "Update all channels")
 
     def tellAFriend_(self, sender):
-        print "NOT IMPLEMENTED" # $$$$$$$$$$$$$$
+        eventloop.addIdle(app.controller.recommendCurrentFeed, "Recommend current feed")
 
     def copyChannelURL_(self, sender):
         eventloop.addIdle(app.controller.copyCurrentFeedURL, "Copy channel URL")
@@ -386,7 +423,7 @@ class MainController (NibClassBuilder.AutoBaseClass):
             self.updateMenuItem(item, 'playlist_remove')
             return self.actionGroups['PlaylistLikeSelected'] or self.actionGroups['PlaylistLikesSelected']
         elif action == 'playPause:':
-            return display is app.controller.videoDisplay
+            return display is app.controller.videoDisplay or self.actionGroups['VideoPlayable']
         elif action == 'stopVideo:':
             return display is app.controller.videoDisplay
         elif action == 'nextVideo:':
@@ -408,7 +445,7 @@ class MainController (NibClassBuilder.AutoBaseClass):
 
 ###############################################################################
 
-class DisplayHostView (NibClassBuilder.AutoBaseClass):
+class DisplayHostView (NSView):
     
     def initWithFrame_(self, frame):
         self = super(DisplayHostView, self).initWithFrame_(frame)
@@ -484,7 +521,7 @@ class NullDisplay (app.Display):
 
 ###############################################################################
 
-class DTVSplitView (NibClassBuilder.AutoBaseClass):
+class DTVSplitView (NSSplitView):
     
     def awakeFromNib(self):
         self.background = NSImage.imageNamed_(u'splitview_divider_background')
@@ -504,7 +541,11 @@ class DTVSplitView (NibClassBuilder.AutoBaseClass):
 
 ###############################################################################
 
-class ProgressDisplayView (NibClassBuilder.AutoBaseClass):
+class ProgressDisplayView (NSView):
+
+    progressSlider          = IBOutlet('progressSlider')
+    timeIndicator           = IBOutlet('timeIndicator')
+    remainingTimeIndicator  = IBOutlet('remainingTimeIndicator')
 
     def awakeFromNib(self):
         self.progressSlider.sliderWasClicked = self.progressSliderWasClicked
@@ -600,7 +641,7 @@ class ProgressDisplayView (NibClassBuilder.AutoBaseClass):
 
 ###############################################################################
 
-class Slider (NibClassBuilder.AutoBaseClass):
+class Slider (NSView):
 
     def initWithFrame_(self, frame):
         self = super(Slider, self).initWithFrame_(frame)
@@ -612,6 +653,7 @@ class Slider (NibClassBuilder.AutoBaseClass):
         self.sliderWasReleased = None
         return self
 
+    @platformutils.onMainThread
     def setFloatValue_(self, value):
         self.value = value
         self.setNeedsDisplay_(YES)
@@ -670,7 +712,7 @@ class Slider (NibClassBuilder.AutoBaseClass):
 
 ###############################################################################
 
-class ProgressSlider (NibClassBuilder.AutoBaseClass):
+class ProgressSlider (Slider):
     
     def initWithFrame_(self, frame):
         self = super(ProgressSlider, self).initWithFrame_(frame)
@@ -701,7 +743,7 @@ class ProgressSlider (NibClassBuilder.AutoBaseClass):
 
 ###############################################################################
 
-class MetalSlider (NibClassBuilder.AutoBaseClass):
+class MetalSlider (NSSlider):
 
     def awakeFromNib(self):
         oldCell = self.cell()
@@ -740,7 +782,7 @@ class MetalSliderCell (NSSliderCell):
 
 ###############################################################################
 
-class VideoSearchField (NibClassBuilder.AutoBaseClass):
+class VideoSearchField (NSSearchField):
 
     def awakeFromNib(self):
         self.setCell_(VideoSearchFieldCell.alloc().initWithCell_(self.cell()))
@@ -751,7 +793,7 @@ class VideoSearchField (NibClassBuilder.AutoBaseClass):
     def search_(self, sender):
         engine = self.selectedEngine()
         query = unicode(self.stringValue())
-        if query is not '':
+        if query != '':
             eventloop.addIdle(lambda:app.controller.performSearch(engine, query), 'Performing chrome search')
 
     def initFromLastEngine(self):
@@ -759,8 +801,9 @@ class VideoSearchField (NibClassBuilder.AutoBaseClass):
         lastEngine = searchengines.getLastEngine()
         for engine in views.searchEngines:
             if engine.name == lastEngine:
-                index = self.searchMenuTemplate().indexOfItemWithRepresentedObject_(engine)
-                self.searchMenuTemplate().performActionForItemAtIndex_(index)
+                menu = self.searchMenuTemplate()
+                index = menu.indexOfItemWithRepresentedObject_(engine)
+                menu.performActionForItemAtIndex_(index)
                 return
 
     def selectedEngine(self):
@@ -818,6 +861,8 @@ class VideoSearchFieldCell (NSSearchFieldCell):
 
 def _getEngineIcon(engine):
     engineIconPath = resources.path('images/search_icon_%s.png' % engine.name)
+    if not os.path.exists(engineIconPath):
+        return nil
     return NSImage.alloc().initByReferencingFile_(engineIconPath)
 
 searchIcons = dict()
@@ -831,6 +876,8 @@ def _makeSearchIcon(engine):
     popupRectangleSize = popupRectangle.size()
 
     engineIconPath = resources.path('images/search_icon_%s.png' % engine.name)
+    if not os.path.exists(engineIconPath):
+        return nil
     engineIcon = NSImage.alloc().initByReferencingFile_(engineIconPath)
     engineIconSize = engineIcon.size()
 
