@@ -1,3 +1,31 @@
+# Miro - an RSS based video player application
+# Copyright (C) 2005-2007 Participatory Culture Foundation
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+#
+# In addition, as a special exception, the copyright holders give
+# permission to link the code of portions of this program with the OpenSSL
+# library.
+#
+# You must obey the GNU General Public License in all respects for all of
+# the code used other than OpenSSL. If you modify file(s) with this
+# exception, you may extend this exception to your version of the file(s),
+# but you are not obligated to do so. If you do not wish to do so, delete
+# this exception statement from your version. If you delete this exception
+# statement from all source files in the program, then also delete it here.
+
 import os
 import time
 import string
@@ -5,10 +33,10 @@ import signal
 import logging
 import threading
 
-from objc import YES, NO, nil, signature
+from objc import YES, NO, nil, signature, IBOutlet
 from AppKit import *
 from Foundation import *
-from PyObjCTools import NibClassBuilder, Conversion
+from PyObjCTools import Conversion, AppHelper
 
 import app
 import feed
@@ -25,11 +53,6 @@ import SparkleUpdater
 
 ###############################################################################
 
-NibClassBuilder.extractClasses(u"PasswordWindow")
-NibClassBuilder.extractClasses(u"TextEntryWindow")
-NibClassBuilder.extractClasses(u'SearchChannelWindow')
-NibClassBuilder.extractClasses(u"ExceptionReporterPanel")
-
 dlTask = None
 
 ###############################################################################
@@ -69,12 +92,37 @@ class UIBackendDelegate:
 
     def __init__(self):
         self.contextItemHandler = ContextItemHandler.alloc().init()
+        self.openPanelHandler = OpenPanelHandler.alloc().init()
+        self.savePanelHandler = SavePanelHandler.alloc().init()
+        self.maximizeMainFrameWhenAvailable = False
+
+    def maximizeWindow(self):
+        window = NSApplication.sharedApplication().mainWindow()
+        if window is not None:
+            self.doMaximizeWindow(window)
+        else:
+            self.maximizeMainFrameWhenAvailable = True
+
+    def doMaximizeWindow(self, window):
+        screen = window.screen()
+        fullFrame = screen.visibleFrame()
+        window.setFrame_display_(fullFrame, YES)
+        self.maximizeMainFrameWhenAvailable = False
 
     def performStartupTasks(self, terminationCallback):
         NSApplication.sharedApplication().delegate().checkQuicktimeVersion(True)
         startupController = StartupPanelController.alloc().init()
         startupController.run(terminationCallback)
 
+    @platformutils.onMainThread
+    def askForSavePathname(self, title, callback, defaultDirectory=None, defaultFilename=None):
+        self.savePanelHandler.run(callback, defaultFilename)
+
+    @platformutils.onMainThread
+    def askForOpenPathname(self, title, callback, defaultDirectory=None,
+            typeString=None, types=None):
+        self.openPanelHandler.run(callback, defaultDirectory, types)
+
     @platformutils.onMainThread
     def runDialog(self, dialog):
         if isinstance(dialog, dialogs.TextEntryDialog):
@@ -82,6 +130,16 @@ class UIBackendDelegate:
             dlog.run()
             call = lambda:dialog.runCallback(dlog.result, dlog.value)
             name = "TextEntryDialog"
+        elif isinstance(dialog, dialogs.CheckboxTextboxDialog):
+            dlog = CheckboxTextboxDialogController.alloc().initWithDialog_(dialog)
+            dlog.run()
+            call = lambda:dialog.runCallback(dlog.result, dlog.value, dlog.text)
+            name = "CheckboxTextboxDialog"
+        elif isinstance(dialog, dialogs.CheckboxDialog):
+            dlog = CheckboxDialogController.alloc().initWithDialog_(dialog)
+            dlog.run()
+            call = lambda:dialog.runCallback(dlog.result, dlog.value)
+            name = "CheckboxDialog"
         elif isinstance(dialog, dialogs.HTTPAuthDialog):
             self.httpAuthLock.acquire()
             try:
@@ -179,22 +237,23 @@ class UIBackendDelegate:
         # wait here indefinitely. We therefore manually poll for a specific 
         # amount of time beyond which we force quit the daemon.
         global dlTask
-        if dlTask is not None and dlTask.isRunning():
-            logging.info('Waiting for the downloader daemon to terminate...')
-            timeout = 5.0
-            sleepTime = 0.2
-            loopCount = int(timeout / sleepTime)
-            for i in range(loopCount):
-                if dlTask.isRunning():
-                    time.sleep(sleepTime)
+        if dlTask is not None:
+            if dlTask.isRunning():
+                logging.info('Waiting for the downloader daemon to terminate...')
+                timeout = 5.0
+                sleepTime = 0.2
+                loopCount = int(timeout / sleepTime)
+                for i in range(loopCount):
+                    if dlTask.isRunning():
+                        time.sleep(sleepTime)
+                    else:
+                        break
                 else:
-                    break
-            else:
-                # If the daemon is still alive at this point, it's likely to be
-                # in a bad state, so nuke it.
-                logging.info("Timeout expired - Killing downloader daemon!")
-                dlTask.terminate()
-        dlTask.waitUntilExit()
+                    # If the daemon is still alive at this point, it's likely to be
+                    # in a bad state, so nuke it.
+                    logging.info("Timeout expired - Killing downloader daemon!")
+                    dlTask.terminate()
+            dlTask.waitUntilExit()
         dlTask = None
 
     def waitUntilDownloadDaemonExit(self):
@@ -203,24 +262,22 @@ class UIBackendDelegate:
             dlTask.waitUntilExit()
             dlTask = None      
 
-    def killDownloadDaemon(self, oldpid=None):
-        if oldpid is not None:
+    def killProcess(self, pid=None):
+        if pid is not None:
             try:
-                os.kill(oldpid, signal.SIGTERM)
+                os.kill(pid, signal.SIGTERM)
                 sleep(1)
-                os.kill(oldpid, signal.SIGKILL)
+                os.kill(pid, signal.SIGKILL)
             except:
                 pass
 
     def launchDownloadDaemon(self, oldpid, env):
-        self.killDownloadDaemon(oldpid)
+        self.killProcess(oldpid)
 
         env['DEMOCRACY_DOWNLOADER_LOG'] = config.get(prefs.DOWNLOADER_LOG_PATHNAME)
         env.update(os.environ)
                 
-        bundle = NSBundle.mainBundle()
-        bundleExe = bundle.executablePath()
-        exe = "%s/Downloader" % os.path.dirname(bundleExe)
+        exe = NSBundle.mainBundle().executablePath()
         
         global dlTask
         dlTask = NSTask.alloc().init()
@@ -308,7 +365,55 @@ class ContextItemHandler (NSObject):
 
 ###############################################################################
 
-class ExceptionReporterController (NibClassBuilder.AutoBaseClass):
+class OpenPanelHandler (NSObject):
+
+    def run(self, callback, defaultDirectory, types):
+        self.callback = callback
+        if defaultDirectory is None:
+            defaultDirectory = NSHomeDirectory()
+        panel = NSOpenPanel.openPanel()
+        panel.beginSheetForDirectory_file_types_modalForWindow_modalDelegate_didEndSelector_contextInfo_(
+            defaultDirectory,
+            nil,
+            types,
+            NSApplication.sharedApplication().mainWindow(),
+            self,
+            'openPanelDidEnd:returnCode:contextInfo:',
+            0)
+    
+    @AppHelper.endSheetMethod
+    def openPanelDidEnd_returnCode_contextInfo_(self, panel, result, contextID):
+        if result == NSOKButton:
+            filenames = panel.filenames()
+            self.callback(filenames[0])
+
+###############################################################################
+
+class SavePanelHandler (NSObject):
+    
+    def run(self, callback, defaultFilename):
+        self.callback = callback
+        panel = NSSavePanel.savePanel()
+        panel.beginSheetForDirectory_file_modalForWindow_modalDelegate_didEndSelector_contextInfo_(
+            NSHomeDirectory(),
+            defaultFilename,
+            NSApplication.sharedApplication().mainWindow(),
+            self,
+            'savePanelDidEnd:returnCode:contextInfo:',
+            0)
+    
+    @AppHelper.endSheetMethod
+    def savePanelDidEnd_returnCode_contextInfo_(self, panel, result, contextID):
+        if result == NSOKButton:
+            self.callback(panel.filename())
+
+###############################################################################
+
+class ExceptionReporterController (NSWindowController):
+    
+    logView     = IBOutlet('logView')
+    msg1Field   = IBOutlet('msg1Field')
+    msg3View    = IBOutlet('msg3View')
     
     def initWithMoment_log_(self, when, log):
         self = super(ExceptionReporterController, self).initWithWindowNibName_owner_(u"ExceptionReporterPanel", self)
@@ -343,7 +448,12 @@ class ExceptionReporterController (NibClassBuilder.AutoBaseClass):
 
 ###############################################################################
 
-class PasswordController (NibClassBuilder.AutoBaseClass):
+class PasswordController (NSObject):
+
+    passwordField   = IBOutlet('passwordField')
+    textArea        = IBOutlet('textArea')
+    usernameField   = IBOutlet('usernameField')
+    window          = IBOutlet('window')
 
     def initWithDialog_(self, dialog):
         NSBundle.loadNibNamed_owner_(u"PasswordWindow", self)
@@ -377,7 +487,12 @@ class PasswordController (NibClassBuilder.AutoBaseClass):
 
 ###############################################################################
 
-class TextEntryController (NibClassBuilder.AutoBaseClass):
+class TextEntryController (NSWindowController):
+    
+    entryField      = IBOutlet('entryField')
+    messageField    = IBOutlet('messageField')
+    mainButton      = IBOutlet('mainButton')
+    secondaryButton = IBOutlet('secondaryButton')
     
     def initWithDialog_(self, dialog):
         self = super(TextEntryController, self).initWithWindowNibName_owner_(u"TextEntryWindow", self)
@@ -413,7 +528,105 @@ class TextEntryController (NibClassBuilder.AutoBaseClass):
 
 ###############################################################################
 
-class SearchChannelController (NibClassBuilder.AutoBaseClass):
+class CheckboxDialogController (NSWindowController):
+    
+    checkbox        = IBOutlet('checkbox')
+    mainButton      = IBOutlet('mainButton')
+    messageField    = IBOutlet('messageField')
+    secondaryButton = IBOutlet('secondaryButton')
+    
+    def initWithDialog_(self, dialog):
+        self = super(CheckboxDialogController, self).initWithWindowNibName_owner_(u"CheckboxDialogWindow", self)
+        self.dialog = dialog
+        self.window().setTitle_(unicode(dialog.title))
+        self.messageField.setStringValue_(dialog.description)
+        self.checkbox.setTitle_(unicode(dialog.checkbox_text))
+        if dialog.checkbox_value:
+            self.checkbox.setState_(NSOnState)
+        else:
+            self.checkbox.setState_(NSOffState)
+        self.mainButton.setTitle_(unicode(dialog.buttons[0].text))
+        self.secondaryButton.setTitle_(dialog.buttons[1].text)
+        self.result = None
+        self.value = None
+        return self
+
+    def run(self):
+        NSApplication.sharedApplication().runModalForWindow_(self.window())
+
+    def acceptEntry_(self, sender):
+        value = (self.checkbox.state() == NSOnState)
+        self.closeWithResult(self.dialog.buttons[0], value)
+
+    def cancelEntry_(self, sender):
+        value = (self.checkbox.state() == NSOnState)
+        self.closeWithResult(self.dialog.buttons[1], value)
+
+    def closeWithResult(self, result, value):
+        self.result = result
+        self.value = value
+        self.window().close()
+        NSApplication.sharedApplication().stopModal()
+
+###############################################################################
+
+class CheckboxTextboxDialogController (NSWindowController):
+
+    checkbox        = IBOutlet('checkbox')
+    checkboxLabel   = IBOutlet('checkboxLabel')
+    entryField      = IBOutlet('entryField')
+    mainButton      = IBOutlet('mainButton')
+    messageField    = IBOutlet('messageField')
+    secondaryButton = IBOutlet('secondaryButton')
+    
+    def initWithDialog_(self, dialog):
+        self = super(CheckboxTextboxDialogController, self).initWithWindowNibName_owner_(u"CheckboxTextboxDialogWindow", self)
+        self.dialog = dialog
+        self.window().setTitle_(unicode(dialog.title))
+        self.messageField.setStringValue_(dialog.description)
+        self.entryField.setStringValue_(unicode(dialog.textbox_value))
+        self.checkboxLabel.setStringValue_(unicode(dialog.checkbox_text))
+        if dialog.checkbox_value:
+            self.checkbox.setState_(NSOnState)
+        else:
+            self.checkbox.setState_(NSOffState)
+        self.mainButton.setTitle_(unicode(dialog.buttons[0].text))
+        self.secondaryButton.setTitle_(dialog.buttons[1].text)
+        self.result = None
+        self.value = None
+        return self
+
+    def run(self):
+        NSApplication.sharedApplication().runModalForWindow_(self.window())
+
+    def acceptEntry_(self, sender):
+        value = (self.checkbox.state() == NSOnState)
+        text = self.entryField.stringValue()
+        self.closeWithResult(self.dialog.buttons[0], value, text)
+
+    def cancelEntry_(self, sender):
+        value = (self.checkbox.state() == NSOnState)
+        text = self.entryField.stringValue()
+        self.closeWithResult(self.dialog.buttons[1], value, text)
+
+    def closeWithResult(self, result, value, text):
+        self.result = result
+        self.value = value
+        self.text = text
+        self.window().close()
+        NSApplication.sharedApplication().stopModal()
+
+###############################################################################
+
+class SearchChannelController (NSWindowController):
+    
+    channelPopup    = IBOutlet('channelPopup')
+    createButton    = IBOutlet('createButton')
+    enginePopup     = IBOutlet('enginePopup')
+    labelField      = IBOutlet('labelField')
+    searchTermField = IBOutlet('searchTermField')
+    targetMatrix    = IBOutlet('targetMatrix')
+    urlField        = IBOutlet('urlField')
     
     def initWithDialog_(self, dialog):
         self = super(SearchChannelController, self).initWithWindowNibName_owner_(u'SearchChannelWindow', self)
