@@ -1,17 +1,44 @@
+# Miro - an RSS based video player application
+# Copyright (C) 2005-2007 Participatory Culture Foundation
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+#
+# In addition, as a special exception, the copyright holders give
+# permission to link the code of portions of this program with the OpenSSL
+# library.
+#
+# You must obey the GNU General Public License in all respects for all of
+# the code used other than OpenSSL. If you modify file(s) with this
+# exception, you may extend this exception to your version of the file(s),
+# but you are not obligated to do so. If you do not wish to do so, delete
+# this exception statement from your version. If you delete this exception
+# statement from all source files in the program, then also delete it here.
+
 import time
 import logging
 
-from objc import YES, NO, nil
+from objc import YES, NO, nil, IBOutlet
 from AppKit import *
 from Foundation import *
-from PyObjCTools import NibClassBuilder
 
 import app
 import eventloop
 import platformcfg
 import platformutils
 
-from MainFrame import handleKey
+from MainFrame import Slider, handleKey
 from QuicktimeRenderer import QuicktimeRenderer
 from QTKit import QTMovieDidEndNotification
 
@@ -127,7 +154,18 @@ class VideoDisplay (app.VideoDisplayBase):
 
 ###############################################################################
 
-class VideoDisplayController (NibClassBuilder.AutoBaseClass):
+class VideoDisplayController (NSObject):
+
+    backwardButton      = IBOutlet('backwardButton')
+    forwardButton       = IBOutlet('forwardButton')
+    fullscreenButton    = IBOutlet('fullscreenButton')
+    muteButton          = IBOutlet('muteButton')
+    playPauseButton     = IBOutlet('playPauseButton')
+    stopButton          = IBOutlet('stopButton')
+    progressDisplayer   = IBOutlet('progressDisplayer')
+    rootView            = IBOutlet('rootView')
+    volumeSlider        = IBOutlet('volumeSlider')
+    videoAreaView       = IBOutlet('videoAreaView')
 
     _instance = nil
 
@@ -310,7 +348,9 @@ class VideoDisplayController (NibClassBuilder.AutoBaseClass):
 
 ###############################################################################
 
-class VideoAreaView (NibClassBuilder.AutoBaseClass):
+class VideoAreaView (NSView):
+    
+    videoWindow = IBOutlet('videoWindow')
     
     def setup(self, item, renderer):
         if not self.videoWindow.isFullScreen:
@@ -347,6 +387,7 @@ class VideoAreaView (NibClassBuilder.AutoBaseClass):
             self.window().deminiaturize_(nil)
         self.window().orderFront_(nil)
         self.videoWindow.orderFront_(nil)
+        self.window().makeFirstResponder_(self.window().delegate())
         if self.videoWindow.parentWindow() is nil:
             self.window().addChildWindow_ordered_(self.videoWindow, NSWindowAbove)
     
@@ -378,7 +419,7 @@ class VideoAreaView (NibClassBuilder.AutoBaseClass):
     def windowDidBecomeKey_(self, notification):
         self.adjustVideoWindowFrame()
         self.window().addChildWindow_ordered_(self.videoWindow, NSWindowAbove)
-        self.window().makeKeyAndOrderFront_(nil)
+        self.window().orderFront_(nil)
     
     @platformutils.onMainThread
     def enterFullScreen(self):
@@ -391,14 +432,17 @@ class VideoAreaView (NibClassBuilder.AutoBaseClass):
     @platformutils.onMainThread
     def exitFullScreen(self):
         if self.videoWindow.isFullScreen:
+            self.window().orderFront_(nil)
             self.window().addChildWindow_ordered_(self.videoWindow, NSWindowAbove)
-            self.window().makeKeyAndOrderFront_(nil)
             self.videoWindow.exitFullScreen()
+            self.window().makeKeyWindow()
     
 
 ###############################################################################
 
-class VideoWindow (NibClassBuilder.AutoBaseClass):
+class VideoWindow (NSWindow):
+    
+    palette = IBOutlet('palette')
     
     def initWithContentRect_styleMask_backing_defer_(self, rect, style, backing, defer):
         self = super(VideoWindow, self).initWithContentRect_styleMask_backing_defer_(
@@ -463,6 +507,9 @@ class VideoWindow (NibClassBuilder.AutoBaseClass):
     def previousVideo_(self, sender):
         eventloop.addIdle(lambda:app.controller.playbackController.skip(-1, False), "Skip Video")
 
+    def stopVideo_(self, sender):
+        eventloop.addIdle(lambda:app.controller.playbackController.stop(), "Stop Video")
+
     def sendEvent_(self, event):
         if self.isFullScreen:
             if event.type() == NSLeftMouseDown:
@@ -553,7 +600,17 @@ class SkipSeekButtonCell (NSButtonCell):
 
 ###############################################################################
 
-class FullScreenPalette (NibClassBuilder.AutoBaseClass):
+class FullScreenPalette (NSWindow):
+    
+    donationLabel       = IBOutlet('donationLabel')
+    feedLabel           = IBOutlet('feedLabel')
+    playPauseButton     = IBOutlet('playPauseButton')
+    progressSlider      = IBOutlet('progressSlider')
+    seekBackwardButton  = IBOutlet('seekBackwardButton')
+    seekForwardButton   = IBOutlet('seekForwardButton')
+    timeIndicator       = IBOutlet('timeIndicator')
+    titleLabel          = IBOutlet('titleLabel')
+    volumeSlider        = IBOutlet('volumeSlider')
     
     HOLD_TIME = 2
     
@@ -696,7 +753,7 @@ class FullScreenPalette (NibClassBuilder.AutoBaseClass):
 
 ###############################################################################
 
-class FullScreenPaletteView (NibClassBuilder.AutoBaseClass):
+class FullScreenPaletteView (NSView):
 
     def awakeFromNib(self):
         self.background = NSImage.imageNamed_(u'fs-background')
@@ -715,7 +772,7 @@ class FullScreenPaletteView (NibClassBuilder.AutoBaseClass):
 
 ###############################################################################
 
-class FullScreenControlsView (NibClassBuilder.AutoBaseClass):
+class FullScreenControlsView (NSView):
     
     def awakeFromNib(self):
         self.background = NSImage.imageNamed_(u'fs-controls-background')
@@ -736,7 +793,7 @@ class FullScreenControlsView (NibClassBuilder.AutoBaseClass):
 
 ###############################################################################
 
-class FullScreenSlider (NibClassBuilder.AutoBaseClass):
+class FullScreenSlider (Slider):
 
     def drawTrack(self):
         self.track.compositeToPoint_operation_((0, 2), NSCompositeSourceOver)
