@@ -1,11 +1,43 @@
+# Miro - an RSS based video player application
+# Copyright (C) 2005-2007 Participatory Culture Foundation
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+#
+# In addition, as a special exception, the copyright holders give
+# permission to link the code of portions of this program with the OpenSSL
+# library.
+#
+# You must obey the GNU General Public License in all respects for all of
+# the code used other than OpenSSL. If you modify file(s) with this
+# exception, you may extend this exception to your version of the file(s),
+# but you are not obligated to do so. If you do not wish to do so, delete
+# this exception statement from your version. If you delete this exception
+# statement from all source files in the program, then also delete it here.
+
 import os.path
 import os
+import time
+import socket
 import copy
 import sys
 import string
 import subprocess
 import zipfile as zip
 from glob import glob
+from xml.sax.saxutils import escape
+from distutils import sysconfig 
 
 ###############################################################################
 ## Paths and configuration                                                   ##
@@ -24,30 +56,25 @@ NSIS_PATH = 'C:\\Program Files\\NSIS\\makensis.exe'
 # below.
 defaultBinaryKitRoot = os.path.join(os.path.dirname(sys.argv[0]), \
                                     '..', '..', '..', 'dtv-binary-kit')
-BINARY_KIT_ROOT = defaultBinaryKitRoot
+BINARY_KIT_ROOT = os.path.abspath(defaultBinaryKitRoot)
 
-# Set BOOST_LIB, BOOST_INCLUDE_PATH, and BOOST_RUNTIMES as appropriate
-# for the location of Boost, the version you built, and the compiler
-# you used. If you are unsure, search your hard drive for a file named
-# "boost_python*". If there is more than one, you probably already
-# know what you're doing.
-# NEEDS: better accomodate non-vc71 compilers in binary kit?
 BOOST_ROOT = os.path.join(BINARY_KIT_ROOT, 'boost', 'win32')
 BOOST_LIB_PATH = os.path.join(BOOST_ROOT, 'lib')
+BOOST_INCLUDE_PATH = os.path.join(BOOST_ROOT, 'include')
+BOOST_LIBRARIES = [os.path.splitext(os.path.basename(f))[0] for f in
+        glob(os.path.join(BOOST_LIB_PATH, '*.lib'))]
+BOOST_RUNTIMES = glob(os.path.join(BOOST_LIB_PATH, '*.dll'))
+
+ZLIB_INCLUDE_PATH = os.path.join(BINARY_KIT_ROOT, 'zlib', 'include')
+ZLIB_LIB_PATH = os.path.join(BINARY_KIT_ROOT, 'zlib', 'lib')
+ZLIB_RUNTIME_LIBRARY_PATH = os.path.join(BINARY_KIT_ROOT, 'zlib')
+ZLIB_RUNTIMES = [os.path.join(ZLIB_RUNTIME_LIBRARY_PATH, 'zlib1.dll')]
+
+OPENSSL_INCLUDE_PATH = os.path.join(BINARY_KIT_ROOT, 'openssl', 'include')
+OPENSSL_LIB_PATH = os.path.join(BINARY_KIT_ROOT, 'openssl', 'lib')
+OPENSSL_LIBRARIES = [ 'ssleay32', 'libeay32']
+
 
-# Hack so we don't break things for people who haven't updated their binary kit
-if os.path.exists(os.path.join(BOOST_ROOT, 'include', 'boost-1_33_1')):
-    BOOST_LIB = os.path.join(BOOST_LIB_PATH, 'boost_python-vc71-mt-1_33_1.lib')
-    BOOST_INCLUDE_PATH = os.path.join(BOOST_ROOT, 'include', 'boost-1_33_1')
-    BOOST_RUNTIMES = [
-        os.path.join(BOOST_LIB_PATH, 'boost_python-vc71-mt-1_33_1.dll'),
-        ]
-else:
-    BOOST_LIB = os.path.join(BOOST_LIB_PATH, 'boost_python-vc71-mt-1_33.lib')
-    BOOST_INCLUDE_PATH = os.path.join(BOOST_ROOT, 'include', 'boost-1_33')
-    BOOST_RUNTIMES = [
-        os.path.join(BOOST_LIB_PATH, 'boost_python-vc71-mt-1_33.dll'),
-        ]
 
 # The 'Democracy.exe' launcher stub, currently provided only in the
 # binary kit.
@@ -145,46 +172,104 @@ platform = 'windows-xul'
 
 # Find the top of the source tree and set search path
 root = os.path.join(os.path.dirname(os.path.abspath(sys.argv[0])), '..', '..')
+platform_dir = os.path.join(root, 'platform', 'windows-xul')
+portable_dir = os.path.join(root, 'portable')
 sys.path[0:0] = [
-    os.path.join(root, 'platform', platform),
-    os.path.join(root, 'platform'),
-    os.path.join(root, 'portable'),
+    platform_dir,
+    portable_dir,
 ]
 root = os.path.normpath(root)
 
+#### The database extension ####
+database_ext = Extension("database", 
+        sources=[os.path.join(root, 'portable', 'database.pyx')])
+
 #### The fasttypes extension ####
 
 fasttypes_ext = \
     Extension("fasttypes", 
         sources = [os.path.join(root, 'portable', 'fasttypes.cpp')],
-        extra_objects = [BOOST_LIB],
+        library_dirs = [BOOST_LIB_PATH],
         include_dirs = [BOOST_INCLUDE_PATH]
     )
 
-os.environ['PATH'] = r'%s;%s' % (os.environ['PATH'], BOOST_LIB_PATH)
+# Setting the path here allows py2exe to find the DLLS
+os.environ['PATH'] = r'%s;%s;%s;%s' % (os.environ['PATH'], BOOST_LIB_PATH,
+        ZLIB_RUNTIME_LIBRARY_PATH, OPENSSL_LIB_PATH)
+
+##### The libtorrent extension ####
+
+def fetchCpp():
+    for root,dirs,files in os.walk(os.path.join(portable_dir, 'libtorrent')):
+        if '.svn' in dirs:
+            dirs.remove('.svn')
+        if '_svn' in dirs:
+            dirs.remove('_svn')
+        for file in files:
+            if file.endswith('.cpp'):
+                yield os.path.join(root,file)
+
+libtorrent_sources=list(fetchCpp())
+libtorrent_sources.remove(os.path.join(portable_dir, 'libtorrent\\src\\file.cpp'))
+
+print [ BOOST_LIB_PATH, OPENSSL_LIB_PATH, ZLIB_LIB_PATH]
+libtorrent_ext = Extension(
+        "libtorrent", 
+        include_dirs = [
+            os.path.join(portable_dir, 'libtorrent', 'include'),
+            os.path.join(portable_dir, 'libtorrent', 'include', 'libtorrent'),
+            BOOST_INCLUDE_PATH, 
+            ZLIB_INCLUDE_PATH, 
+            OPENSSL_INCLUDE_PATH,
+        ],
+        library_dirs = [ BOOST_LIB_PATH, OPENSSL_LIB_PATH, ZLIB_LIB_PATH],
+        libraries = OPENSSL_LIBRARIES + BOOST_LIBRARIES + [
+            'wsock32', 'gdi32', 'ws2_32', 'zdll'
+            ],
+        extra_compile_args = [  '-DBOOST_WINDOWS',
+            '-DWIN32_LEAN_AND_MEAN',
+            '-D_WIN32_WINNT=0x0500',
+            '-D__USE_W32_SOCKETS',
+            '-D_WIN32',
+            '-DWIN32',
+            '-DBOOST_ALL_NO_LIB',
+            '-D_FILE_OFFSET_BITS=64',
+            '-DBOOST_THREAD_USE_LIB',
+            '-DTORRENT_USE_OPENSSL=1',
+            '-DNDEBUG=1',
+            '/EHa', '/GR',
+            ],
+        sources = libtorrent_sources)
 
 # Private extension modules to build.
 ext_modules = [
     fasttypes_ext,
+    libtorrent_ext,
+    database_ext,
 
     # Pyrex sources.
     #Extension("vlc", [os.path.join(root, 'platform',platform, 'vlc.pyx')],libraries=["simplevlc"]),
-    Extension("database", [os.path.join(root, 'portable', 'database.pyx')]),
     Extension("sorts", [os.path.join(root, 'portable', 'sorts.pyx')]),
     #Extension("template", [os.path.join(root, 'portable', 'template.pyx')]),
 ]
 
 import template_compiler
 
-def fillTemplate(templatepath, outpath, **vars):
+def fillTemplate(templatepath, outpath, xml=False, **vars):
     s = open(templatepath, 'rt').read()
-    s = string.Template(s).safe_substitute(**vars)
+    if xml:
+        xmlvars = {}
+        for var in vars:
+            xmlvars[var] = escape(vars[var],{"'":"&apos;","\"":"&quot;"})
+        s = string.Template(s).safe_substitute(**xmlvars)
+    else:
+        s = string.Template(s).safe_substitute(**vars)
     f = open(outpath, "wt")
     f.write(s)
     f.close()
 
 def dtdQuoteHack(path):
-    """Replace the "'" charactor with "&quot;" instead. 
+    """Replace the "'" charactor with "&apos;" instead. 
 
     As far as I (BDK) can tell, it's legal for the XML spec, but we have had
     reports of it not working in the wild (#3579).
@@ -193,10 +278,9 @@ def dtdQuoteHack(path):
     content = f.read()
     f.close()
     f = open(path, 'w')
-    f.write(content.replace("'", "&quot;"))
+    f.write(content.replace("'", "&apos;"))
     f.close()
 
-
 ###############################################################################
 
 # WHEN BUILDING A REDISTRIBUTABLE IMAGE:
@@ -265,6 +349,7 @@ class bdist_xul_dumb(Command):
                                            'xul')
 
     def run(self):
+        self.buildMovieDataUtil()
         self.makeAppConfig()
         self.makeStubEXE()
         self.setTemplateVariable("pyxpcomIsEmbedded", "true")
@@ -448,7 +533,8 @@ class bdist_xul_dumb(Command):
         # current directory set appropriately.]
 #        dllDistDir = self.xulrunnerOut
         dllDistDir = self.dist_dir
-        allRuntimes = PYTHON_RUNTIMES + BOOST_RUNTIMES + COMPILER_RUNTIMES
+        allRuntimes = (PYTHON_RUNTIMES + BOOST_RUNTIMES + COMPILER_RUNTIMES +
+                ZLIB_RUNTIMES)
         for runtime in allRuntimes:
             shutil.copy2(runtime, dllDistDir)
 
@@ -457,8 +543,11 @@ class bdist_xul_dumb(Command):
         copyTreeExceptSvn(self.appResources,
                           os.path.join(self.dist_dir, 'resources'),
                           filterOut=['unittest', 'testdata'])
+        self.copyMovieDataUtil()
         shutil.copy2("Democracy.nsi", self.dist_dir)
-        shutil.copy2("%s.ico" % (self.getTemplateVariable('shortAppName')), self.dist_dir)
+        shutil.copy2("Miro.ico", os.path.join(self.dist_dir, "%s.ico" % (self.getTemplateVariable('shortAppName'))))
+        shutil.copy2("iHeartMiro-installer-page.ini", self.dist_dir)
+#        copyTreeExceptSvn(os.path.join(platform_dir, "iHeartMiro"), os.path.join(self.dist_dir, "iHeartMiro"))
 
 
         locale_dir = os.path.join (self.appResources, "locale")
@@ -480,7 +569,7 @@ class bdist_xul_dumb(Command):
         os.rename(os.path.join(self.xulrunnerOut, "xulrunner.exe"),
                   os.path.join(self.xulrunnerOut, "%s.exe" % (self.getTemplateVariable('shortAppName'))))
         os.remove(os.path.join(self.xulrunnerOut, "xulrunner-stub.exe"))
-
+        
         # Finally, build the download daemon
         self.buildDownloadDaemon(self.dist_dir)
         
@@ -566,8 +655,34 @@ class bdist_xul_dumb(Command):
 
     def buildDownloadDaemon(self, baseDir):
         print "building download daemon"
-        os.system("%s setup_daemon.py py2exe --dist-dir daemon --bundle-files 1" % PYTHON_BINARY)
-        shutil.copy2(os.path.join("daemon","%s_Downloader.exe" % (self.getTemplateVariable('shortAppName'))), baseDir)
+        os.system("%s setup_daemon.py py2exe --dist-dir %s" % 
+                (PYTHON_BINARY, self.dist_dir))
+
+    def buildMovieDataUtil(self):
+        print "building movie data utility"
+        from distutils.ccompiler import new_compiler
+
+        platform_dir = os.path.join(root, 'platform', platform)
+        sources = [ os.path.join(platform_dir, 'moviedata_util.c') ]
+        python_base = sysconfig.get_config_var('prefix')
+        include_dirs = [ sysconfig.get_python_inc() ]
+        library_dirs = [ os.path.join(python_base, 'libs')]
+
+
+        compiler = new_compiler( verbose=self.verbose, dry_run=self.dry_run,
+                force=self.force)
+        sysconfig.customize_compiler(compiler)
+        objects = compiler.compile(sources, output_dir=self.bdist_base,
+                include_dirs=include_dirs)
+
+        compiler.link_executable(objects, 'Miro_MovieData',
+                library_dirs=library_dirs, output_dir=self.bdist_base)
+
+    def copyMovieDataUtil(self):
+        print "building movie data utility to dist"
+        built_exe = os.path.join(self.bdist_base, 'Miro_MovieData.exe')
+        shutil.copy2(built_exe, self.dist_dir)
+        shutil.copy2("moviedata_util.py", self.dist_dir)
 
     # NEEDS: if you look at the usage of this function, we're dropping
     # the plugin into the xulrunner plugin directory, rather than the
@@ -578,7 +693,7 @@ class bdist_xul_dumb(Command):
         if not os.access(destDir, os.F_OK):
             os.mkdir(destDir)
 
-        pluginFiles = ['npvlc.dll', 'vlcintf.xpt']
+        pluginFiles = ['libvlc.dll', 'npvlc.dll', 'vlcintf.xpt']
         for f in pluginFiles:
             shutil.copy2(os.path.join(VLC_MOZ_PLUGIN_DIR, f), destDir)
 
@@ -652,14 +767,20 @@ class bdist_xul_dumb(Command):
 
         path = os.path.join(root, 'resources', 'app.config')
         fillTemplate("%s.template" % path, path,
-            APP_REVISION = revision, APP_PLATFORM = 'windows-xul')
+                     BUILD_MACHINE="%s@%s" % (os.environ['username'],
+                                              socket.gethostname()),
+                     BUILD_TIME=str(time.time()),
+                     APP_REVISION = revision, APP_PLATFORM = 'windows-xul')
         self.templateVars = util.readSimpleConfigFile(path)
 
     def makeStubEXE(self):
          version = self.getTemplateVariable('appVersion')
          versionComponents = version.split('-')[0].split('.')
-         while len(versionComponents) > 4:
+         # FIXME - should this be < 4 or <= 4 ?
+         while len(versionComponents) < 4:
              versionComponents.append('0')
+         if len(versionComponents) > 4:
+             versionComponents = versionComponents[0:4]
          versionWithCommas = ','.join(versionComponents)
          stubDir = os.path.join(root, '..', 'dtv-binary-kit', 'stub')
          template = os.path.join(stubDir, 'Democracy.rc.template')
@@ -670,7 +791,9 @@ class bdist_xul_dumb(Command):
          os.chdir(stubDir)
          rv = os.system("rc Democracy.rc")
          if rv == 0:
-             rv = os.system('cl  /DXULRUNNER_BIN=\\"%s.exe\\" Democracy.cpp /link /subsystem:windows /machine:x86 Democracy.RES /out:%s.exe"' % (self.getTemplateVariable('shortAppName'),self.getTemplateVariable('shortAppName')))
+             cmd = 'cl  /DXULRUNNER_BIN="\\"%s.exe\\"" Democracy.cpp /link shell32.lib /subsystem:windows /machine:x86 Democracy.RES /out:"%s.exe"' % (self.getTemplateVariable('shortAppName'),self.getTemplateVariable('shortAppName'))
+             #print cmd
+             rv = os.system(cmd)
          os.chdir(olddir)
          if rv != 0:
             raise OSError("Making stub exe failed")
@@ -691,10 +814,10 @@ class bdist_xul_dumb(Command):
     # plus anything set with setTemplateVariable.
     # NEEDS: same deal as makeAppConfig: sloppy; shouldn't drop files in
     # the source tree.
-    def fillTemplate(self, filename):
+    def fillTemplate(self, filename, xml = False):
         assert self.templateVars, \
             "Must call makeAppConfig before fillTemplate"
-        fillTemplate("%s.template" % filename, filename, **self.templateVars)
+        fillTemplate("%s.template" % filename, filename, xml = xml, **self.templateVars)
 
     def fillTemplates(self):
         self.fillTemplate(os.path.join(root,'portable','dl_daemon','daemon.py'))
@@ -709,7 +832,7 @@ class bdist_xul_dumb(Command):
                 continue
             for fname in glob (os.path.join (lang, '*.template')):
                 dtd_fname = fname[:-len('.template')]
-                self.fillTemplate(dtd_fname)
+                self.fillTemplate(dtd_fname, xml=True)
                 dtdQuoteHack(dtd_fname)
         import validate_dtds
         validate_dtds.check_dtds(os.path.join (xulBase, 'chrome', 'locale'))
@@ -773,6 +896,7 @@ class bdist_xul (bdist_xul_dumb):
             nsisVars[nsisName] = self.getTemplateVariable(ourName)
 
         nsisVars['CONFIG_EXECUTABLE'] = "%s.exe" % (self.getTemplateVariable('shortAppName'))
+        nsisVars['CONFIG_MOVIE_DATA_EXECUTABLE'] = "%s_MovieData.exe" % (self.getTemplateVariable('shortAppName'))
         nsisVars['CONFIG_DL_EXECUTABLE'] = "%s_Downloader.exe" % (self.getTemplateVariable('shortAppName'))
         nsisVars['CONFIG_ICON'] = "%s.ico" % (self.getTemplateVariable('shortAppName'))
         nsisVars['CONFIG_PROG_ID'] = self.getTemplateVariable('longAppName').replace(" ",".")+".1"
@@ -820,8 +944,6 @@ class bdist_xul (bdist_xul_dumb):
             self.addFile ("python24.dll")
         else: # We must be using Python 2.5
             self.addFile ("python25.dll")
-        for boostFile in BOOST_RUNTIMES:
-            self.addFile (os.path.basename(boostFile))
 
         self.addDirectory ("chrome")
         self.addDirectory ("components")
