@@ -1,3 +1,33 @@
+/*
+# Miro - an RSS based video player application
+# Copyright (C) 2005-2007 Participatory Culture Foundation
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+#
+# In addition, as a special exception, the copyright holders give
+# permission to link the code of portions of this program with the OpenSSL
+# library.
+#
+# You must obey the GNU General Public License in all respects for all of
+# the code used other than OpenSSL. If you modify file(s) with this
+# exception, you may extend this exception to your version of the file(s),
+# but you are not obligated to do so. If you do not wish to do so, delete
+# this exception statement from your version. If you delete this exception
+# statement from all source files in the program, then also delete it here.
+*/
+
 const JSBRIDGE_CONTRACTID = "@participatoryculture.org/dtv/jsbridge;1";
 const JSBRIDGE_CLASSID    = Components.ID("{421AA951-F53D-4499-B362-E432CAE920F4}");
 
@@ -17,6 +47,43 @@ function twoDigits(data) {
     else return ""+data;
 }
 
+
+function formatTime(milliseconds) {
+    if(milliseconds < 0) milliseconds = 0;
+    total_seconds = Math.floor(milliseconds / 1000);
+    var hours = Math.floor(total_seconds/3600);
+    var mins = Math.floor((total_seconds - hours*3600)/60);
+    var secs = total_seconds - hours*3600 - mins*60;
+    return twoDigits(hours)+":"+twoDigits(mins)+":"+twoDigits(secs);
+}
+
+function makeLocalFile(path) {
+    var file = Components.classes["@mozilla.org/file/local;1"].
+            createInstance(Components.interfaces.nsILocalFile);
+    file.initWithPath(path);
+    return file;
+}
+
+function pickSavePath(window, title, defaultDirectory, defaultFilename) {
+    var nsIFilePicker = Components.interfaces.nsIFilePicker;
+    var fp = Components.classes["@mozilla.org/filepicker;1"]
+                  .createInstance(nsIFilePicker);
+    fp.init(window, title, nsIFilePicker.modeSave);
+    if(defaultDirectory) {
+       fp.setDefaultDirectory(makeLocalFile(defaultDirectory));
+    }
+    if(defaultFilename) {
+      fp.defaultString = defaultFilename;
+    }
+    var returncode = fp.show();
+    if (returncode == nsIFilePicker.returnOK || 
+            returncode == nsIFilePicker.returnReplace) {
+       return fp.file.path;
+    }  else {
+       return null;
+    }
+}
+
 function LoadFinishedListener(area)
 {
     this.area = area;
@@ -109,6 +176,11 @@ jsBridge.prototype = {
   closeWindow: function() {
     this.window.close();
   },
+ 
+  maximizeWindow: function () {
+    this.window.maximize();
+    this.window.maximized = true;
+  },
 
   initBrowser: function(area) {
     var browser = this.document.getElementById(area);
@@ -174,6 +246,26 @@ jsBridge.prototype = {
             "dialog", "chrome,dependent,centerscreen,modal", params);
   },
 
+  showCheckboxDialog: function(id, title, description, defaultLabel,
+              otherLabel, checkboxText, checkboxValue) {
+    var params = { "id": id, "title": title, "description" : description, 
+        "defaultLabel": defaultLabel, "otherLabel": otherLabel, 
+        "checkboxText": checkboxText, "checkboxValue": checkboxValue,
+        "out" : -1};
+    this.window.openDialog("chrome://dtv/content/checkbox_dialog.xul",
+            "dialog", "chrome,dependent,centerscreen,modal", params);
+  },
+  showCheckboxTextboxDialog: function(id, title, description, defaultLabel,
+              otherLabel, checkboxText, checkboxValue, textboxValue) {
+    var params = { "id": id, "title": title, "description" : description, 
+        "defaultLabel": defaultLabel, "otherLabel": otherLabel, 
+        "checkboxText": checkboxText, "checkboxValue": checkboxValue,
+        "out" : -1, "textboxValue": textboxValue};
+    this.window.openDialog("chrome://dtv/content/checkboxtextbox_dialog.xul",
+            "dialog", "chrome,dependent,centerscreen,modal", params);
+  },
+
+
   showThreeChoiceDialog: function(id, title, description, defaultLabel,
                         secondLabel, thirdLabel) {
     var params = { "id": id, "title": title, "description" : description, 
@@ -371,7 +463,7 @@ jsBridge.prototype = {
 
   xulNavigateDisplay: function(area, uri) {
     var browser = this.document.getElementById(area);
-    browser.contentDocument.location = uri;
+    browser.loadURI(uri);
   },
 
   getDocument: function(area) {
@@ -451,19 +543,13 @@ jsBridge.prototype = {
     if(newFilename) this.videoFilename = newFilename;
     else this.videoFilename = null;
   },
+
   saveVideo: function() {
     if(this.videoFilename == null) return;
-
-    var fp = Components.classes["@mozilla.org/filepicker;1"]
-            .createInstance(Components.interfaces.nsIFilePicker);
     var saveMenuItem = this.document.getElementById('menuitem-savevideo');
-    fp.init(this.window, saveMenuItem.getAttribute('label'),
-        Components.interfaces.nsIFilePicker.modeSave);
-    fp.defaultString = this.videoFilename;
-    var res = fp.show();
-    if (res == Components.interfaces.nsIFilePicker.returnOK){
-        pybridge.saveVideoFile(fp.file.path);
-    }
+    var picked = pickSavePath(this.window, saveMenuItem.getAttribute('label'),
+            null, this.videoFilename);
+    if (picked) pybridge.saveVideoFile(picked);
   },
 
   performStartupTasks: function() {
@@ -488,13 +574,13 @@ jsBridge.prototype = {
   },
 
   setSliderText: function(elapsed) {
-    elapsed = Math.floor(elapsed / 1000);
-    var hours = Math.floor(elapsed/3600);
-    var mins = Math.floor((elapsed - hours*3600)/60);
-    var secs = elapsed - hours*3600 - mins*60;
-    var text = twoDigits(hours)+":"+twoDigits(mins)+":"+twoDigits(secs);
     var sliderText = this.document.getElementById("progress-text");
-    sliderText.childNodes[0].nodeValue = text;
+    sliderText.childNodes[0].nodeValue = formatTime(elapsed);
+  },
+
+  setDuration: function(duration) {
+    var sliderText = this.document.getElementById("duration-text");
+    sliderText.childNodes[0].nodeValue = formatTime(duration);
   },
 
   moveSlider: function(fractionDone) {
@@ -634,17 +720,16 @@ jsBridge.prototype = {
     }
     this.prefDocument.selectDirectoryWatch(true);
   },
-  hidePopup: function() {
-      this.popup.hidePopup();
-  },
   showPopup: function(x, y) {
       // show popup and adjust position once we know width / height
       // Based on core.js from Firefox minimizetotray extension
       var screenwidth = this.window.screen.width;
       var screenheight = this.window.screen.height;
       var document = this.document;
+      var window = this.window;
 
       this.popup = document.getElementById('traypopup');
+      var self = this;
 
       function minimize_onshown(event, x, y, screenWidth, screenHeight, document) {
 
@@ -662,11 +747,13 @@ jsBridge.prototype = {
           // re-show the popup in the right position
           popup.hidePopup();
           document.popupNode = null;
+          minimizer.contextMenuHack();
           popup.showPopup(
                           document.documentElement,
                           x, y,
                           "context",
                           "", "");
+          minimizer.contextMenuHack2();
       }
       this.popup._minimizetotray_onshown = function(event){ return minimize_onshown(event, x, y, screenwidth, screenheight, document); };
       this.popup.addEventListener("popupshown",
@@ -681,6 +768,32 @@ jsBridge.prototype = {
                       "");
 
   },
+
+  showOpenDialog: function (id, title, defaultDirectory, typeString, types) {
+      var nsIFilePicker = Components.interfaces.nsIFilePicker;
+      var fp = Components.classes["@mozilla.org/filepicker;1"]
+                      .createInstance(nsIFilePicker);
+      fp.init(this.window, title, nsIFilePicker.modeOpen);
+      if(defaultDirectory) {
+          fp.setDefaultDirectory(makeLocalFile(defaultDirectory));
+      }
+      if(types) {
+        var filters = new Array();
+        for(var i = 0; i < types.length; i++) {
+          filters[i] = "*." + types[i];
+        }
+        fp.appendFilter(typeString, filters.join(";"));
+      }
+      fp.appendFilters(nsIFilePicker.filterAll);
+      if (fp.show() == nsIFilePicker.returnOK){
+        pybridge.handleFileDialog(id, fp.file.path);
+    }
+  },
+
+  showSaveDialog: function (id, title, defaultDirectory, defaultFilename) {
+      var picked = pickSavePath(this.window, title, defaultDirectory, defaultFilename);
+      if (picked) pybridge.handleFileDialog(id, picked);
+  },
 };
 
 var Module = {
