@@ -1,3 +1,31 @@
+# Miro - an RSS based video player application
+# Copyright (C) 2005-2007 Participatory Culture Foundation
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+#
+# In addition, as a special exception, the copyright holders give
+# permission to link the code of portions of this program with the OpenSSL
+# library.
+#
+# You must obey the GNU General Public License in all respects for all of
+# the code used other than OpenSSL. If you modify file(s) with this
+# exception, you may extend this exception to your version of the file(s),
+# but you are not obligated to do so. If you do not wish to do so, delete
+# this exception statement from your version. If you delete this exception
+# statement from all source files in the program, then also delete it here.
+
 from xpcom import components
 import ctypes
 import logging
@@ -6,6 +34,10 @@ UINT = ctypes.c_uint
 WCHAR = ctypes.c_wchar
 INT = ctypes.c_int
 
+WH_MOUSE_LL = 14
+WH_MOUSE    = 7
+
+WM_NULL = 0x0000
 WM_USER = 0x0400
 WM_TRAYICON = WM_USER+0x1EEF
 WM_GETICON = 0x007F
@@ -16,6 +48,7 @@ WM_GETMINMAXINFO = 0x0024
 WM_ACTIVATEAPP = 0x001C
 WM_ACTIVATE = 0x0006
 WM_INITMENUPOPUP = 0x0117
+WM_MOUSEMOVE = 0x0200
 WS_EX_APPWINDOW = 0x00040000L
 WS_MAXIMIZE = 0x01000000L
 SIZE_MAXIMIZED = 2
@@ -62,7 +95,9 @@ IDANI_OPEN         = 1
 IDANI_CAPTION      = 3
 
 SW_HIDE = 0
+SW_SHOWMINIMIZED = 2
 SW_SHOW = 5
+SW_RESTORE = 9
 
 GWL_WNDPROC = -4
 
@@ -141,6 +176,16 @@ class MINMAXINFO(ctypes.Structure):
                 ("ptMinTrackSize", POINT),
                 ("ptMaxTrackSize",POINT)]
 
+class WINDOWPLACEMENT(ctypes.Structure):
+    _fields_ = [
+                ("length", UINT),
+                ("flags", UINT),
+                ("showCmd", UINT),
+                ("ptMinPosition", POINT),
+                ("ptMaxPosition", POINT),
+                ("rcNormalPosition", RECT),
+    ]
+
 def getTaskbarEdge():
     appBarData = APPBARDATA(0,0,0,0,RECT(0,0,0,0),0)
     if (ctypes.windll.shell32.SHAppBarMessage(ABM_GETTASKBARPOS,ctypes.byref(appBarData)) != 0):
@@ -181,16 +226,6 @@ def PyWindProc(hWnd, uMsg, wParam, lParam):
             Minimize.minimizers[hWnd].minimizeOrRestore()
         elif lParam == WM_RBUTTONUP:
             Minimize.minimizers[hWnd].showPopup(mouseX, mouseY)
-    elif uMsg == WM_ACTIVATEAPP:
-        pass
-        #logging.info("TACTIVATEAPP %d %d" % (wParam, lParam))
-        #jsbridge = components.classes["@participatoryculture.org/dtv/jsbridge;1"].getService(components.interfaces.pcfIDTVJSBridge)
-        #jsbridge.hidePopup()
-    elif uMsg == WM_ACTIVATE:
-        pass
-        #logging.info("TACTIVATE %d %d" % (wParam, lParam))
-        #jsbridge = components.classes["@participatoryculture.org/dtv/jsbridge;1"].getService(components.interfaces.pcfIDTVJSBridge)
-        #jsbridge.hidePopup()
 
     #components.classes['@mozilla.org/consoleservice;1'].getService(components.interfaces.nsIConsoleService).logStringMessage("PYWINPROC %d %d %d %d" % (hWnd, uMsg, wParam, lParam))
     return ctypes.windll.user32.CallWindowProcW(ctypes.windll.user32.DefWindowProcW,hWnd, uMsg, wParam, lParam)
@@ -222,20 +257,9 @@ def PyMainWindProc(hWnd, uMsg, wParam, lParam):
             info.ptMaxSize.y = window.screen.height
             info.ptMaxPosition.x = height
             info.ptMaxPosition.y = 0
-    elif uMsg == WM_ACTIVATEAPP:
-        pass
-        #logging.info("ACTIVATEAPP %d %d" % (wParam, lParam))
-        #jsbridge = components.classes["@participatoryculture.org/dtv/jsbridge;1"].getService(components.interfaces.pcfIDTVJSBridge)
-        #jsbridge.hidePopup()
-    elif uMsg == WM_ACTIVATE:
-        pass
-        #logging.info("ACTIVATE %d %d" % (wParam, lParam))
-        #jsbridge = components.classes["@participatoryculture.org/dtv/jsbridge;1"].getService(components.interfaces.pcfIDTVJSBridge)
-        #jsbridge.hidePopup()
             
     return ctypes.windll.user32.CallWindowProcW(Minimize.oldWindowProcs[hWnd],hWnd, uMsg, wParam, lParam)
 
-
 WindProc = WNDPROCTYPE(PyWindProc)
 MainWindProc = WNDPROCTYPE(PyMainWindProc)
 
@@ -304,6 +328,21 @@ class Minimize:
         href = self.getHREFFromDOMWindow(win)
         Minimize.oldWindowProcs[href.value] = ctypes.windll.user32.SetWindowLongW(href,GWL_WNDPROC, MainWindProc)
 
+    def contextMenuHack(self):
+        """Hack to make context menus work, must be called BEFORE the menu is
+        shown.
+        """
+        # Need to make the XUL window the foreground window.  See 
+        # http://support.microsoft.com/kb/135788
+        # If the msdn URL doesn't work, try searhing for Q135788
+        ctypes.windll.user32.SetForegroundWindow(self.trayIconWindow)
+
+    def contextMenuHack2(self):
+        """Hack to make context menus work, must be called AFTER the menu is
+        shown.
+        """
+        ctypes.windll.user32.PostMessageA(self.trayIconWindow, WM_NULL, 0, 0)
+
     def getHREFFromBaseWindow(self, win):
         return ctypes.c_int(self._gethrefcomp.getit(win))
 
@@ -383,7 +422,6 @@ class Minimize:
         return None
     
     def showPopup(self, x, y):
-        print "x and y is %d  %d" % (x, y)
         jsbridge = components.classes["@participatoryculture.org/dtv/jsbridge;1"].getService(components.interfaces.pcfIDTVJSBridge)
         jsbridge.showPopup(x, y)
 
@@ -396,13 +434,22 @@ class Minimize:
         ctypes.windll.user32.ShowWindow(href, ctypes.c_int(SW_HIDE))
 
     def restore(self, href):
+        placement = WINDOWPLACEMENT()
+        rv = ctypes.windll.user32.GetWindowPlacement(href, 
+                ctypes.byref(placement))
+        if rv and placement.showCmd == SW_SHOWMINIMIZED:
+            show_flag = SW_RESTORE
+        else:
+            show_flag = SW_SHOW
+        ctypes.windll.user32.ShowWindow(href, ctypes.c_int(show_flag))
+        ctypes.windll.user32.SetForegroundWindow(href)
+
         fromer = RECT(0,0,0,0)
         ctypes.windll.user32.GetWindowRect(href,ctypes.byref(fromer))
         to = self.getTrayRect()
         if to:
             ctypes.windll.user32.DrawAnimatedRects(href,IDANI_CAPTION,ctypes.byref(to),ctypes.byref(fromer))
 
-        ctypes.windll.user32.ShowWindow(href, ctypes.c_int(SW_SHOW))
 
     def minimizeOrRestore(self):
         pybridge = components.classes["@participatoryculture.org/dtv/pybridge;1"].getService(components.interfaces.pcfIDTVPyBridge)
