@@ -1,3 +1,31 @@
+# Miro - an RSS based video player application
+# Copyright (C) 2005-2007 Participatory Culture Foundation
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+#
+# In addition, as a special exception, the copyright holders give
+# permission to link the code of portions of this program with the OpenSSL
+# library.
+#
+# You must obey the GNU General Public License in all respects for all of
+# the code used other than OpenSSL. If you modify file(s) with this
+# exception, you may extend this exception to your version of the file(s),
+# but you are not obligated to do so. If you do not wish to do so, delete
+# this exception statement from your version. If you delete this exception
+# statement from all source files in the program, then also delete it here.
+
 from gettext import gettext as _
 from xpcom import components
 import ctypes
@@ -263,10 +291,6 @@ class PyBridge:
         registerHttpObserver()
         app.main()
         initializeHTTPProxy()
-        views.waitForInit()
-        self.initializeSearchEngines()
-        
-        migrateappname.migrateVideos('Democracy', 'Miro')
 
     @asUrgent
     def initializeViews(self):
@@ -332,6 +356,14 @@ class PyBridge:
         return config.get(prefs.UPSTREAM_LIMIT_IN_KBS)
     def setLimitUpstreamAmount(self, value):
         return config.set(prefs.UPSTREAM_LIMIT_IN_KBS, value)
+    def getLimitDownstream(self):
+        return config.get(prefs.LIMIT_DOWNSTREAM_BT)
+    def setLimitDownstream(self, value):
+        config.set(prefs.LIMIT_DOWNSTREAM_BT, value)
+    def getLimitDownstreamAmount(self):
+        return config.get(prefs.DOWNSTREAM_BT_LIMIT_IN_KBS)
+    def setLimitDownstreamAmount(self, value):
+        return config.set(prefs.DOWNSTREAM_BT_LIMIT_IN_KBS, value)
     def getMaxManual(self):
         return config.get(prefs.MAX_MANUAL_DOWNLOADS)
     def setMaxManual(self, value):
@@ -372,11 +404,34 @@ class PyBridge:
         return config.get(prefs.STARTUP_TASKS_DONE)
     def setStartupTasksDone(self, value):
         return config.set(prefs.STARTUP_TASKS_DONE, value)
+    def getWarnIfDownloadingOnQuit(self):
+        return config.get(prefs.WARN_IF_DOWNLOADING_ON_QUIT)
+    def setWarnIfDownloadingOnQuit(self, value):
+        return config.set(prefs.WARN_IF_DOWNLOADING_ON_QUIT, value)
+    def getUseUpnp(self):
+        return config.get(prefs.USE_UPNP)
+    def setUseUpnp(self, value):
+        config.set(prefs.USE_UPNP, value)
+    def getBitTorrentEncReq(self):
+        return config.get(prefs.BT_ENC_REQ)
+    def setBitTorrentEncReq(self, value):
+        config.set(prefs.BT_ENC_REQ, value)
 
-    @asUrgent
     def handleCommandLine(self, commandLine):
+        # Here's a massive hack to get command line parameters into config
+        args = getArgumentList(commandLine)
+        for x in range(len(args)-1):
+            if args[x] == '--theme':
+                config.__currentThemeHack = args[x+1]
+                config.load(theme = args[x+1])
+                break
+
         # Doesn't matter if this executes before the call to
         # parseCommandLineArgs in app.py. -clahey
+        self._handleCommandLine(commandLine)
+
+    @asUrgent
+    def _handleCommandLine(self, commandLine):
         singleclick.handleCommandLineArgs(getArgumentList(commandLine))
 
     def pageLoadFinished(self, area, url):
@@ -391,11 +446,13 @@ class PyBridge:
     def setVolume(self, volume):
         volume = float(volume)
         config.set(prefs.VOLUME_LEVEL, volume)
-        app.controller.videoDisplay.setVolume(volume, moveSlider=False)
+        if hasattr(app.controller, 'videoDisplay'):
+            app.controller.videoDisplay.setVolume(volume, moveSlider=False)
 
     @asUrgent
     def quit(self):
-        app.controller.quit()
+        if app.controller.finishedStartup:
+            app.controller.quit()
 
     @asUrgent
     def removeCurrentChannel(self):
@@ -429,6 +486,17 @@ class PyBridge:
     def handleSimpleDialog(self, id, buttonIndex):
         self.delegate.handleDialog(id, buttonIndex)
 
+    @asUrgent
+    def handleCheckboxDialog(self, id, buttonIndex, checkbox_value):
+        self.delegate.handleDialog(id, buttonIndex,
+                checkbox_value=checkbox_value)
+    @asUrgent
+    def handleCheckboxTextboxDialog(self, id, buttonIndex, checkbox_value,
+                                    textbox_value):
+        self.delegate.handleDialog(id, buttonIndex,
+                                   checkbox_value=checkbox_value,
+                                   textbox_value=textbox_value)
+
     @asUrgent
     def handleHTTPAuthDialog(self, id, buttonIndex, username, password):
         self.delegate.handleDialog(id, buttonIndex, username=username,
@@ -441,6 +509,9 @@ class PyBridge:
     @asUrgent
     def handleSearchChannelDialog(self, id, buttonIndex, term, style, loc):
         self.delegate.handleDialog(id, buttonIndex, term=term, style=style, loc=loc)
+    @asUrgent
+    def handleFileDialog(self, id, pathname):
+        self.delegate.handleFileDialog(id, pathname)
 
     @asUrgent
     def addChannel(self, url):
@@ -456,7 +527,8 @@ class PyBridge:
 
     @asUrgent
     def pause(self):
-        app.controller.playbackController.pause()
+        if hasattr(app.controller, 'playbackController'):
+            app.controller.playbackController.pause()
 
     @asUrgent
     def stop(self):
@@ -570,6 +642,18 @@ class PyBridge:
     def createNewChannelGuide(self):
         app.controller.addAndSelectGuide()
 
+    @asUrgent
+    def createNewDownload(self):
+        app.controller.newDownload()
+
+    @asUrgent
+    def importChannels(self):
+        app.controller.importChannels()
+
+    @asUrgent
+    def exportChannels(self):
+        app.controller.exportChannels()
+
     @asUrgent
     def addChannel(self):
         app.controller.addAndSelectFeed()
@@ -580,7 +664,7 @@ class PyBridge:
 
     @asUrgent
     def recommendCurrentChannel(self):
-        print "WARNING: recommendCurrentChannel not supported"
+        app.controller.recommendCurrentFeed()
 
     @asUrgent
     def renameCurrentPlaylist(self):
@@ -600,8 +684,7 @@ class PyBridge:
     def saveVideoFile(self, path):
         if frontend.currentVideoPath is None:
             return
-        print "saving video %s to %s" % (frontend.currentVideoPath, path)
-        shutil.copyfile(frontend.currentVideoPath, path)
+        app.saveVideo(frontend.currentVideoPath, path)
 
     def startupDoSearch(self, path):
         if path.endswith(":"):
@@ -620,18 +703,6 @@ class PyBridge:
     def createProxyObjects(self):
         createProxyObjects()
 
-    @asUrgent
-    def initializeSearchEngines(self):
-        # Send the search engine info to jsbridge.  This is a little tricky
-        # because we need to access views.searchEngines from the main thread.
-        names = []
-        titles = []
-        for engine in views.searchEngines:
-            names.append(engine.name)
-            titles.append(engine.title)
-        frontend.jsBridge.setSearchEngineInfo(names, titles)
-        frontend.jsBridge.setSearchEngine(searchengines.getLastEngine())
-
     def printOut(self, output):
         print output
 
@@ -791,6 +862,8 @@ class PyBridge:
             keyboard.handleKey(key, shiftDown, controlDown)
 
     def handleCloseButton(self):
+        if not app.controller.finishedStartup:
+            return
         if config.get(prefs.MINIMIZE_TO_TRAY_ASK_ON_CLOSE):
             self.askUserForCloseBehaviour()
         elif config.get(prefs.MINIMIZE_TO_TRAY):
