@@ -14,6 +14,17 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+#
+# In addition, as a special exception, the copyright holders give
+# permission to link the code of portions of this program with the OpenSSL
+# library.
+#
+# You must obey the GNU General Public License in all respects for all of
+# the code used other than OpenSSL. If you modify file(s) with this
+# exception, you may extend this exception to your version of the file(s),
+# but you are not obligated to do so. If you do not wish to do so, delete
+# this exception statement from your version. If you delete this exception
+# statement from all source files in the program, then also delete it here.
 
 """Responsible for upgrading old versions of the database.
 
@@ -24,6 +35,10 @@ olddatabaseupgrade.py)
 import schema
 import util
 import types
+from urlparse import urlparse
+from storedatabase import SavableObject
+import config
+import prefs
 
 class DatabaseTooNewError(Exception):
     """Error that we raise when we see a database that is newer than the
@@ -51,7 +66,7 @@ def upgrade(savedObjects, saveVersion, upgradeTo=None):
         upgradeTo = schema.VERSION
 
     if saveVersion > upgradeTo:
-        msg = ("Database was created by a newer version of Democracy " 
+        msg = ("Database was created by a newer version of Miro " 
                "(db version is %s)" % saveVersion)
         raise DatabaseTooNewError(msg)
 
@@ -586,7 +601,8 @@ def upgrade41(objectList):
                 if field == 'actualFeed':
                     o.savedData[field].__dict__ = \
                          unicodify(o.savedData[field].__dict__)
-                elif field == 'iconCache':
+                elif (field == 'iconCache' and
+                      o.savedData['iconCache'] is not None):
                     for icfield in icStrings:
                         o.savedData['iconCache'].savedData[icfield] = \
                           unicodify(o.savedData['iconCache'].savedData[icfield])
@@ -654,7 +670,7 @@ def upgrade43(objectList):
 def upgrade44(objectList):
     changed = set()
     for o in objectList:
-        if 'iconCache' in o.savedData:
+        if 'iconCache' in o.savedData and o.savedData['iconCache'] is not None:
             iconCache = o.savedData['iconCache']
             iconCache.savedData['resized_filenames'] = {}
             changed.add(o)
@@ -727,6 +743,192 @@ def upgrade50(objectList):
                 changed.add(o)
     return changed
 
+def upgrade51(objectList):
+    """Added title field to channel guides"""
+    changed = set()
+    for o in objectList:
+        if o.classString in ('channel-guide'):
+            o.savedData['title'] = None
+            changed.add(o)
+    return changed
+
+def upgrade52(objectList):
+    import filetypes
+    changed = set()
+    removed = set()
+    search_id = 0
+    downloads_id = 0
+
+    def getVideoInfo(o):
+        """Find the first video enclosure in a feedparser entry.  Returns the
+        enclosure, or None if no video enclosure is found.
+        """
+        entry = o.savedData['entry']
+        enc = None
+        try:
+            enclosures = entry.enclosures
+        except (KeyError, AttributeError):
+            pass
+        else:
+            for enclosure in enclosures:
+                if filetypes.isVideoEnclosure(enclosure):
+                    enc = enclosure
+        if enc is not None:
+            url = enc.get('url')
+        else:
+            url = None
+        id = entry.get('id')
+        id = entry.get('guid', id)
+        title = entry.get('title')
+        return (url, id, title)
+
+
+    for o in objectList:
+        if o.classString == 'feed':
+            feedImpl = o.savedData['actualFeed']
+            if feedImpl.classString == 'search-feed-impl':
+                search_id = o.savedData['id']
+            elif feedImpl.classString == 'search-downloads-feed-impl':
+                downloads_id = o.savedData['id']
+
+    items_by_idURL = {}
+    items_by_titleURL = {}
+    if search_id != 0:
+        for o in objectList:
+            if o.classString == 'item':
+                if o.savedData['feed_id'] == search_id:
+                    (url, id, title) = getVideoInfo(o)
+                    if url and id:
+                        items_by_idURL[(id, url)] = o
+                    if url and title:
+                        items_by_titleURL[(title, url)] = o
+    if downloads_id != 0:
+        for i in xrange (len(objectList) - 1, -1, -1):
+            o = objectList [i]
+            if o.classString == 'item':
+                if o.savedData['feed_id'] == downloads_id:
+                    remove = False
+                    (url, id, title) = getVideoInfo(o)
+                    if url and id:
+                        if items_by_idURL.has_key((id, url)):
+                            remove = True
+                        else:
+                            items_by_idURL[(id, url)] = o
+                    if url and title:
+                        if items_by_titleURL.has_key((title, url)):
+                            remove = True
+                        else:
+                            items_by_titleURL[(title, url)] = o
+                    if remove:
+                        removed.add (o.savedData['id'])
+                        changed.add(o)
+                        del objectList[i]
+
+        for i in xrange (len(objectList) - 1, -1, -1):
+            o = objectList [i]
+            if o.classString == 'file-item':
+                if o.savedData['parent_id'] in removed:
+                    changed.add(o)
+                    del objectList[i]
+    return changed
+
+def upgrade53(objectList):
+    """Added favicon and icon cache field to channel guides"""
+    changed = set()
+    for o in objectList:
+        if o.classString in ('channel-guide'):
+            o.savedData['favicon'] = None
+            o.savedData['iconCache'] = None
+            o.savedData['updated_url'] = o.savedData['url']
+            changed.add(o)
+    return changed
+
+def upgrade54(objectList):
+    changed = set()
+    if config.get(prefs.APP_PLATFORM) != "windows-xul":
+        return changed
+    for o in objectList:
+        if o.classString in ('item', 'file-item'):
+            o.savedData['screenshot'] = None
+            o.savedData['duration'] = None
+            changed.add(o)
+    return changed
+
+def upgrade55(objectList):
+    """Add resized_screenshots attribute. """
+    changed = set()
+    for o in objectList:
+        if o.classString in ('item', 'file-item'):
+            o.savedData['resized_screenshots'] = {}
+            changed.add(o)
+    return changed
+
+def upgrade56(objectList):
+    """Added firstTime field to channel guides"""
+    changed = set()
+    for o in objectList:
+        if o.classString in ('channel-guide'):
+            o.savedData['firstTime'] = False
+            changed.add(o)
+    return changed
+
+def upgrade57(objectList):
+    """Added ThemeHistory"""
+    changed = set()
+    return changed
+
+
+def upgrade58(objectList):
+    """clear fastResumeData for libtorrent"""
+    changed = set()
+    for o in objectList:
+        if o.classString == 'remote-downloader':
+            try:
+                o.savedData['status']['fastResumeData'] = None
+                changed.add(o)
+            except:
+                pass
+    return changed
+
+def upgrade59(objectList):
+    """
+    We changed ThemeHistory to allow None in the pastTheme list.  Since we're
+    upgrading, we can assume that the default channels have been added, so
+    we'll add None to that list manually.  We also require a URL for channel
+    guides.  If it's None, eplace it with https://www.miroguide.com/.
+    """
+    changed = set()
+    for o in objectList:
+        if o.classString == 'channel-guide' and o.savedData['url'] is None:
+            o.savedData['url'] = u'https://www.miroguide.com/'
+            changed.add(o)
+        elif o.classString == 'theme-history':
+            if None not in o.savedData['pastThemes']:
+                o.savedData['pastThemes'].append(None)
+                changed.add(o)
+    return changed
+
+def upgrade60(objectList):
+    """search feed impl is now a subclass of rss multi, so add the needed fields"""
+    changed = set()
+    for o in objectList:
+        if o.classString == 'feed':
+            feedImpl = o.savedData['actualFeed']
+            if feedImpl.classString == 'search-feed-impl':
+                feedImpl.savedData['etag'] = {}
+                feedImpl.savedData['modified'] = {}
+                changed.add(o)
+    return changed
+
+def upgrade61(objectList):
+    """Add resized_screenshots attribute. """
+    changed = set()
+    for o in objectList:
+        if o.classString in ('item', 'file-item'):
+            o.savedData['channelTitle'] = None
+            changed.add(o)
+    return changed
+
 #def upgradeX (objectList):
 #    """ upgrade an object list to X.  return set of changed savables. """
 #    changed = set()
