@@ -10,22 +10,23 @@ from threading import RLock, Event, Thread
 import traceback
 from copy import copy
 
-from BitTorrent.bencode import bdecode
+import libtorrent as lt
 from clock import clock
 from download_utils import cleanFilename, nextFreeFilename, checkFilenameExtension, filterDirectoryName
-from download_utils import filenameFromURL
+from download_utils import filenameFromURL, getFileURLPath
 import eventloop
 import httpclient
 import datetime
 import logging
+import fileutil
 
 import config
 import prefs
 from sha import sha
 
-from dl_daemon import command, daemon, bittorrentdtv
+from dl_daemon import command, daemon
 from datetime import timedelta
-from util import checkF, checkU
+from util import checkF, checkU, stringify
 
 import platformutils
 import string
@@ -37,6 +38,9 @@ _downloads = {}
 
 _lock = RLock()
 
+def configReceived ():
+    torrentSession.startup()
+
 def createDownloader(url, contentType, dlid):
     checkU(url)
     checkU(contentType)
@@ -107,8 +111,6 @@ def migrateDownload(dlid, directory):
     else:
         if download.state in (u"finished", u"uploading"):
             download.moveToDirectory(directory)
-            download.updateClient()
-            print download.filename
 
 def getDownloadStatus(dlids = None):
     statuses = {}
@@ -124,6 +126,7 @@ def shutDown():
     logging.info ("Shutting down downloaders...")
     for dlid in _downloads:
         _downloads[dlid].shutdown()
+    torrentSession.shutdown()
 
 def restoreDownloader(downloader):
     downloader = copy(downloader)
@@ -140,6 +143,101 @@ def restoreDownloader(downloader):
 
     _downloads[downloader['dlid']] = dl
 
+class TorrentSession:
+    """Contains the bittorrent session and handles updating all running bittorrents"""
+
+    def __init__(self):
+        self.torrents = set()
+        self.session = None
+        self.pnp_on = None
+        self.pe_set = None
+        self.enc_req = None
+
+    def startup(self):
+        fingerprint = lt.fingerprint("MR", 1, 1, 0, 0)
+        self.session = lt.session(fingerprint)
+        self.listen()
+        self.setUpnp()
+        self.setUploadLimit()
+        self.setDownloadLimit()
+        self.setEncryption()
+        config.addChangeCallback(self.configChanged)
+
+    def listen(self):
+        self.session.listen_on(config.get(prefs.BT_MIN_PORT), config.get(prefs.BT_MAX_PORT))
+
+    def setUpnp(self):
+        useUpnp = config.get(prefs.USE_UPNP)
+        if useUpnp != self.pnp_on:
+            self.pnp_on = useUpnp
+            if useUpnp:
+                self.session.start_upnp()
+            else:
+                self.session.stop_upnp()
+
+    def setUploadLimit(self):
+        limit = -1
+        if config.get(prefs.LIMIT_UPSTREAM):
+            limit = config.get(prefs.UPSTREAM_LIMIT_IN_KBS) * (2 ** 10)
+        self.session.set_upload_rate_limit(limit)
+
+    def setDownloadLimit(self):
+        limit = -1
+        if config.get(prefs.LIMIT_DOWNSTREAM_BT):
+            limit = config.get(prefs.DOWNSTREAM_BT_LIMIT_IN_KBS) * (2 ** 10)
+        self.session.set_download_rate_limit(limit)
+
+    def setEncryption(self):
+        if self.pe_set is None:
+            self.pe_set = lt.pe_settings()
+        enc_req = config.get(prefs.BT_ENC_REQ)
+        if enc_req != self.enc_req:
+            self.enc_req = enc_req
+            if enc_req:
+                self.pe_set.in_enc_policy = lt.enc_policy.forced
+                self.pe_set.out_enc_policy = lt.enc_policy.forced
+            else:
+                self.pe_set.in_enc_policy = lt.enc_policy.enabled
+                self.pe_set.out_enc_policy = lt.enc_policy.enabled
+            self.session.set_pe_settings(self.pe_set)
+
+    def shutdown(self):
+        config.removeChangeCallback(self.configChanged)
+        del self.session
+
+    def configChanged(self, key, value):
+        if key == prefs.BT_MIN_PORT.key:
+            if value > self.session.listen_port():
+                self.listen()
+        if key == prefs.BT_MAX_PORT.key:
+            if value < self.session.listen_port():
+                self.listen()
+        if key == prefs.USE_UPNP.key:
+            self.setUpnp()
+        if key in (prefs.LIMIT_UPSTREAM.key, prefs.UPSTREAM_LIMIT_IN_KBS.key):
+            self.setUploadLimit()
+        if key in (prefs.LIMIT_DOWNSTREAM_BT.key, prefs.DOWNSTREAM_BT_LIMIT_IN_KBS.key):
+            self.setDownloadLimit()
+        if key == prefs.BT_ENC_REQ.key:
+            self.setEncryption()
+
+    def addTorrent(self, torrent):
+        self.torrents.add(torrent)
+
+    def removeTorrent(self, torrent):
+        try:
+            self.torrents.remove(torrent)
+        except:
+            pass
+
+    def updateTorrents(self):
+        # Copy this set into a list in case any of the torrents gets removed during the iteration.
+        for torrent in [x for x in self.torrents]:
+            torrent.updateStatus()
+        
+
+torrentSession = TorrentSession()
+
 class DownloadStatusUpdater:
     """Handles updating status for all in progress downloaders.
 
@@ -169,6 +267,7 @@ class DownloadStatusUpdater:
 
     def doUpdate(self):
         try:
+            torrentSession.updateTorrents()
             statuses = []
             for downloader in self.toUpdate:
                 statuses.append(downloader.getStatus())
@@ -254,8 +353,8 @@ class BGDownloader:
             os.makedirs(downloadDir)
         except:
             pass
-        baseFilename = os.path.join(downloadDir, self.shortFilename+".part")
-        self.filename = nextFreeFilename(cleanFilename(baseFilename))
+        cleaned = cleanFilename(self.shortFilename+".part")
+        self.filename = nextFreeFilename(os.path.join(downloadDir, cleaned))
 
     def moveToMoviesDirectory(self):
         """Move our downloaded file from the Incomplete Downloads directoy to
@@ -263,7 +362,7 @@ class BGDownloader:
         """
         if chatter:
             logging.info ("moving to movies directory filename is %s", self.filename)
-        self.moveToDirectory(cleanFilename(config.get(prefs.MOVIES_DIRECTORY)))
+        self.moveToDirectory(config.get(prefs.MOVIES_DIRECTORY))
 
     def moveToDirectory (self, directory):
         checkF(directory)
@@ -278,17 +377,10 @@ class BGDownloader:
         if newfilename == self.filename:
             return
         newfilename = nextFreeFilename(newfilename)
-        try:
-            shutil.move(self.filename, newfilename)
-        except (IOError, OSError, shutil.Error), e:
-            logging.info ("WARNING: Error moving %s to %s (%s)", self.filename,
-                          newfilename, e)
-        except TypeError, e:
-            logging.warning ("Error moving %s (%s) to %s (%s) (%s)", self.filename, type(self.filename),
-                             newfilename, type(newfilename), e)
-        else:
+        def callback():
             self.filename = newfilename
-            logging.info ("new file name is %s", self.filename)
+            self.updateClient()
+        fileutil.migrate_file(self.filename, newfilename, callback)
 
     ##
     # Returns a float with the estimated number of seconds left
@@ -366,6 +458,19 @@ class BGDownloader:
     def handleGenericError(self, longDescription):
         self.handleError(_("Error"), longDescription)
 
+    ##
+    # Checks the download file size to see if we can accept it based on the 
+    # user disk space preservation preference
+    def acceptDownloadSize(self, size):
+        accept = True
+        if config.get(prefs.PRESERVE_DISK_SPACE):
+            if size < 0:
+                size = 0
+            preserved = config.get(prefs.PRESERVE_X_GB_FREE) * 1024 * 1024 * 1024
+            available = platformutils.getAvailableBytesForMovies() - preserved
+            accept = (size <= available)
+        return accept
+
 
 class HTTPDownloader(BGDownloader):
     UPDATE_CLIENT_WINDOW = 12
@@ -437,7 +542,8 @@ class HTTPDownloader(BGDownloader):
         self.cancelRequest()
 
     def handleWriteError(self, error):
-        self.handleGenericError(_("Could not write to %s") % self.filename)
+        self.handleGenericError(_("Could not write to %s") % 
+                                  stringify(self.filename))
         if self.filehandle is not None:
             try:
                 self.filehandle.close()
@@ -470,7 +576,7 @@ class HTTPDownloader(BGDownloader):
             self.filehandle = file(self.filename,"w+b")
         except IOError:
             self.handleGenericError("Couldn't open %s for writing" % 
-                self.filename)
+                stringify(self.filename))
             return
         if self.totalSize > 0:
             try:
@@ -488,7 +594,12 @@ class HTTPDownloader(BGDownloader):
             self.currentSize = 0
             self.totalSize = -1
             self.resetBlockTimes()
-            return self.onHeaders(info)
+            if not self.client.gotBadStatusCode:
+                self.onHeaders(info)
+            else:
+                self.cancelRequest()
+                self.startNewDownload()
+            return
         try:
             self.parseContentRange(info['content-range'])
         except ValueError:
@@ -580,19 +691,6 @@ class HTTPDownloader(BGDownloader):
                 break
         self.blockTimes = self.blockTimes[i:]
 
-    ##
-    # Checks the download file size to see if we can accept it based on the 
-    # user disk space preservation preference
-    def acceptDownloadSize(self, size):
-        accept = True
-        if config.get(prefs.PRESERVE_DISK_SPACE):
-            if size < 0:
-                size = 0
-            preserved = config.get(prefs.PRESERVE_X_GB_FREE) * 1024 * 1024 * 1024
-            available = platformutils.getAvailableBytesForMovies() - preserved
-            accept = (size <= available)
-        return accept
-
     ##
     # Pauses the download.
     def pause(self):
@@ -653,27 +751,69 @@ class BTDownloader(BGDownloader):
         self.channelName = None
         self.uploadedStart = 0
         self.restarting = False
+        self.seeders = -1
+        self.leechers = -1
         if restore is not None:
+            self.firstTime = False
             self.restoreState(restore)
         else:
+            self.firstTime = True
             BGDownloader.__init__(self,url,item)
             self.runDownloader()
 
+    def _startTorrent(self):
+        try:
+            torrent_info = lt.torrent_info(lt.bdecode(self.metainfo))
+            self.totalSize = torrent_info.total_size()
+
+            if self.firstTime and not self.acceptDownloadSize(self.totalSize):
+                self.handleError(_("Not enough disk space"),
+                                 _("%s MB required to store this video") % 
+                                 (self.totalSize / (2 ** 20)))
+                return
+
+            name = stringify(self.filename)
+            if self.fastResumeData:
+                resume = lt.bdecode(self.fastResumeData)
+                self.torrent = torrentSession.session.add_torrent(torrent_info, name, lt.bdecode(self.fastResumeData), lt.storage_mode_t.storage_mode_allocate)
+            else:
+                self.torrent = torrentSession.session.add_torrent(torrent_info, name, None, lt.storage_mode_t.storage_mode_allocate)
+        except:
+            self.handleError(_('BitTorrent failure'), 
+                    _('BitTorrent failed to startup'))
+        else:
+            torrentSession.addTorrent (self)
+
     def _shutdownTorrent(self):
         try:
+            torrentSession.removeTorrent (self)
             if self.torrent is not None:
-                self.fastResumeData = self.torrent.shutdown()
+                self.torrent.pause()
+                self.fastResumeData = lt.bencode(self.torrent.write_resume_data())
+                torrentSession.session.remove_torrent(self.torrent, 0)
+                self.torrent = None
         except:
             logging.exception ("DTV: Warning: Error shutting down torrent")
 
-    def _startTorrent(self):
-        self.torrent = bittorrentdtv.TorrentDownload(self.metainfo,
-                self.filename, self.fastResumeData)
-        self.torrent.set_status_callback(self.updateStatus)
-        self.torrent.start()
+    def _pauseTorrent(self):
+        try:
+            torrentSession.removeTorrent (self)
+            if self.torrent is not None:
+                self.torrent.pause()
+        except:
+            logging.exception ("DTV: Warning: Error pausing torrent")
 
-    @eventloop.asIdle
-    def updateStatus(self, newStatus):
+    def _resumeTorrent(self):
+        if self.torrent is not None:
+            try:
+                self.torrent.resume()
+                torrentSession.addTorrent (self)
+            except:
+                logging.exception ("DTV: Warning: Error resuming torrent")
+        else:
+            self._startTorrent()
+
+    def updateStatus(self):
         """
         activity -- string specifying what's currently happening or None for
                 normal operations.  
@@ -686,14 +826,29 @@ class BTDownloader(BGDownloader):
         totalSize -- total size of the torrent in bytes
         """
 
-        self.totalSize = newStatus['totalSize']
-        self.rate = newStatus['downRate']
-        self.upRate = newStatus['upRate']
-        self.uploaded = newStatus['upTotal'] + self.uploadedStart
-        self.eta = newStatus['timeEst']
-        self.activity = newStatus['activity']
-        self.currentSize = int(self.totalSize * newStatus['fractionDone'])
-        if self.state == "downloading" and newStatus['fractionDone'] == 1.0:
+        status = self.torrent.status()
+        self.totalSize = status.total_wanted
+        self.rate = status.download_payload_rate
+        self.upRate = status.upload_payload_rate
+        self.uploaded = status.total_payload_upload + self.uploadedStart
+        self.seeders = status.num_complete
+        self.leechers = status.num_incomplete
+        try:
+            self.eta = (status.total_wanted - status.total_wanted_done) / float (status.download_payload_rate)
+        except ZeroDivisionError:
+            self.eta = 0
+        if status.state == lt.torrent_status.states.queued_for_checking:
+            self.activity = "waiting to check existing files"
+        elif status.state == lt.torrent_status.states.checking_files:
+            self.activity = "checking existing files"
+        elif status.state == lt.torrent_status.states.connecting_to_tracker:
+            self.activity = "connecting to peers"
+        elif status.state == lt.torrent_status.states.allocating:
+            self.activity = "allocating disk space"
+        else:
+            self.activity = None
+        self.currentSize = status.total_wanted_done
+        if self.state == "downloading" and status.state == lt.torrent_status.states.seeding:
             self.moveToMoviesDirectory()
             self.state = "uploading"
             self.endTime = clock()
@@ -713,7 +868,7 @@ class BTDownloader(BGDownloader):
         if self.state in ('uploading', 'downloading'):
             self._shutdownTorrent()
             BGDownloader.moveToDirectory(self, directory)
-            self._startTorrent()
+            self._resumeTorrent()
         else:
             BGDownloader.moveToDirectory(self, directory)
 
@@ -735,6 +890,8 @@ class BTDownloader(BGDownloader):
         data['fastResumeData'] = self.fastResumeData
         data['activity'] = self.activity
         data['dlerType'] = 'BitTorrent'
+        data['seeders'] = self.seeders
+        data['leechers'] = self.leechers
         return data
 
     def getRate(self):
@@ -745,7 +902,7 @@ class BTDownloader(BGDownloader):
         
     def pause(self):
         self.state = "paused"
-        self._shutdownTorrent()
+        self._pauseTorrent()
         self.updateClient()
 
     def stop(self, delete):
@@ -787,17 +944,17 @@ class BTDownloader(BGDownloader):
         #        because it only affects the incomplete filename
         if not self.restarting:
             try:
-                metainfo = bdecode(self.metainfo)
+                metainfo = lt.bdecode(self.metainfo)
                 name = metainfo['info']['name']
-            except (ValueError, KeyError):
+            except (RuntimeError):
                 self.handleError(_("Corrupt Torrent"),
-                        _("The torrent file at %s was not valid") % self.url)
+                        _("The torrent file at %s was not valid") % stringify(self.url))
                 return
             name = name.decode('utf-8', 'replace')
             self.shortFilename = cleanFilename(name)
             self.pickInitialFilename()
         self.updateClient()
-        self._startTorrent()
+        self._resumeTorrent()
 
     def handleMetainfo(self, metainfo):
         self.metainfo = metainfo
@@ -812,14 +969,13 @@ class BTDownloader(BGDownloader):
     def getMetainfo(self):
         if self.metainfo is None:
             if self.url.startswith('file://'):
-                path = self.url[len('file://'):]
-                path = platformutils.unmakeURLSafe(path)
+                path = getFileURLPath(self.url)
                 try:
                     metainfoFile = open(path, 'rb')
                 except IOError:
                     self.handleError(_("Torrent file deleted"),
                             _("The torrent file for this item was deleted "
-                                "outside of democracy."))
+                                "outside of Miro."))
 
                     return
                 try:
