@@ -14,6 +14,17 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+#
+# In addition, as a special exception, the copyright holders give
+# permission to link the code of portions of this program with the OpenSSL
+# library.
+#
+# You must obey the GNU General Public License in all respects for all of
+# the code used other than OpenSSL. If you modify file(s) with this
+# exception, you may extend this exception to your version of the file(s),
+# but you are not obligated to do so. If you do not wish to do so, delete
+# this exception statement from your version. If you delete this exception
+# statement from all source files in the program, then also delete it here.
 
 from base64 import b64encode
 from gtcache import gettext as _
@@ -24,8 +35,8 @@ import shutil
 
 from database import DDBObject, defaultDatabase
 from dl_daemon import daemon, command
-from download_utils import nextFreeFilename, parseURL, filterDirectoryName
-from util import getTorrentInfoHash, returnsUnicode, checkU, returnsFilename, unicodify, checkF
+from download_utils import nextFreeFilename, getFileURLPath, filterDirectoryName
+from util import getTorrentInfoHash, returnsUnicode, checkU, returnsFilename, unicodify, checkF, stringify
 from platformutils import FilenameType
 import app
 import config
@@ -38,6 +49,8 @@ import platformutils
 import flashscraper
 import logging
 import traceback
+import templatehelper
+import fileutil
 
 # a hash of download ids that the server knows about.
 _downloads = {}
@@ -107,7 +120,7 @@ def generateDownloadID():
 class RemoteDownloader(DDBObject):
     """Download a file using the downloader daemon."""
 
-    def __init__(self, url, item, contentType = None):
+    def __init__(self, url, item, contentType = None, channelName = None):
         checkU(url)
         if contentType:
             checkU(contentType)
@@ -125,7 +138,7 @@ class RemoteDownloader(DDBObject):
                 contentType = enclosureContentType
         self.contentType = u""
         self.deleteFiles = True
-        self.channelName = None
+        self.channelName = channelName
         self.manualUpload = False
         DDBObject.__init__(self)
         if contentType is None:
@@ -184,6 +197,20 @@ class RemoteDownloader(DDBObject):
             return (0, self.status.get('upRate', 0))
         return (0, 0)
 
+    def beforeChangingStatus(self):
+        global totalDownRate
+        global totalUpRate
+        rates = self._getRates()
+        totalDownRate -= rates[0]
+        totalUpRate -= rates[1]
+
+    def afterChangingStatus(self):
+        global totalDownRate
+        global totalUpRate
+        rates = self._getRates()
+        totalDownRate += rates[0]
+        totalUpRate += rates[1]
+
     @classmethod
     def updateStatus(cls, data):
         for field in data:
@@ -201,11 +228,7 @@ class RemoteDownloader(DDBObject):
                 print "WARNING exception when comparing status: %s" % e
 
             wasFinished = self.isFinished()
-            global totalDownRate
-            global totalUpRate
-            rates = self._getRates()
-            totalDownRate -= rates[0]
-            totalUpRate -= rates[1]
+            self.beforeChangingStatus()
 
             # FIXME: how do we get all of the possible bit torrent
             # activity strings into gettext? --NN
@@ -216,9 +239,7 @@ class RemoteDownloader(DDBObject):
 
             # Store the time the download finished
             finished = self.isFinished() and not wasFinished
-            rates = self._getRates()
-            totalDownRate += rates[0]
-            totalUpRate += rates[1]
+            self.afterChangingStatus()
 
             if self.getState() == u'uploading' and not self.manualUpload and self.getUploadRatio() > 1.5:
                 self.stopUpload()
@@ -258,7 +279,9 @@ class RemoteDownloader(DDBObject):
                                              self.dlid)
             c.send()
         else:
+            self.beforeChangingStatus()
             self.status["state"] = u"paused"
+            self.afterChangingStatus()
             self.signalChange()
 
     ##
@@ -283,25 +306,22 @@ class RemoteDownloader(DDBObject):
         except KeyError:
             return
         try:
-            if os.path.isfile(filename):
-                os.remove (filename)
-            elif os.path.isdir(filename):
-                shutil.rmtree (filename)
+            fileutil.delete(filename)
         except:
             logging.warn("Error deleting downloaded file: %s\n%s" % 
-                    (filename, traceback.format_exc()))
+                    (templatehelper.toUni(stringify(filename)), traceback.format_exc()))
 
         parent = os.path.join(filename, os.path.pardir)
         parent = os.path.normpath(parent)
         moviesDir = config.get(prefs.MOVIES_DIRECTORY)
-        if (os.path.exists(parent) and
+        if (os.path.exists(parent) and os.path.exists(moviesDir) and
             not platformutils.samefile(parent, moviesDir) and
             len(os.listdir(parent)) == 0):
             try:
                 os.rmdir(parent)
             except:
                 logging.warn("Error deleting empty download directory: %s\n%s" %
-                        (parent, traceback.format_exc()))
+                        (templatehelper.toUni(parent), traceback.format_exc()))
 
     ##
     # Continues a paused, stopped, or failed download thread
@@ -311,7 +331,9 @@ class RemoteDownloader(DDBObject):
                 del _downloads[self.dlid]
             self.dlid = generateDownloadID()
             views.remoteDownloads.recomputeIndex(indexes.downloadsByDLID)
+            self.beforeChangingStatus()
             self.status = {}
+            self.afterChangingStatus()
             if self.contentType == u"":
                 self.getContentType()
             else:
@@ -360,14 +382,10 @@ URL was %s""" % self.url
                 if newfilename == filename:
                     return
                 newfilename = nextFreeFilename(newfilename)
-                try:
-                    shutil.move(filename, newfilename)
-                except (IOError, OSError), error:
-                    print "WARNING: Error moving %s to %s (%s)" % (self.status['filename'],
-                            newfilename, error)
-                else:
+                def callback():
                     self.status['filename'] = newfilename
-            self.signalChange()
+                    self.signalChange()
+                fileutil.migrate_file(filename, newfilename, callback)
         for i in self.itemList:
             i.migrateChildren(directory)
 
@@ -500,7 +518,7 @@ URL was %s""" % self.url
         size = self.getCurrentSize()
         if size == 0:
             return 0
-        return self.status.get('uploaded', 0) * 1024 * 1024 / size
+        return self.status.get('uploaded', 0) / size
     
     def restartIfNeeded(self):
         if self.getState() in (u'downloading',u'offline'):
@@ -532,7 +550,9 @@ URL was %s""" % self.url
                                              self.dlid)
             c.send()
         else:
+            self.beforeChangingStatus()
             self.status['state'] = u'uploading'
+            self.afterChangingStatus()
             self.restart()
             self.signalChange()
 
@@ -544,7 +564,9 @@ URL was %s""" % self.url
                                           self.dlid)
             c.send()
             del _downloads[self.dlid]
+        self.beforeChangingStatus()
         self.status["state"] = u"finished"
+        self.afterChangingStatus()
         self.signalChange()
 
 def cleanupIncompleteDownloads():
@@ -563,16 +585,17 @@ def cleanupIncompleteDownloads():
                     filename = os.path.join(downloadDir, filename)
                 filesInUse.add(filename)
 
-    for file in os.listdir(downloadDir):
-        file = os.path.join(downloadDir, file)
-        if file not in filesInUse:
+    for f in os.listdir(downloadDir):
+        f = os.path.join(downloadDir, f)
+        if f not in filesInUse:
             try:
-                if os.path.isfile(file):
-                    os.remove (file)
-                elif os.path.isdir(file):
-                    shutil.rmtree (file)
+                if os.path.isfile(f):
+                    os.remove (f)
+                elif os.path.isdir(f):
+                    shutil.rmtree (f)
             except:
-                pass # maybe a permissions error?  
+                # FIXME - maybe a permissions error?
+                pass
 
 def restartDownloads():
     views.remoteDownloads.confirmDBThread()
@@ -609,11 +632,16 @@ def startupDownloader():
     restartDownloads()
 
 def shutdownDownloader(callback = None):
-    RemoteDownloader.dldaemon.shutdownDownloaderDaemon(callback=callback)
+    if hasattr(RemoteDownloader, 'dldaemon') and RemoteDownloader.dldaemon is not None:
+        RemoteDownloader.dldaemon.shutdownDownloaderDaemon(callback=callback)
 
 def lookupDownloader(url):
     return views.remoteDownloads.getItemWithIndex(indexes.downloadsByURL, url)
 
+def getExistingDownloaderByURL(url):
+    downloader = lookupDownloader(url)
+    return downloader
+
 def getExistingDownloader(item):
     downloader = lookupDownloader(item.getURL())
     if downloader:
@@ -625,17 +653,18 @@ def getDownloader(item):
     if existing:
         return existing
     url = item.getURL()
+    channelName = platformutils.unicodeToFilename(item.getChannelTitle(True))
+    if not channelName:
+        channelName = None
     if url.startswith(u'file://'):
-        scheme, host, port, path = parseURL(url)
-        path = platformutils.unmakeURLSafe (path)
-        if re.match(r'/[a-zA-Z]:', path): 
-            # fix windows pathnames (/C:/blah/blah/blah)
-            path = path[1:]
+        path = getFileURLPath(url)
         try:
             getTorrentInfoHash(path)
         except ValueError:
             raise ValueError("Don't know how to handle %s" % url)
+        except IOError:
+            return None
         else:
-            return RemoteDownloader(url, item, u'application/x-bittorrent')
+            return RemoteDownloader(url, item, u'application/x-bittorrent', channelName=channelName)
     else:
-        return RemoteDownloader(url, item)
+        return RemoteDownloader(url, item, channelName=channelName)
