@@ -14,6 +14,17 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+#
+# In addition, as a special exception, the copyright holders give
+# permission to link the code of portions of this program with the OpenSSL
+# library.
+#
+# You must obey the GNU General Public License in all respects for all of
+# the code used other than OpenSSL. If you modify file(s) with this
+# exception, you may extend this exception to your version of the file(s),
+# but you are not obligated to do so. If you do not wish to do so, delete
+# this exception statement from your version. If you delete this exception
+# statement from all source files in the program, then also delete it here.
 
 """httpclient.py 
 
@@ -27,6 +38,7 @@ reference (http://www.w3.org/Protocols/rfc2616/rfc2616.html).
 """
 
 import errno
+import logging
 import re
 import socket
 import traceback
@@ -40,7 +52,7 @@ from clock import clock
 import httpauth
 import config
 import prefs
-from download_utils import URIPattern, cleanFilename, parseURL, defaultPort
+from download_utils import URIPattern, cleanFilename, parseURL, defaultPort, getFileURLPath, filenameFromURL
 from xhtmltools import URLEncodeDict, multipartEncode
 import eventloop
 import util
@@ -80,7 +92,8 @@ class NetworkError(Exception):
 class ConnectionError(NetworkError):
     def __init__(self, errorMessage):
         self.friendlyDescription = _("Can't connect")
-        self.longDescription = _("Connection Error: %s") % errorMessage
+        self.longDescription = _("Connection Error: %s") % \
+                               util.stringify(errorMessage, "replace")
 
 class SSLConnectionError(ConnectionError):
     def __init__(self):
@@ -92,16 +105,16 @@ class HTTPError(NetworkError):
         NetworkError.__init__(self, _("HTTP error"), longDescription)
 class BadStatusLine(HTTPError):
     def __init__(self, line):
-        HTTPError.__init__(self, _("Bad Status Line: %s" % line))
+        HTTPError.__init__(self, _("Bad Status Line: %s") % line)
 class BadHeaderLine(HTTPError):
     def __init__(self, line):
-        HTTPError.__init__(self, _("Bad Header Line: %s" % line))
+        HTTPError.__init__(self, _("Bad Header Line: %s") % line)
 class BadChunkSize(HTTPError):
     def __init__(self, line):
-        HTTPError.__init__(self, _("Bad Chunk size: %s" % line))
+        HTTPError.__init__(self, _("Bad Chunk size: %s") % line)
 class CRLFExpected(HTTPError):
     def __init__(self, crlf):
-        HTTPError.__init__(self, _("Expected CRLF got: %r" % crlf))
+        HTTPError.__init__(self, _("Expected CRLF got: %r") % crlf)
 class ServerClosedConnection(HTTPError):
     def __init__(self, host):
         HTTPError.__init__(self, _('%s closed connection') % host)
@@ -112,7 +125,7 @@ class UnexpectedStatusCode(HTTPError):
             self.friendlyDescription = _("File not found")
             self.longDescription = _("Got 404 status code")
         else:
-            HTTPError.__init__(self, _("Bad Status Code: %s" % code))
+            HTTPError.__init__(self, _("Bad Status Code: %s") % code)
 
 class AuthorizationFailed(NetworkError):
     def __init__(self):
@@ -132,7 +145,18 @@ class ConnectionTimeout(NetworkError):
 class MalformedURL(NetworkError):
     def __init__(self, url):
         NetworkError.__init__(self, _('Invalid URL'),
-                _('%s is not a valid URL') % url)
+                _('"%s" is not a valid URL') % url)
+
+class FileURLNotFoundError(NetworkError):
+    """A file: URL doesn't exist"""
+    def __init__(self, path):
+        NetworkError.__init__(self, _('File not found'),
+            _('The file: "%s" doesn\'t exist') % path)
+
+class FileURLReadError(NetworkError):
+    def __init__(self, path):
+        NetworkError.__init__(self, _('Read error'),
+            _('Error while reading from "%s"') % path)
 
 def trapCall(object, function, *args, **kwargs):
     """Convenience function do a util.trapCall, where when = 'While talking to
@@ -140,6 +164,33 @@ def trapCall(object, function, *args, **kwargs):
     """
     return util.timeTrapCall("Calling %s on %s" % (function, object), function, *args, **kwargs)
 
+
+DATEINFUTURE = time.mktime( (2030, 7, 12, 12, 0, 0, 4, 193, -1) )
+
+def get_cookie_expiration_date(val):
+    """Tries a bunch of possible cookie expiration date formats
+    until it finds the magic one (or doesn't and returns 0).
+    """
+    fmts = ( '%a, %d %b %Y %H:%M:%S %Z',
+             '%a, %d %b %y %H:%M:%S %Z',
+             '%a, %d-%b-%Y %H:%M:%S %Z',
+             '%a, %d-%b-%y %H:%M:%S %Z' )
+    
+    for fmt in fmts:
+        try:
+            return time.mktime(time.strptime(val, fmt))
+        except OverflowError, oe:
+            # an overflow error means the cookie expiration is far in the
+            # future.  so we return a date that's not so far in the
+            # future.
+            return DATEINFUTURE
+        except ValueError, ve:
+            pass
+
+    print "DTV: Warning: Can't process cookie expiration: '%s'" % val
+    return 0
+
+
 class NetworkBuffer(object):
     """Responsible for storing incomming network data and doing some basic
     parsing of it.  I think this is about as fast as we can do things in pure
@@ -257,14 +308,19 @@ class AsyncSocket(object):
             self.readTimeout.cancel()
             self.readTimeout = None
 
-    def openConnection(self, host, port, callback, errback):
+    def openConnection(self, host, port, callback, errback, disableReadTimeout = None):
         """Open a connection.  On success, callback will be called with this
         object.
         """
-
+        if disableReadTimeout is not None:
+            self.disableReadTimeout = disableReadTimeout
         self.name = "Outgoing %s:%s" % (host, port)
 
-        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        try:
+            self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        except socket.error, e:
+            trapCall(self, errback, ConnectionError(e[1]))
+            return
         self.socket.setblocking(0)
         self.connectionErrback = errback
         def handleGetHostByNameException(e):
@@ -293,7 +349,7 @@ class AsyncSocket(object):
             if rv == 0:
                 trapCall(self, callback, self)
             else:
-                msg = errno.errorcode[rv]
+                msg = errno.errorcode.get(rv, _('Unknown Error code'))
                 trapCall(self, errback, ConnectionError(msg))
             self.connectionErrback = None
         def onWriteTimeout():
@@ -455,7 +511,7 @@ class AsyncSSLStream(AsyncSocket):
         super(AsyncSSLStream, self).__init__(closeCallback)
         self.interruptedOperation = None
 
-    def openConnection(self, host, port, callback, errback):
+    def openConnection(self, host, port, callback, errback, disableReadTimeout = None):
         def onSocketOpen(self):
             self.socket.setblocking(1)
             eventloop.callInThread(onSSLOpen, handleSSLError, socket.ssl,
@@ -472,7 +528,7 @@ class AsyncSSLStream(AsyncSocket):
         def handleSSLError(error):
             errback(SSLConnectionError())
         super(AsyncSSLStream, self).openConnection(host, port, onSocketOpen,
-                errback)
+                errback, disableReadTimeout)
 
     def resumeNormalCallbacks(self):
         if self.readCallback is not None:
@@ -531,7 +587,7 @@ class AsyncSSLStream(AsyncSocket):
             self.handleReadData(data)
 
 class ProxiedAsyncSSLStream(AsyncSSLStream):
-    def openConnection(self, host, port, callback, errback):
+    def openConnection(self, host, port, callback, errback, disableReadTimeout):
         def onSocketOpen(self):
             self.socket.setblocking(1)
             eventloop.callInThread(onSSLOpen, handleSSLError, lambda: openProxyConnection(self),
@@ -580,7 +636,7 @@ class ProxiedAsyncSSLStream(AsyncSSLStream):
         proxy_host = config.get(prefs.HTTP_PROXY_HOST)
         proxy_port = config.get(prefs.HTTP_PROXY_PORT)
         AsyncSocket.openConnection(self, proxy_host, proxy_port, onSocketOpen,
-                errback)
+                errback, disableReadTimeout)
     
     
 class ConnectionHandler(object):
@@ -611,14 +667,14 @@ class ConnectionHandler(object):
     def __str__(self):
         return "%s -- %s" % (self.__class__, self.state)
 
-    def openConnection(self, host, port, callback, errback):
+    def openConnection(self, host, port, callback, errback, disableReadTimeout=None):
         self.name = "Outgoing %s:%s" % (host, port)
         self.host = host
         self.port = port
         def callbackIntercept(asyncSocket):
             if callback:
                 trapCall(self, callback, self)
-        self.stream.openConnection(host, port, callbackIntercept, errback)
+        self.stream.openConnection(host, port, callbackIntercept, errback, disableReadTimeout)
 
     def closeConnection(self):
         if self.stream.isOpen():
@@ -770,7 +826,7 @@ class HTTPConnection(ConnectionHandler):
         elif method == "POST" and postFiles is not None:
             (postData, boundary) = multipartEncode(postVariables, postFiles)
             headers['Content-Type'] = 'multipart/form-data; boundary=%s' % boundary
-            headers['Content-Length'] = '%d' % len(postData)            
+            headers['Content-Length'] = '%d' % len(postData)
         else:
             postData = None
 
@@ -1181,14 +1237,25 @@ class HTTPConnectionPool(object):
         elif conn in conns['free']:
             conns['free'].remove(conn)
             self.freeConnectionCount -= 1
+        else:
+            logging.warn("_onConnectionClosed called with connection not "
+                    "in either queue")
         self.runPendingRequests()
 
     def _onConnectionReady(self, conn):
         conns = self._getServerConnections(conn.scheme, conn.host, conn.port)
-        conns['active'].remove(conn)
-        self.activeConnectionCount -= 1
-        conns['free'].add(conn)
-        self.freeConnectionCount += 1
+        if conn in conns['active']:
+            conns['active'].remove(conn)
+            self.activeConnectionCount -= 1
+        else:
+            logging.warn("_onConnectionReady called with connection not "
+                    "in the active queue")
+        if conn not in conns['free']:
+            conns['free'].add(conn)
+            self.freeConnectionCount += 1
+        else:
+            logging.warn("_onConnectionReady called with connection already "
+                    "in the free queue")
         self.runPendingRequests()
 
     def addRequest(self, callback, errback, requestStartCallback,
@@ -1273,6 +1340,7 @@ class HTTPConnectionPool(object):
                 self._dropAFreeConnection()
 
     def _makeNewConnection(self, req):
+        disableReadTimeout = req['postFiles'] is not None
         def openConnectionCallback(conn):
             conn.sendRequest(req['callback'], req['errback'],
                              req['host'], req['port'],
@@ -1317,12 +1385,14 @@ class HTTPConnectionPool(object):
         if req['proxy_host']:
             eventloop.addIdle(lambda : conn.openConnection(req['proxy_host'],
                                                            req['proxy_port'],
-                         openConnectionCallback, openConnectionErrback),
+                         openConnectionCallback, openConnectionErrback,
+                         disableReadTimeout),
                           "Open connection %s" % str(self))
         else:
             eventloop.addIdle(lambda : conn.openConnection(req['host'],
                                                            req['port'],
-                         openConnectionCallback, openConnectionErrback),
+                         openConnectionCallback, openConnectionErrback,
+                         disableReadTimeout),
                           "Open connection %s" % str(self))
         return conn
 
@@ -1525,7 +1595,6 @@ class HTTPClient(object):
             self.handleAuthorize(response)
         else:
             self.connection = None
-            expectedStatusCodes = [200]
             if not self.gotBadStatusCode:
                 if self.callback:
                     response = self.prepareResponse(response)
@@ -1589,30 +1658,6 @@ class HTTPClient(object):
             traceback.print_exc()
         return response
 
-    def getCookieExpirationDate(self, val):
-        """Tries a bunch of possible cookie expiration date formats
-        until it finds the magic one (or doesn't and returns 0).
-        """
-        fmts = ( '%a, %d %b %Y %H:%M:%S %Z',
-                 '%a, %d %b %y %H:%M:%S %Z',
-                 '%a, %d-%b-%Y %H:%M:%S %Z',
-                 '%a, %d-%b-%y %H:%M:%S %Z' )
-
-        for fmt in fmts:
-            try:
-                return time.mktime(time.strptime(val, fmt))
-            except OverflowError, oe:
-                # an overflow error means the cookie expiration is far in the
-                # future.  so we return a date that's not so far in the
-                # future.
-                return time.mktime( (2030, 7, 12, 12, 0, 0, 4, 193, -1) )
-            except ValueError, ve:
-                pass
-
-        print "DTV: Warning: Can't process cookie expiration: '%s'" % val
-        return 0
-
-
     def getCookiesFromResponse(self, response):
         """Generates a cookie dictionary from headers in response
         """
@@ -1688,7 +1733,7 @@ class HTTPClient(object):
                         # FIXME: "expires" isn't very well defined and
                         # this code will probably puke in certain cases
                         cookieval = getAttrPair(attr)[1].strip()
-                        expires = self.getCookieExpirationDate(cookieval)
+                        expires = get_cookie_expiration_date(cookieval)
                         
                         expires -= time.timezone
                         if expires < now:
@@ -1736,21 +1781,7 @@ class HTTPClient(object):
             filename = self.findValueFromHeader(disposition, 'filename')
             if filename is not None:
                 return cleanFilename(filename)
-        match = URIPattern.match(response['path'])
-        if match is None:
-            # This code path will never be executed.
-            return cleanFilename(response['path'])
-        filename = match.group(2)
-        query = match.group(4)
-        if not filename:
-            ret = query
-        elif not query:
-            ret = filename
-        else:
-            ret = u"%s-%s" % (filename, query)
-        if ret is None:
-            ret = u'unknown'
-        return cleanFilename(ret)
+        return filenameFromURL(util.unicodify(response['redirected-url']), clean=True)
 
     def getCharsetFromResponse(self, response):
         try:
@@ -1818,11 +1849,22 @@ def grabURL(url, callback, errback, headerCallback=None,
     if cookies == None:
         cookies = {}
     if url.startswith("file://"):
-        callback({"body":file(url[7:]).read(),
-                      "updated-url":url,
-                      "redirected-url":url,
-                      "content-type": defaultMimeType,
-                   })
+        path = getFileURLPath(url)
+        try:
+            f = file(path)
+        except EnvironmentError:
+            errback(FileURLNotFoundError(path))
+        else:
+            try:
+                data = f.read()
+            except:
+                errback(FileURLReadError(path))
+            else:
+                callback({"body": data,
+                              "updated-url":url,
+                              "redirected-url":url,
+                              "content-type": defaultMimeType,
+                           })
     else:
         client = clientClass(url, callback, errback, headerCallback,
             bodyDataCallback, method, start, etag, modified, cookies, postVariables, postFiles)
