@@ -14,6 +14,17 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+#
+# In addition, as a special exception, the copyright holders give
+# permission to link the code of portions of this program with the OpenSSL
+# library.
+#
+# You must obey the GNU General Public License in all respects for all of
+# the code used other than OpenSSL. If you modify file(s) with this
+# exception, you may extend this exception to your version of the file(s),
+# but you are not obligated to do so. If you do not wish to do so, delete
+# this exception statement from your version. If you delete this exception
+# statement from all source files in the program, then also delete it here.
 
 from copy import copy
 from datetime import datetime, timedelta
@@ -21,12 +32,13 @@ from gtcache import gettext as _
 from math import ceil
 from xhtmltools import unescape,xhtmlify
 from xml.sax.saxutils import unescape
-from util import checkU, returnsUnicode, checkF, returnsFilename, quoteUnicodeURL
+from util import checkU, returnsUnicode, checkF, returnsFilename, quoteUnicodeURL, stringify
 from platformutils import FilenameType
 import locale
 import os
 import os.path
 import urllib
+import urlparse
 import shutil
 import traceback
 
@@ -61,6 +73,9 @@ import logging
 import platformutils
 import filetypes
 import searchengines
+import fileutil
+import imageresize
+import license
 
 _charset = locale.getpreferredencoding()
 
@@ -69,10 +84,9 @@ class Item(DDBObject):
     associated with it.
     """
 
-    ICON_CACHE_SIZES = [
-        (108, 81),
-        (226, 170),
-    ]
+    SMALL_ICON_SIZE = (108, 81)
+    BIG_ICON_SIZE = (226, 170)
+    ICON_CACHE_SIZES = [SMALL_ICON_SIZE, BIG_ICON_SIZE]
 
     def __init__(self, entry, linkNumber = 0, feed_id=None, parent_id=None):
         self.feed_id = feed_id
@@ -92,7 +106,9 @@ class Item(DDBObject):
         self.eligibleForAutoDownload = True
         self.duration = None
         self.screenshot = None
+        self.resized_screenshots = {}
         self.resumeTime = 0
+        self.channelTitle = None
 
         self.iconCache = IconCache(self)
         
@@ -115,6 +131,11 @@ class Item(DDBObject):
         else:
             self.iconCache.dbItem = self
             self.iconCache.requestUpdate()
+        # For unknown reason(s), some users still have databases with item 
+        # objects missing the isContainerItem attribute even after
+        # a db upgrade (#8819).
+        if not hasattr(self, 'isContainerItem'):
+            self.isContainerItem = None
         self._initRestore()
 
     def _initRestore(self):
@@ -149,7 +170,7 @@ class Item(DDBObject):
 
     def toggleShowMoreInfo(self):
         self.showMoreInfo = not self.showMoreInfo
-        self.signalChange(needsSave=False, needsUpdateXML=True);
+        self.signalChange(needsSave=False, needsUpdateXML=True)
 
     @returnsUnicode
     def getMoreInfoState(self):
@@ -168,7 +189,7 @@ class Item(DDBObject):
             for (dirpath, dirnames, filenames) in os.walk(filename_root):
                 for name in filenames:
                     filename = os.path.join (dirpath, name)
-                    if filetypes.isVideoFilename(filename):
+                    if filetypes.isVideoFilename(filename) or filetypes.isAudioFilename(filename):
                         videos.add(filename)
         return videos
 
@@ -373,7 +394,7 @@ class Item(DDBObject):
         self.confirmDBThread()
         videoEnclosure = self.getFirstVideoEnclosure()
         if videoEnclosure is not None and 'url' in videoEnclosure:
-            return quoteUnicodeURL(videoEnclosure['url'])
+            return quoteUnicodeURL(videoEnclosure['url'].replace('+', '%20'))
         else:
             return u''
 
@@ -444,7 +465,8 @@ class Item(DDBObject):
         self.confirmDBThread()
         self.removeFromPlaylists()
         UandA = self.getUandA()
-        self.deleteFiles()
+        if not self.isExternal():
+            self.deleteFiles()
         self.expired = True
         if self.isContainerItem:
             for item in self.getChildren():
@@ -462,16 +484,47 @@ class Item(DDBObject):
                 pass
         # This should be done even if screenshot = ""
         self.screenshot = None
-        if self.getFeedURL() != "dtv:manualFeed":
-            self.signalChange()
-        else:
+        if self.isExternal():
+            if self.isDownloaded():
+                new_item = FileItem (self.getVideoFilename(), feed_id=self.feed_id, parent_id=self.parent_id, deleted=True)
+                if self.downloader is not None:
+                    self.downloader.setDeleteFiles(False)
             self.remove()
+        else:
+            self.signalChange()
 
     ##
     # Marks this item as expired
     def expire(self):
-        if self.isContainerItem:
-            title = _("Deleting %s") % (os.path.basename(self.getTitle()))
+        title = _("Removing %s") % os.path.basename(self.getTitle())
+        if self.isExternal():
+            if self.isContainerItem:
+                description = _("""\
+Would you like to delete this folder and all of its videos or just remove \
+its entry from the Library?""")
+                button = dialogs.BUTTON_DELETE_FILES
+            else:
+                if self.isDownloaded():
+                    description = _("""\
+Would you like to delete this file or just remove its entry from the \
+Library?""")
+                    button = dialogs.BUTTON_DELETE_FILE
+                else:
+                    self.executeExpire()
+                    return
+            d = dialogs.ThreeChoiceDialog(title, description,
+                    dialogs.BUTTON_REMOVE_ENTRY, button,
+                    dialogs.BUTTON_CANCEL)
+            def callback(dialog):
+                if not self.idExists():
+                    return
+                if dialog.choice == button:
+                    self.deleteFiles()
+                if dialog.choice in (button, dialogs.BUTTON_REMOVE_ENTRY):
+                    self.executeExpire()
+    
+            d.run(callback)
+        elif self.isContainerItem:
             description = _("""\
 This item is a folder.  When you remove a folder, any items inside that \
 folder will be deleted.""")
@@ -631,7 +684,7 @@ folder will be deleted.""")
 
     ##
     # Marks the item as seen
-    def markItemSeen(self):
+    def markItemSeen(self, markOtherItems=True):
         self.confirmDBThread()
         if self.seen == False:
             self.seen = True
@@ -639,6 +692,10 @@ folder will be deleted.""")
                 self.watchedTime = datetime.now()
             self.clearParentsChildrenSeen()
             self.signalChange()
+            if markOtherItems and self.downloader:
+                for item in self.downloader.itemList:
+                    if item != self:
+                        item.markItemSeen(False)
 
     def clearParentsChildrenSeen(self):
         if self.parent_id:
@@ -646,7 +703,7 @@ folder will be deleted.""")
             parent.childrenSeen = None
             parent.signalChange()
 
-    def markItemUnseen(self):
+    def markItemUnseen(self, markOtherItems=True):
         self.confirmDBThread()
         if self.isContainerItem:
             self.childrenSeen = False
@@ -661,6 +718,10 @@ folder will be deleted.""")
             self.watchedTime = None
             self.clearParentsChildrenSeen()
             self.signalChange()
+            if markOtherItems and self.downloader:
+                for item in self.downloader.itemList:
+                    if item != self:
+                        item.markItemUnseen(False)
 
     @returnsUnicode
     def getRSSID(self):
@@ -728,11 +789,12 @@ folder will be deleted.""")
 
         if self.downloader is None:
             self.downloader = downloader.getDownloader(self)
-        self.downloader.setChannelName (platformutils.unicodeToFilename(self.getChannelTitle(True)))
-        if self.downloader.isFinished():
-            self.onDownloadFinished()
-        else:
-            self.downloader.start()
+        if self.downloader is not None:
+            self.downloader.setChannelName (platformutils.unicodeToFilename(self.getChannelTitle(True)))
+            if self.downloader.isFinished():
+                self.onDownloadFinished()
+            else:
+                self.downloader.start()
         self.signalChange()
 
     def pause(self):
@@ -789,24 +851,30 @@ folder will be deleted.""")
         except:
             return None
 
+    # When changing this function, change feed.iconChanged to signal the right set of items
     @returnsUnicode
     def getThumbnail (self):
         self.confirmDBThread()
+        if self.showMoreInfo:
+            width, height = Item.BIG_ICON_SIZE
+        else:
+            width, height = Item.SMALL_ICON_SIZE
         if self.iconCache.isValid():
-            if self.showMoreInfo:
-                width, height = 226, 170
-            else:
-                width, height = 108, 81
             path = self.iconCache.getResizedFilename(width, height)
             return resources.absoluteUrl(path)
         elif self.screenshot:
-            return resources.absoluteUrl(self.screenshot)
+            path = self.getResizedScreenshot(width, height)
+            return resources.absoluteUrl(path)
         elif self.isContainerItem:
             return resources.url(u"images/container-icon.png")
         else:
-            if self.showMoreInfo:
+            feedThumbnail = self.getFeed().getItemThumbnail(width, height)
+            if feedThumbnail is not None:
+                return feedThumbnail
+            elif self.showMoreInfo:
                 return resources.url(u"images/thumb-more-info.png")
-            else: return resources.url(u"images/thumb.png")
+            else: 
+                return resources.url(u"images/thumb.png")
 
     ##
     # returns the title of the item
@@ -827,6 +895,10 @@ folder will be deleted.""")
     def getQuotedTitle(self):
         return urllib.quote_plus(self.getTitle().encode('utf8')).decode('ascii', 'replace')
 
+    def setChannelTitle(self, title):
+        checkU(title)
+        self.channelTitle = title
+
     @returnsUnicode
     def getChannelTitle(self, allowSearchFeedTitle=False):
         implClass = self.getFeed().actualFeed.__class__
@@ -834,6 +906,8 @@ folder will be deleted.""")
             return self.getFeed().getTitle()
         elif implClass == feed.SearchFeedImpl and allowSearchFeedTitle:
             return searchengines.getLastEngineTitle()
+        elif self.channelTitle:
+            return self.channelTitle
         else:
             return u''
 
@@ -917,30 +991,57 @@ folder will be deleted.""")
 
     def getDownloadDetails(self):
         status = self.downloader.status
-        return [
-            (_('Total Down:'), formatSizeForDetails(
-                status.get('currentSize', 0))),
+        details = [
+            (_('Total Down:'), formatSizeForDetails(status.get('currentSize', 0))),
         ]
+        if status.get("reasonFailed"):
+            details.append((_('Error:'), status['reasonFailed']))
+        return details
 
     def getTorrentDetails(self):
         status = self.downloader.status
-        return [
-            (_('Seeders:'), status.get('seeders', 0)),
-            (_('Leachers:'), status.get('leachers', 0)),
+        retval = []
+        seeders = status.get('seeders', -1)
+        leechers = status.get('leechers', -1)
+        if seeders != -1:
+            retval.append((_('Seeders:'), seeders))
+        if leechers != -1:
+            retval.append((_('Leechers:'), leechers))
+        retval.extend ([
             (_('Down Rate:'), formatRateForDetails(status.get('rate', 0))),
             (_('Down Total:'), formatSizeForDetails(
                 status.get('currentSize', 0))),
             (_('Up Rate:'), formatRateForDetails(status.get('upRate', 0))),
-            (_('Up Total:'), formatSizeForDetails(status.get('uploaded', 0) * 1024 * 1024)),
-        ]
+            (_('Up Total:'), formatSizeForDetails(status.get('uploaded', 0))),
+        ])
+
+        return retval
 
     def getItemDetails(self):
-        rv = [
-            (_('Web page:'), util.makeAnchor(_('permalink'), self.getLink())),
-            (_('File link:'), util.makeAnchor(_('direct link to file'),
-                self.getURL())),
-            (_('File type:'), self.getFormat())
-        ]
+        rv = []
+        
+        link = self.getLink()
+        if link:
+            rv.append((_('Web page:'), util.makeAnchor(_('permalink'), link)))
+
+        url = self.getURL()
+        if url and not url.startswith("file:"):
+            rv.append((_('File link:'), util.makeAnchor(_('direct link to file'),
+                                              url)))
+        rv.append((_('File type:'), self.getFormat()))
+
+        if self.getLicence():
+            # check the license to see if it's a url by seeing if it has a 
+            # protocol
+            if urlparse.urlparse(self.getLicence())[0]:
+                ln = license.license_name(self.getLicence())
+                rv.append((_('License:'), util.makeAnchor(ln,
+                                                          self.getLicence())))
+            else:
+                rv.append((_('License:'), _('see permalink')))
+        else:
+            rv.append((_('License:'), _('see permalink')))
+
         if self.isDownloaded():
             basename = os.path.basename(self.getFilename())
             basename = util.clampText(basename, 40)
@@ -959,7 +1060,7 @@ folder will be deleted.""")
         return [
             (_('Down Total'), formatSizeForDetails(
                 status.get('currentSize', 0))),
-            (_('Up Total'), formatSizeForDetails(status.get('uploaded', 0) * 1024 * 1024)),
+            (_('Up Total'), formatSizeForDetails(status.get('uploaded', 0))),
         ]
 
     def makeMoreInfoTable(self, title, moreInfoData):
@@ -986,9 +1087,10 @@ folder will be deleted.""")
             if self.isTransferring():
                 addTable(_('Torrent Details'), self.getTorrentDetails())
             elif self.downloader and self.downloader.isFinished():
-                addTable(u'Torrent Details <i>not connected</i>',
+                addTable(_('Torrent Details <i>stopped</i>'),
                         self.getTorrentDetailsFinished())
-        elif self.getState() == u'downloading' and not self.pendingManualDL:
+        elif ((self.getState() == u'downloading' and not self.pendingManualDL)
+                or self.isFailedDownload()):
             addTable(_('Download Details'), self.getDownloadDetails())
         return u'\n'.join(details)
 
@@ -1117,26 +1219,29 @@ folder will be deleted.""")
     ##
     # Returns the size of the item to be displayed.
     def getSizeForDisplay(self):
-        if not hasattr(self, "_size"):
-            self._size = self.getSize()
-        return util.formatSizeForUser(self._size)
-    
-    ##
-    # Returns the size of the item. If the item has a corresponding downloaded 
-    # enclosure we use the pysical size of the file, otherwise we use the RSS 
-    # enclosure tag values.
+        return util.formatSizeForUser(self.getSize())
+
     def getSize(self):
+        if not hasattr(self, "_size"):
+            self._size = self._getSize()
+        return self._size
+
+    ##
+    # Returns the size of the item. We use the following methods to get the
+    # size:
+    #
+    # Physical size of a downloaded file
+    # HTTP content-length
+    # RSS enclosure tag value.
+    def _getSize(self):
         fname = self.getFilename()
-        if os.path.exists(fname):
-            if os.path.isdir(fname):
-                size = 0
-                for (dirpath, dirnames, filenames) in os.walk(fname):
-                    for name in filenames:
-                        size += os.path.getsize(os.path.join(dirpath, name))
-                    size += os.path.getsize(dirpath)
-                return size
-            else:
-                return os.path.getsize(fname)
+        if self.isDownloaded():
+            try:
+                return util.getsize(fname)
+            except OSError:
+                return 0
+        elif self.downloader is not None:
+            return self.downloader.getTotalSize()
         else:
             try:
                 return int(self.getFirstVideoEnclosure()['length'])
@@ -1202,7 +1307,7 @@ folder will be deleted.""")
     def downloadETA(self):
         if self.downloader is not None:
             totalSecs = self.downloader.getETA()
-            if totalSecs == -1:
+            if totalSecs <= 0:
                 return _('downloading...')
         else:
             totalSecs = 0
@@ -1248,22 +1353,19 @@ folder will be deleted.""")
     # Returns the published date of the item
     @returnsUnicode
     def getPubDate(self):
-        try:
-            return self.releaseDateObj.strftime("%b %d %Y").decode(_charset)
-        except: 
-            return u""
+        return getReleaseDate()
     
     ##
     # Returns the published date of the item as a datetime object
     def getPubDateParsed(self):
-        return self.releaseDateObj
+        return self.getReleaseDateObj()
 
     ##
     # returns the date this video was released or when it was published
     @returnsUnicode
     def getReleaseDate(self):
         try:
-            return self.releaseDateObj.strftime("%b %d %Y").decode(_charset)
+            return self.getReleaseDateObj().strftime("%b %d %Y").decode(_charset)
         except:
             return u""
 
@@ -1295,7 +1397,7 @@ folder will be deleted.""")
     ##
     # returns string with the format of the video
     KNOWN_MIME_TYPES = (u'audio', u'video')
-    KNOWN_MIME_SUBTYPES = (u'mov', u'wmv', u'mp4', u'mp3', u'mpg', u'mpeg', u'avi', u'x-flv', u'x-msvideo')
+    KNOWN_MIME_SUBTYPES = (u'mov', u'wmv', u'mp4', u'mp3', u'mpg', u'mpeg', u'avi', u'x-flv', u'x-msvideo', u'm4v', u'mkv', u'm2v')
     MIME_SUBSITUTIONS = {
         u'QUICKTIME': u'MOV',
     }
@@ -1321,10 +1423,9 @@ folder will be deleted.""")
                         format += u' AUDIO'
                     if format.startswith(u'X-'):
                         format = format[2:]
-                    return u'.%s' % self.MIME_SUBSITUTIONS.get(format, format).lowercase()
-            else:
-                if extension in self.KNOWN_MIME_SUBTYPES:
-                    return u'.%s' % extension
+                    return u'.%s' % self.MIME_SUBSITUTIONS.get(format, format).lower()
+            if extension in self.KNOWN_MIME_SUBTYPES:
+                return u'.%s' % extension
         except:
             pass
         if emptyForUnknown:
@@ -1346,6 +1447,7 @@ folder will be deleted.""")
     # return the license associated with the video
     @returnsUnicode
     def getLicence(self):
+
         self.confirmDBThread()
         try:
             return self.entry.license
@@ -1427,6 +1529,9 @@ folder will be deleted.""")
             ]
             if self.getSeen():
                 items.append((self.markItemUnseen, _('Mark as Unwatched')))
+            else:
+                items.append((self.markItemSeen, _('Mark as Watched')))
+				
             if self.downloader and self.downloader.getState() == 'finished' and self.downloader.getType() == 'bittorrent':
                 items.append((self.startUpload, _('Restart Upload')))
         elif self.getState() == 'downloading':
@@ -1467,6 +1572,20 @@ folder will be deleted.""")
         
         app.delegate.notifyDownloadCompleted(self)
 
+    def getResizedScreenshot(self, width, height):
+        try:
+            return imageresize.getImage(self.resized_screenshots, width, height)
+        except KeyError:
+            return self.screenshot
+
+    def resizeScreenshot(self):
+        imageresize.removeResizedFiles(self.resized_screenshots)
+        if self.screenshot:
+            self.resized_screenshots = imageresize.multiResizeImage(
+                    self.screenshot, self.ICON_CACHE_SIZES)
+        else:
+            self.resized_screenshots = {}
+
     def save(self):
         self.confirmDBThread()
         if self.keep != True:
@@ -1512,7 +1631,7 @@ folder will be deleted.""")
         """Returns True iff this item was not downloaded from a Democracy
         channel.
         """
-        return False
+        return self.feed_id is not None and self.getFeedURL() == 'dtv:manualFeed'
 
     def isPlayable(self):
         """Returns True iff this item should have a play button."""
@@ -1538,6 +1657,7 @@ folder will be deleted.""")
         if self.iconCache is not None:
             self.iconCache.remove()
             self.iconCache = None
+        imageresize.removeResizedFiles(self.resized_screenshots)
         if self.isContainerItem:
             for item in self.getChildren():
                 item.remove()
@@ -1561,7 +1681,7 @@ folder will be deleted.""")
         if self.screenshot and not os.path.exists(self.screenshot):
             self.screenshot = None
             self.signalChange()
-        if self.duration == None or self.screenshot == None:
+        if self.duration is None or self.screenshot is None:
             moviedata.movieDataUpdater.requestUpdate (self)
 
     def __str__(self):
@@ -1574,33 +1694,37 @@ def reconnectDownloaders():
         reconnected.add(item.downloader)
     for downloader in views.remoteDownloads:
         if downloader not in reconnected:
-            print "removing orphaned downloader: ", downloader.url
+            logging.warn("removing orphaned downloader: %s", downloader.url)
             downloader.remove()
     manualFeed = util.getSingletonDDBObject(views.manualFeed)
     manualItems = views.items.filterWithIndex(indexes.itemsByFeed,
             manualFeed.getID())
     for item in manualItems:
         if item.downloader is None and item.__class__ == Item:
-            print "removing cancelled external torrent: ", item
+            logging.warn("removing cancelled external torrent: %s", item)
             item.remove()
 
 def getEntryForFile(filename):
     return FeedParserDict({'title':platformutils.filenameToUnicode(os.path.basename(filename)),
             'enclosures':[{'url': resources.url(filename)}]})
 
-def getEntryForURL(url):
+def getEntryForURL(url, contentType=None):
+    if contentType is None:
+        contentType = u'video/x-unknown'
+    else:
+        contentType = unicode(contentType)
     return FeedParserDict({'title' : url,
-            'enclosures':[{'url' : url}]})
+            'enclosures':[{'url' : url, 'type' : contentType}]})
 
 ##
 # An Item that exists as a local file
 class FileItem(Item):
 
-    def __init__(self,filename, feed_id=None, parent_id=None, offsetPath=None):
+    def __init__(self,filename, feed_id=None, parent_id=None, offsetPath=None, deleted=False):
         checkF(filename)
         filename = os.path.abspath(filename)
         self.filename = filename
-        self.deleted = False
+        self.deleted = deleted
         self.offsetPath = offsetPath
         self.shortFilename = cleanFilename(os.path.basename(self.filename))
         Item.__init__(self, getEntryForFile(filename), feed_id=feed_id, parent_id=parent_id)
@@ -1678,34 +1802,6 @@ class FileItem(Item):
                 self.deleted = True
                 self.signalChange()
 
-    def expire(self):
-        if not self.isExternal():
-            self.executeExpire()
-            return
-        title = _("Removing %s") % os.path.basename(platformutils.filenameToUnicode(self.filename))
-        if self.isContainerItem:
-            description = _("""\
-Would you like to delete this folder and all of its videos or just remove \
-its entry from the Library?""")
-            button = dialogs.BUTTON_DELETE_FILES
-        else:
-            description = _("""\
-Would you like to delete this file or just remove its entry from My \
-Collection?""")
-            button = dialogs.BUTTON_DELETE_FILE
-        d = dialogs.ThreeChoiceDialog(title, description,
-                dialogs.BUTTON_REMOVE_ENTRY, button,
-                dialogs.BUTTON_CANCEL)
-        def callback(dialog):
-            if not self.idExists():
-                return
-            if dialog.choice == button:
-                self.deleteFiles()
-            if dialog.choice in (button, dialogs.BUTTON_REMOVE_ENTRY):
-                self.executeExpire()
-
-        d.run(callback)
-
     def deleteFiles(self):
         try:
             if self.getParent():
@@ -1717,8 +1813,8 @@ Collection?""")
             elif os.path.isdir(self.filename):
                 shutil.rmtree(self.filename)
         except:
-            print "WARNING: error deleting files"
-            traceback.print_exc()
+            logging.warn("WARNING: error deleting files:\n%s",
+                    traceback.format_exc())
 
     def getDownloadedTime(self):
         self.confirmDBThread()
@@ -1737,7 +1833,19 @@ Collection?""")
     def download(self,autodl=False):
         self.deleted = False
         self.signalChange()
-        
+
+    def updateReleaseDate(self):
+        # This should be called whenever we get a new entry
+        try:
+            self.releaseDateObj = datetime.fromtimestamp(os.path.getmtime(self.filename))
+        except:
+            self.releaseDateObj = datetime.min
+
+    def getReleaseDateObj(self):
+        if self.parent_id:
+            return self.getParent().releaseDateObj
+        else:
+            return self.releaseDateObj
 
     def migrate(self, newDir):
         self.confirmDBThread()
@@ -1746,23 +1854,19 @@ Collection?""")
             self.filename = os.path.join (parent.getFilename(), self.offsetPath)
             return
         if self.shortFilename is None:
-            print """\
-WARNING: can't migrate download because we don't have a shortFilename!
-filename was %s""" % self.filename
+            logging.warn("""\
+can't migrate download because we don't have a shortFilename!
+filename was %s""", stringify(self.filename))
             return
         newFilename = os.path.join(newDir, self.shortFilename)
         if self.filename == newFilename:
             return
         if os.path.exists(self.filename):
             newFilename = nextFreeFilename(newFilename)
-            try:
-                shutil.move(self.filename, newFilename)
-            except Exception, e:
-                print "WARNING: Error moving %s to %s (%s)" % (self.filename,
-                        newFilename, e)
-            else:
+            def callback():
                 self.filename = newFilename
                 self.signalChange()
+            fileutil.migrate_file(self.filename, newFilename, callback)
         elif os.path.exists(newFilename):
             self.filename = newFilename
             self.signalChange()
@@ -1777,7 +1881,9 @@ filename was %s""" % self.filename
                 if self.filename.startswith(parent_file):
                     self.shortFilename = cleanFilename(self.filename[len(parent_file):])
                 else:
-                    print "WARNING: %s is not a subdirectory of %s" % (self.filename, parent_file)
+                    logging.warn("%s is not a subdirectory of %s",
+                            self.filename, parent_file)
+        self.updateReleaseDate()
         Item.setupLinks(self)
 
 def expireItems(items):
