@@ -14,6 +14,17 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+#
+# In addition, as a special exception, the copyright holders give
+# permission to link the code of portions of this program with the OpenSSL
+# library.
+#
+# You must obey the GNU General Public License in all respects for all of
+# the code used other than OpenSSL. If you modify file(s) with this
+# exception, you may extend this exception to your version of the file(s),
+# but you are not obligated to do so. If you do not wish to do so, delete
+# this exception statement from your version. If you delete this exception
+# statement from all source files in the program, then also delete it here.
 
 """Module used to upgrade from databases before we had our current scheme.
 
@@ -34,10 +45,13 @@ import pickle
 import shutil
 import threading
 import types
+import time
 
-from schema import ObjectSchema, SchemaInt, SchemaFloat
+from schema import ObjectSchema, SchemaInt, SchemaFloat, SchemaSimpleItem
 from schema import SchemaObject, SchemaBool, SchemaDateTime, SchemaTimeDelta
-from schema import SchemaList, SchemaDict, SchemaString, SchemaSimpleContainer
+from schema import SchemaList, SchemaDict
+from fasttypes import LinkedList
+from types import NoneType
 import storedatabase
 
 ######################### STAGE 1 helpers  #############################
@@ -327,6 +341,48 @@ class DDBObjectSchema(ObjectSchema):
         ('id', SchemaInt())
     ]
 
+# Unlike the SchemaString in schema.py, this allows binary strings or
+# unicode strings
+class SchemaString(SchemaSimpleItem):
+    def validate(self, data):
+        super(SchemaSimpleItem, self).validate(data)
+        self.validateTypes(data, (unicode, str))
+
+# Unlike the simple container in schema.py, this allows binary strings
+class SchemaSimpleContainer(SchemaSimpleItem):
+    """Allows nested dicts, lists and tuples, however the only thing they can
+    store are simple objects.  This currently includes bools, ints, longs,
+    floats, strings, unicode, None, datetime and struct_time objects.
+    """
+
+    def validate(self, data):
+        super(SchemaSimpleContainer, self).validate(data)
+        self.validateTypes(data, (dict, list, tuple))
+        self.memory = set()
+        toValidate = LinkedList()
+        while data:
+            if id(data) in self.memory:
+                return
+            else:
+                self.memory.add(id(data))
+    
+            if isinstance(data, list) or isinstance(data, tuple):
+                for item in data:
+                    toValidate.append(item)
+            elif isinstance(data, dict):
+                for key, value in data.items():
+                    self.validateTypes(key, [bool, int, long, float, unicode,
+                        str, NoneType, datetime, time.struct_time])
+                    toValidate.append(value)
+            else:
+                self.validateTypes(data, [bool, int, long, float, unicode,str,
+                        NoneType, datetime, time.struct_time])
+            try:
+                data = toValidate.pop()
+            except:
+                data = None
+
+
 class ItemSchema(DDBObjectSchema):
     klass = OldItem
     classString = 'item'
