@@ -14,6 +14,17 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+#
+# In addition, as a special exception, the copyright holders give
+# permission to link the code of portions of this program with the OpenSSL
+# library.
+#
+# You must obey the GNU General Public License in all respects for all of
+# the code used other than OpenSSL. If you modify file(s) with this
+# exception, you may extend this exception to your version of the file(s),
+# but you are not obligated to do so. If you do not wish to do so, delete
+# this exception statement from your version. If you delete this exception
+# statement from all source files in the program, then also delete it here.
 
 """The schema module is responsible for defining what data in the database
 gets stored on disk.  
@@ -244,7 +255,7 @@ class ObjectSchema(object):
 
 from database import DDBObject
 from downloader import RemoteDownloader, HTTPAuthPassword
-from feed import Feed, FeedImpl, RSSFeedImpl, ScraperFeedImpl
+from feed import Feed, FeedImpl, RSSFeedImpl, RSSMultiFeedImpl, ScraperFeedImpl
 from feed import SearchFeedImpl, DirectoryWatchFeedImpl, DirectoryFeedImpl, SearchDownloadsFeedImpl
 from feed import ManualFeedImpl, SingleFeedImpl
 from folder import ChannelFolder, PlaylistFolder
@@ -253,6 +264,7 @@ from item import Item, FileItem
 from iconcache import IconCache
 from playlist import SavedPlaylist
 from tabs import TabOrder
+from theme import ThemeHistory
 
 class DDBObjectSchema(ObjectSchema):
     klass = DDBObject
@@ -297,7 +309,9 @@ class ItemSchema(DDBObjectSchema):
         ('eligibleForAutoDownload', SchemaBool()),
         ('duration', SchemaInt(noneOk=True)),
         ('screenshot', SchemaFilename(noneOk=True)),
+        ('resized_screenshots', SchemaDict(SchemaString(), SchemaFilename())),
         ('resumeTime', SchemaInt()),
+        ('channelTitle', SchemaString(noneOk=True)),
     ]
 
 class FileItemSchema(ItemSchema):
@@ -354,6 +368,14 @@ class RSSFeedImplSchema(FeedImplSchema):
         ('modified', SchemaString(noneOk=True)),
     ]
 
+class RSSMultiFeedImplSchema(FeedImplSchema):
+    klass = RSSMultiFeedImpl
+    classString = 'rss-multi-feed-impl'
+    fields = FeedImplSchema.fields + [
+        ('etag', SchemaDict(SchemaString(),SchemaString(noneOk=True))),
+        ('modified', SchemaDict(SchemaString(),SchemaString(noneOk=True))),
+    ]
+
 class ScraperFeedImplSchema(FeedImplSchema):
     klass = ScraperFeedImpl
     classString = 'scraper-feed-impl'
@@ -363,10 +385,10 @@ class ScraperFeedImplSchema(FeedImplSchema):
         ('linkHistory', SchemaSimpleContainer()),
     ]
 
-class SearchFeedImplSchema(FeedImplSchema):
+class SearchFeedImplSchema(RSSMultiFeedImplSchema):
     klass = SearchFeedImpl
     classString = 'search-feed-impl'
-    fields = FeedImplSchema.fields + [
+    fields = RSSMultiFeedImplSchema.fields + [
         ('searching', SchemaBool()),
         ('lastEngine', SchemaString()),
         ('lastQuery', SchemaString()),
@@ -464,15 +486,28 @@ class ChannelGuideSchema(DDBObjectSchema):
     classString = 'channel-guide'
     fields = DDBObjectSchema.fields + [
         ('url', SchemaURL(noneOk=True)),
+        ('updated_url', SchemaURL(noneOk=True)),
+        ('favicon', SchemaURL(noneOk=True)),
+        ('title', SchemaString(noneOk=True)),
+        ('iconCache', SchemaObject(IconCache, noneOk=True)),
+        ('firstTime', SchemaBool()),
     ]
 
-VERSION = 50
-objectSchemas = [ 
+class ThemeHistorySchema(DDBObjectSchema):
+    klass = ThemeHistory
+    classString = 'theme-history'
+    fields = DDBObjectSchema.fields + [
+        ('lastTheme', SchemaString(noneOk=True)),
+        ('pastThemes', SchemaList(SchemaString(noneOk=True), noneOk=False)),
+    ]
+
+VERSION = 61
+objectSchemas = [
     DDBObjectSchema, IconCacheSchema, ItemSchema, FileItemSchema, FeedSchema,
-    FeedImplSchema, RSSFeedImplSchema, ScraperFeedImplSchema,
+    FeedImplSchema, RSSFeedImplSchema, RSSMultiFeedImplSchema, ScraperFeedImplSchema,
     SearchFeedImplSchema, DirectoryFeedImplSchema, DirectoryWatchFeedImplSchema,
     SearchDownloadsFeedImplSchema, RemoteDownloaderSchema,
     HTTPAuthPasswordSchema, ChannelGuideSchema, ManualFeedImplSchema, SingleFeedImplSchema,
     PlaylistSchema, ChannelFolderSchema, PlaylistFolderSchema,
-    TabOrderSchema
+    TabOrderSchema, ThemeHistorySchema
 ]
