@@ -14,6 +14,17 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+#
+# In addition, as a special exception, the copyright holders give
+# permission to link the code of portions of this program with the OpenSSL
+# library.
+#
+# You must obey the GNU General Public License in all respects for all of
+# the code used other than OpenSSL. If you modify file(s) with this
+# exception, you may extend this exception to your version of the file(s),
+# but you are not obligated to do so. If you do not wish to do so, delete
+# this exception statement from your version. If you delete this exception
+# statement from all source files in the program, then also delete it here.
 
 """Handle selection."""
 
@@ -23,12 +34,15 @@ import app
 import database
 import eventloop
 import folder
+import config
+import prefs
 import guide
 import item
 import tabs
 import playlist
 import feed
 import views
+import indexes
 import template
 import util
 from gtcache import gettext as _
@@ -341,12 +355,33 @@ class TabSelectionArea(SelectionArea):
                 prevTab = self.currentView.getNext()
             if prevTab is None:
                 # That was the last tab in the list, select the guide
-                self.selectFirstGuide()
+                self.selectFirstTab()
             else:
                 self.selectItem(self.currentView, prevTab.objID())
             self.handler.displayCurrentTabContent()
 
-    def selectFirstGuide(self):
+    def selectFirstTab(self):
+        if config.get(prefs.OPEN_CHANNEL_ON_STARTUP) is not None:
+            view = views.feeds.filterWithIndex(indexes.feedsByURL,
+                          unicode(config.get(prefs.OPEN_CHANNEL_ON_STARTUP)))
+            if len(view) > 0:
+                self.selectItem(views.allTabs, view[0].getID())
+                self.handler.displayCurrentTabContent()
+                view.unlink()
+                return
+            else:
+                view.unlink()
+        if config.get(prefs.OPEN_FOLDER_ON_STARTUP) is not None:
+            view = views.channelFolders.filterWithIndex(indexes.foldersByTitle,
+                          unicode(config.get(prefs.OPEN_FOLDER_ON_STARTUP)))
+            if len(view) > 0:
+                self.selectItem(views.allTabs, view[0].getID())
+                self.handler.displayCurrentTabContent()
+                view.unlink()
+                return
+            else:
+                view.unlink()
+
         views.guideTabs.resetCursor()
         guide = views.guideTabs.getNext()
         self.selectItem(views.guideTabs, guide.objID())
@@ -396,7 +431,8 @@ class SelectionHandler(object):
             return
 
         # ignore control and shift when selecting static tabs
-        if isinstance(selectedObj, tabs.Tab) and selectedObj.isStatic():
+        if (isinstance(selectedObj, tabs.Tab) and 
+                selectedObj.type in ('statictab', 'guide')):
             controlSelect = shiftSelect = False
 
         if controlSelect:
@@ -433,8 +469,8 @@ class SelectionHandler(object):
         else:
             return set([sourceID])
 
-    def selectFirstGuide(self):
-        self.tabListSelection.selectFirstGuide()
+    def selectFirstTab(self):
+        self.tabListSelection.selectFirstTab()
 
     def selectTabByTemplateBase(self, tabTemplateBase, displayTabContent=True):
         tabViews = [ 
