@@ -14,6 +14,17 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+#
+# In addition, as a special exception, the copyright holders give
+# permission to link the code of portions of this program with the OpenSSL
+# library.
+#
+# You must obey the GNU General Public License in all respects for all of
+# the code used other than OpenSSL. If you modify file(s) with this
+# exception, you may extend this exception to your version of the file(s),
+# but you are not obligated to do so. If you do not wish to do so, delete
+# this exception statement from your version. If you delete this exception
+# statement from all source files in the program, then also delete it here.
 
 """This module does the reading/writing of our database to/from disk.  It
 works with the schema module to validate the data that we read/write and with
@@ -57,6 +68,10 @@ import app
 import bsddb.db
 import dialogs
 import logging
+from zipfile import ZipFile
+import tempfile
+from random import randrange, seed
+import os.path
 try:
     from pysqlite2 import dbapi2 as sql
 except ImportError:
@@ -176,7 +191,7 @@ class ConverterBase(object):
         rv = []
         for i in xrange(len(list)):
             child = list[i]
-            newPath = path + "\n[%d] -> %s" % (i, child)
+            newPath = path + "\n[%d] -> %s" % (i, util.stringify(child))
             rv.append(self.convertData(child, childSchema, newPath))
         return rv
 
@@ -189,7 +204,7 @@ class ConverterBase(object):
             newPath = path + "\nkey: %s" % key
             newKey = self.convertData(key, keySchema, newPath)
             # convert the value
-            newPath = path + "\n{%s} -> %s" % (key, value)
+            newPath = path + "\n{%s} -> %s" % (util.stringify(key), util.stringify(value))
             newValue = self.convertData(value, valueSchema, newPath)
             # put it together
             rv[newKey] = newValue
@@ -235,7 +250,7 @@ class ConverterBase(object):
             except Exception, e:
                 # this will happen when data is invalid unicode
                 dataStr = "<couldn't convert (%s)>" % e
-            newPath = path + "\n%s -> %s" % (name, dataStr)
+            newPath = path + "\n%s -> %s" % (util.stringify(name), util.stringify(dataStr))
             convertedData = self.convertData(data, schema, newPath)
             self.setTargetAttr(convertedObject, name, convertedData)
         return convertedObject
@@ -897,6 +912,9 @@ class LiveStorageBDB:
         except bsddb.db.DBNoSpaceError:
             pass
         eventloop.addTimeout(60, self.checkpoint, "Remove Unused Database Logs")
+    def backupDatabase(self):
+        return None
+
 class LiveStorage:
     TRANSACTION_TIMEOUT = 10
     TRANSACTION_NAME = "Save database"
@@ -1031,6 +1049,41 @@ class LiveStorage:
         output.write ('</database>\n')
         output.close()
 
+    def backupDatabase(self):
+        # backs up the database and support directories to a zip file
+        # returns the name of the zip file
+        logging.info("Attempting to back up database")
+        if not self.closed:
+            try:
+                self.conn.close()
+            except:
+                traceback.print_exc()
+        try:
+            try:
+                tempfilename = os.path.join(tempfile.gettempdir(),("%012ddatabasebackup.zip"%randrange(0,999999999999)))
+                zipfile = ZipFile(tempfilename,"w")
+                for root, dirs, files in os.walk(config.get(prefs.SUPPORT_DIRECTORY)):
+                    if ((os.path.normpath(root) !=
+                        os.path.normpath(config.get(prefs.ICON_CACHE_DIRECTORY)))
+                        and not os.path.islink(root)):
+                        relativeroot = root[len(config.get(prefs.SUPPORT_DIRECTORY)):]
+                        while len(relativeroot)>0 and relativeroot[0] in ['/','\\']:
+                            relativeroot = relativeroot[1:]
+                        for filen in files:
+                            if not os.path.islink(os.path.join(root,filen)):
+                                zipfile.write(os.path.join(root,filen),
+                                              os.path.join(relativeroot, filen).encode('ascii','replace'))
+                zipfile.close()
+                logging.info("Database backed up to %s" % tempfilename)
+                return tempfilename
+            except:
+                traceback.print_exc()
+        finally:
+            if not self.closed:
+                self.conn = sql.connect(self.dbPath, isolation_level=None)
+                self.cursor = self.conn.cursor()
+        return None
+
     def handleDatabaseLoadError(self):
         database.confirmDBThread()
         logging.exception ("exception while loading database")
