@@ -1,5 +1,4 @@
 import unittest
-#import rfc822
 import email.Utils
 import socket
 import tempfile
@@ -176,13 +175,14 @@ class FakeStream:
     def stopReadTimeout(self):
         pass
 
-    def openConnection(self, host, port, callback, errback):
+    def openConnection(self, host, port, callback, errback, disabledReadTimeout=None):
         self.name = "Outgoing %s:%s" % (host, port)
         self.output = ''
         self.host = host
         self.port = port
         self.open = True
         self.errback = errback
+        self.dsiabledReadTimeout = disabledReadTimeout
         callback(self)
 
     def acceptConnection(self, host, port, callback, errback):
@@ -225,6 +225,7 @@ class DumbFakeStream(FakeStream):
 class TestingHTTPConnection(httpclient.HTTPConnection):
     """HTTPConnection that doesn't actually connect to the network."""
     streamFactory = FakeStream
+
 class TestingHTTPSConnection(httpclient.HTTPSConnection):
     """HTTPSConnection that doesn't actually connect to the network."""
     streamFactory = FakeStream
@@ -275,6 +276,7 @@ class DumbTestingHTTPConnectionPool(TestingHTTPConnectionPool):
 
 class DumbTestHTTPClient(httpclient.HTTPClient):
     connectionPool = DumbTestingHTTPConnectionPool()
+
 class TestHTTPClient(httpclient.HTTPClient):
     connectionPool = TestingHTTPConnectionPool()
 
@@ -503,7 +505,7 @@ class DumbHTTPClientTest(AsyncSocketTest):
         AsyncSocketTest.setUp(self)
         self.testRequest = DumbTestingHTTPConnection()
         self.testRequest.openConnection('foo.com', 80, lambda x: None,lambda x: None)
-        self.testRequest.sendRequest(self.callback, self.errback, 
+        self.testRequest.sendRequest(self.callback, self.errback, "", 80,
                 method='GET', path='/bar/baz;123?a=b') 
         self.authDelegate = TestingAuthDelegate()
         dialogs.setDelegate(self.authDelegate)
@@ -659,7 +661,7 @@ HELLO: WORLD\r\n"""
         self.testRequest.handleData(startResponse(
             headers={'Content-Length': 128}))
         self.assertEquals(self.testRequest.canSendRequest(), True)
-        self.testRequest.sendRequest(self.callback, self.errback,
+        self.testRequest.sendRequest(self.callback, self.errback, "", 80,
                 path="/pipelined/path")
         self.assertEquals(self.testRequest.pipelinedRequest[6],
                 '/pipelined/path')
@@ -680,7 +682,7 @@ HELLO: WORLD\r\n"""
         # no content length means we can't pipeline a request
         self.assertEquals(self.testRequest.canSendRequest(), False)
         self.assertRaises(httpclient.NetworkError,
-            self.testRequest.sendRequest, self.callback, self.errback)
+            self.testRequest.sendRequest, self.callback, self.errback, "", 80)
 
     def testPipelineNeverStarted(self):
         self.pipelineError = None
@@ -688,7 +690,7 @@ HELLO: WORLD\r\n"""
             headers={'Content-Length': 128}))
         def pipelineErrback(error):
             self.pipelineError = error
-        self.testRequest.sendRequest(self.callback, pipelineErrback,
+        self.testRequest.sendRequest(self.callback, pipelineErrback, "", 80, 
                 path="/pipelined/path")
         self.testRequest.handleClose(socket.SHUT_RDWR)
         self.assert_(isinstance(self.pipelineError,
@@ -700,7 +702,7 @@ HELLO: WORLD\r\n"""
             headers={'Content-Length': 128}))
         def pipelineErrback(error):
             self.pipelineError = error
-        self.testRequest.sendRequest(self.callback, pipelineErrback,
+        self.testRequest.sendRequest(self.callback, pipelineErrback, "", 80,
                 path="/pipelined/path")
         self.testRequest.closeConnection()
         self.assert_(isinstance(self.pipelineError,
@@ -800,13 +802,12 @@ HELLO: WORLD\r\n"""
         self.assert_(self.errbackCalled)
         self.assert_(isinstance(self.data, httpclient.ServerClosedConnection))
 
-class HTTPClientTest(AsyncSocketTest):
+class HTTPClientTestBase(AsyncSocketTest):
     def setUp(self):
         AsyncSocketTest.setUp(self)
         self.testRequest = TestingHTTPConnection()
-        self.testRequest.openConnection('foo.com', 80, lambda x: None,lambda x: None)
-        self.testRequest.sendRequest(self.callback, self.errback, 
-                method='GET', path='/bar/baz;123?a=b') 
+        self.testRequest.openConnection('foo.com', 80, lambda x: None, lambda x: None)
+        self.testRequest.sendRequest(self.callback, self.errback, "", 80, method='GET', path='/bar/baz;123?a=b') 
         self.authDelegate = TestingAuthDelegate()
         dialogs.setDelegate(self.authDelegate)
         TestHTTPClient.connectionPool = TestingHTTPConnectionPool()
@@ -816,6 +817,8 @@ class HTTPClientTest(AsyncSocketTest):
         # clear out any HTTPAuth objects in there
         AsyncSocketTest.tearDown(self)
 
+
+class HTTPClientTest(HTTPClientTestBase):
     def testRealRequest(self):
         url = 'http://participatoryculture.org/democracytest/normalpage.txt'
         httpclient.grabURL(url, self.callback, self.errback, clientClass=TestHTTPClient)
@@ -865,15 +868,23 @@ class HTTPClientTest(AsyncSocketTest):
     def testMultipleRequests(self):
         def middleCallback(data):
             self.firstData = data
-            req.sendRequest(self.callback, self.errback, method='GET', path='/democracytest/normalpage.txt')
+            req.sendRequest(self.callback, self.errback, "participatoryculture.org", 80, method='GET', path='/democracytest/normalpage.txt')
 
         req = TestingHTTPConnection()
         def stopEventLoop(conn):
             self.stopEventLoop(False)
-        self.addIdle(lambda: req.openConnection('participatoryculture.org', 80, stopEventLoop, stopEventLoop), "Open connection")
+        self.addIdle(lambda: req.openConnection('participatoryculture.org', 80, stopEventLoop, self.errback), "Open connection")
         self.runEventLoop()
-        self.addIdle(lambda: req.sendRequest(middleCallback, self.errback, method='GET', path='/democracytest/normalpage.txt'), "Send Request")
+        self.assert_(not self.errbackCalled)
+        self.addIdle(lambda: req.sendRequest(middleCallback,
+                                             self.errback,
+                                             "participatoryculture.org",
+                                             80,
+                                             method='GET',
+                                             path='/democracytest/normalpage.txt'),
+                     "Send Request")
         self.runEventLoop()
+        self.assert_(not self.errbackCalled)
         self.assertEquals(self.firstData['body'], self.data['body'])
 
     def testUnexpectedStatusCode(self):
@@ -996,8 +1007,8 @@ class HTTPClientTest(AsyncSocketTest):
         self.assertEquals(self.gotData, 'I AM A NORMAL PAGE\n')
 
     def testAuth(self):
-        self.authDelegate.addLogin('ben', 'baddpassword')
-        self.authDelegate.addLogin('guest', 'guest')
+        self.authDelegate.addLogin(u'ben', u'baddpassword')
+        self.authDelegate.addLogin(u'guest', u'guest')
         url = 'http://jigsaw.w3.org/HTTP/Basic/'
         client = TestHTTPClient(url, self.callback, self.errback)
         client.startRequest()
@@ -1007,9 +1018,9 @@ class HTTPClientTest(AsyncSocketTest):
         self.assertEquals(client.authAttempts, 2)
 
     def testBadAuth(self):
-        self.authDelegate.addLogin('baduser', 'baddpass')
-        self.authDelegate.addLogin('anotherbadtry', 'god')
-        self.authDelegate.addLogin('billgates', 'password')
+        self.authDelegate.addLogin(u'baduser', u'baddpass')
+        self.authDelegate.addLogin(u'anotherbadtry', u'god')
+        self.authDelegate.addLogin(u'billgates', u'password')
         url = 'http://jigsaw.w3.org/HTTP/Basic/'
         client = TestHTTPClient(url, self.callback, self.errback)
         client.startRequest()
@@ -1029,6 +1040,8 @@ class HTTPClientTest(AsyncSocketTest):
         self.assert_(isinstance(self.data, httpclient.AuthorizationFailed))
         self.assertEquals(client.authAttempts, 0)
 
+
+
 #     def testChunkedData(self):
 #         url = 'http://jigsaw.w3.org/HTTP/ChunkedScript'
 #         httpclient.grabURL(url, self.callback, self.errback, clientClass=TestHTTPClient)
@@ -1166,31 +1179,34 @@ class HTTPClientTest(AsyncSocketTest):
         testIt(u'\xf8???.\xf2\xf3x')
 
     def testGetFilenameFromResponse(self):
-        client = TestHTTPClient('http://www.foo.com', self.callback,
-                self.errback)
+        client = TestHTTPClient('http://www.foo.com', self.callback, self.errback)
+
         def getIt(path, cd=None):
             response = {'path': path}
             if cd:
                 response['content-disposition'] = cd
             return client.getFilenameFromResponse(response)
+
         self.assertEquals("unknown", getIt("/"))
         self.assertEquals("index.html", getIt("/index.html"))
         self.assertEquals("index.html", getIt("/path/path2/index.html"))
         self.assertEquals("unknown", getIt("/path/path2/"))
         self.assertEquals("myfile.txt", getIt("/", 'filename="myfile.txt"'))
-        self.assertEquals("myfile.txt", getIt("/",
-            'filename="myfile.txt"; size=45'))
-        self.assertEquals("myfile.txt", getIt("/",
-            ' filename =  "myfile.txt"'))
+        self.assertEquals("myfile.txt",
+                          getIt("/", 'filename="myfile.txt"; size=45'))
+        self.assertEquals("myfile.txt",
+                          getIt("/", ' filename =  "myfile.txt"'))
         self.assertEquals("myfile.txt", getIt("/", 'filename=myfile.txt'))
-        self.assertEquals("myfile.txt", getIt("/index.html",
-            cd='filename="myfile.txt"'))
+        self.assertEquals("myfile.txt",
+                          getIt("/index.html", 'filename="myfile.txt"'))
         self.assertEquals("lots.of.extensions",
-            getIt("/", 'filename="lots.of.extensions"'))
+                          getIt("/", 'filename="lots.of.extensions"'))
+
+        # FIXME - these two fail
         self.assertEquals("uncleanfilename", 
-                getIt("/index", 'filename="\\un/cl:ean*fi?lena<m>|e"'))
+                          getIt("/index", 'filename="\\un/cl:ean*fi?lena<m>|e"'))
         self.assertEquals("uncleanfil-ename2", 
-                getIt('/uncl*ean"fil?"ena|m""e2"'))
+                          getIt('/uncl*ean"fil?"ena|m""e2"'))
 
     def testGetCharsetFromResponse(self):
         client = TestHTTPClient('http://participatoryculture.org/democracytest/normal.txt', self.callback,
@@ -1521,7 +1537,7 @@ class GrabURLTest(AsyncSocketTest):
 #         self.runPendingIdles()
 #         self.checkPipelineCanceled()
 
-class BadURLTest(HTTPClientTest):
+class BadURLTest(HTTPClientTestBase):
     def testScheme(self):
         url = 'participatoryculture.org/democracytest/normalpage.txt'
         httpclient.grabURL(url, self.callback, self.errback, clientClass=TestHTTPClient)
@@ -1601,3 +1617,34 @@ class BadURLTest(HTTPClientTest):
 #         self.runEventLoop()
 #         self.assertEquals(self.count, 1)
 
+
+class CookieExpirationDateTestCase(unittest.TestCase):
+    def testCookieExpirationDate(self):
+        """Tests get_cookie_expiration_date to make sure it's returning
+        sane values and handles cookie expiration formats we've had
+        problems with.
+        """
+        from time import mktime, strptime, localtime
+        from httpclient import get_cookie_expiration_date
+        
+        for cd in ( ("Thu, 03-May-07 22:48:52 GMT", "2007-05-03 22:48:52 GMT" ),
+                    ("Fri, 03-Jun-11 13:41:15 GMT", "2011-06-03 13:41:15 GMT" ),
+                    ("Sun, 17-Jan-2038 19:14:07 GMT", "2038-01-17 19:14:07 GMT" ),
+                    ("Mon, 09-Apr-07 23:50:49 GMT", "2007-04-09 23:50:49 GMT" ),
+                    ("Tue, 01-Jan-2030 10:00:00 GMT", "2030-01-01 10:00:00 GMT" ),
+                    ("Tue, 17-Jul-2007 02:09:00 GMT", "2007-07-17 02:09:00 GMT") ):
+            
+            # compare the 9-tuple we get from localtime because tuples are
+            # easier to compare (and more accurate for what we're looking for)
+            self.assertEquals( localtime(get_cookie_expiration_date(cd[0])),
+                               localtime(mktime(strptime(cd[1], "%Y-%m-%d %H:%M:%S %Z"))) )
+            
+    def testOverflowCookieExpirationDate(self):
+        """tests the case of get_cookie_expiration_date where the cookie
+        expiration date causes an overflow error when parsing it.
+        """
+        from time import localtime
+        from httpclient import get_cookie_expiration_date, DATEINFUTURE
+            
+        self.assertEquals( localtime(get_cookie_expiration_date("Tue, 26-Jul-2050 10:00:00 GMT")),
+                           localtime(httpclient.DATEINFUTURE) )
