@@ -16,7 +16,8 @@ import gtcache
 import gettext
 import resources
 import template
-from template_compiler import TemplateUnicodeError
+import util
+from template_compiler import TemplateError
 import template_compiler
 
 from test.framework import DemocracyTestCase
@@ -98,17 +99,22 @@ class UnicodeFeedTestCase(framework.EventLoopTest):
     # This is a latin1 feed that claims to be UTF-8
     def testInvalidLatin1Feed(self):
         [handle, self.filename] = mkstemp(".xml")
-        handle =file(self.filename,"wb")
+        handle = file(self.filename,"wb")
         handle.write('<?xml version="1.0"?>\n<rss version="2.0">\n   <channel>\n      <title>H\xe4ppy Birthday</title>\n      <description>H\xe4ppy Birthday</description>\n <language>zh-zh</language>\n      <pubDate>Fri, 25 Aug 2006 17:39:21 GMT</pubDate>\n      <generator>Weblog Editor 2.0</generator>\n      <managingEditor>editor@example.com</managingEditor>\n      <webMaster>webmaster@example.com</webMaster>\n      <item>\n         <title>H\xe4ppy Birthday</title>\n         <link>http://participatoryculture.org/boguslink</link>\n         <description>H\xe4ppy Birthday</description>\n         <enclosure url="file://crap" length="0" type="video/mpeg"/>\n         <pubDate>Fri, 25 Aug 2006 17:39:21 GMT</pubDate>\n      </item>\n   </channel>\n</rss>')
         handle.close()
 
         dialogs.delegate = UnicodeTestDelegate()
         dialogs.delegate.choice = dialogs.BUTTON_YES
 
-        myFeed = feed.Feed("file://"+self.filename)
+        myFeed = feed.Feed(u"file://"+self.filename)
 
-        # Make sure the feed shows up as invalid
-        self.assertEqual(dialogs.delegate.numCalls,1)
+        myFeed.update()
+        self.processThreads()
+        self.processIdles()
+        self.assertEqual(len(myFeed.items), 1)
+        myItem = myFeed.items[0]
+        self.assertEqual(len(myItem.getTitle()), 14)
+        self.assertEqual(myItem.getTitle(), u"H\xe4ppy Birthday")
 
     # This is latin1 HTML that claims to be Latin 1
     def testLatin1HTML(self):
@@ -120,13 +126,13 @@ class UnicodeFeedTestCase(framework.EventLoopTest):
         dialogs.delegate = UnicodeTestDelegate()
         dialogs.delegate.choice = dialogs.BUTTON_YES
 
-        myFeed = feed.Feed("file://"+self.filename)
+        myFeed = feed.Feed(u"file://"+self.filename)
         
-        self.assertEqual(dialogs.delegate.numCalls,1)
+        self.assertEqual(dialogs.delegate.numCalls, 1)
         myFeed.update()
-        self.assertEqual(len(myFeed.items),1)
+        self.assertEqual(len(myFeed.items), 1)
         myItem = myFeed.items[0]
-        self.assertEqual(len(myItem.getTitle()),14)
+        self.assertEqual(len(myItem.getTitle()), 14)
         self.assertEqual(myItem.getTitle(), u"H\xe4ppy Birthday")
 
     # This is latin1 HTML that claims to be UTF-8
@@ -139,7 +145,7 @@ class UnicodeFeedTestCase(framework.EventLoopTest):
         dialogs.delegate = UnicodeTestDelegate()
         dialogs.delegate.choice = dialogs.BUTTON_YES
 
-        myFeed = feed.Feed("file://"+self.filename)
+        myFeed = feed.Feed(u"file://"+self.filename)
         
         self.assertEqual(dialogs.delegate.numCalls,1)
         myFeed.update()
@@ -158,7 +164,7 @@ class UnicodeFeedTestCase(framework.EventLoopTest):
         dialogs.delegate = UnicodeTestDelegate()
         dialogs.delegate.choice = dialogs.BUTTON_YES
 
-        myFeed = feed.Feed("file://"+self.filename)
+        myFeed = feed.Feed(u"file://"+self.filename)
         
         self.assertEqual(dialogs.delegate.numCalls,1)
         myFeed.update()
@@ -176,7 +182,7 @@ class UnicodeFeedTestCase(framework.EventLoopTest):
         dialogs.delegate = UnicodeTestDelegate()
         dialogs.delegate.choice = dialogs.BUTTON_YES
 
-        myFeed = feed.Feed("file://"+self.filename)
+        myFeed = feed.Feed(u"file://"+self.filename)
         
         self.assertEqual(dialogs.delegate.numCalls,1)
         myFeed.update()
@@ -196,7 +202,7 @@ class UnicodeFeedTestCase(framework.EventLoopTest):
         dialogs.delegate = UnicodeTestDelegate()
         dialogs.delegate.choice = dialogs.BUTTON_YES
 
-        myFeed = feed.Feed("file://"+self.filename)
+        myFeed = feed.Feed(u"file://"+self.filename)
         
         self.assertEqual(dialogs.delegate.numCalls,1)
         myFeed.update()
@@ -216,7 +222,7 @@ class UnicodeFeedTestCase(framework.EventLoopTest):
         dialogs.delegate = UnicodeTestDelegate()
         dialogs.delegate.choice = dialogs.BUTTON_YES
 
-        myFeed = feed.Feed("file://"+self.filename)
+        myFeed = feed.Feed(u"file://"+self.filename)
         
         self.assertEqual(dialogs.delegate.numCalls,1)
         myFeed.update()
@@ -236,7 +242,7 @@ class UnicodeFeedTestCase(framework.EventLoopTest):
         dialogs.delegate = UnicodeTestDelegate()
         dialogs.delegate.choice = dialogs.BUTTON_YES
 
-        myFeed = feed.Feed("file://"+self.filename)
+        myFeed = feed.Feed(u"file://"+self.filename)
         
         self.assertEqual(dialogs.delegate.numCalls,1)
         myFeed.update()
@@ -259,7 +265,7 @@ class UnicodeFeedTestCase(framework.EventLoopTest):
         dialogs.delegate = UnicodeTestDelegate()
         dialogs.delegate.choice = dialogs.BUTTON_YES
 
-        myFeed = feed.Feed("file://"+self.filename)
+        myFeed = feed.Feed(u"file://"+self.filename)
         
         self.assertEqual(dialogs.delegate.numCalls,1)
         myFeed.update()
@@ -282,7 +288,7 @@ class UnicodeFeedTestCase(framework.EventLoopTest):
         dialogs.delegate = UnicodeTestDelegate()
         dialogs.delegate.choice = dialogs.BUTTON_YES
 
-        myFeed = feed.Feed("file://"+self.filename)
+        myFeed = feed.Feed(u"file://"+self.filename)
         
         self.assertEqual(dialogs.delegate.numCalls,1)
         myFeed.update()
@@ -355,46 +361,46 @@ class UnicodeFeedTestCase(framework.EventLoopTest):
 class TemplateCompilerTest(framework.DemocracyTestCase):
     def testNonUnicode(self):
         # genRepeatText
-        self.assertRaises(TemplateUnicodeError,
+        self.assertRaises(util.DemocracyUnicodeError,
                           lambda : template_compiler.genRepeatText("out","123","    ","boo"))
-        self.assertRaises(TemplateUnicodeError,
+        self.assertRaises(util.DemocracyUnicodeError,
                           lambda : template_compiler.genRepeatText("out","123","    ","Chinese Numbers \x25cb\x4e00\x4e8c\x4e09\x56db\x4e94\x516d\x4e03\x516b\x4e5d"))
         self.assertEqual(u"    out.write(u'Chinese Numbers \\u25cb\\u4e00\\u4e8c\\u4e09\\u56db\\u4e94\\u516d\\u4e03\\u516b\\u4e5d')\n",
                          template_compiler.genRepeatText("out","123","    ",u"Chinese Numbers \u25cb\u4e00\u4e8c\u4e09\u56db\u4e94\u516d\u4e03\u516b\u4e5d"))
 
 
         # genRepeatTranslate
-        self.assertRaises(TemplateUnicodeError,
+        self.assertRaises(util.DemocracyUnicodeError,
                           lambda : template_compiler.genRepeatTranslate("out","123","    ",("boo",{})))
-        self.assertRaises(TemplateUnicodeError,
+        self.assertRaises(util.DemocracyUnicodeError,
                           lambda : template_compiler.genRepeatTranslate("out","123","    ",("Chinese Numbers \x25cb\x4e00\x4e8c\x4e09\x56db\x4e94\x516d\x4e03\x516b\x4e5d",{})))
 
 
         # genRepeatTextHide
-        self.assertRaises(TemplateUnicodeError,
+        self.assertRaises(util.DemocracyUnicodeError,
                           lambda : template_compiler.genRepeatTextHide("out","123","    ",(False,"boo")))
-        self.assertRaises(TemplateUnicodeError,
+        self.assertRaises(util.DemocracyUnicodeError,
                           lambda : template_compiler.genRepeatTextHide("out","123","    ",(False, "Chinese Numbers \x25cb\x4e00\x4e8c\x4e09\x56db\x4e94\x516d\x4e03\x516b\x4e5d")))
-        self.assertRaises(TemplateUnicodeError,
+        self.assertRaises(util.DemocracyUnicodeError,
                           lambda : template_compiler.genRepeatTextHide("out","123","    ",(True,"boo")))
-        self.assertRaises(TemplateUnicodeError,
+        self.assertRaises(util.DemocracyUnicodeError,
                           lambda : template_compiler.genRepeatTextHide("out","123","    ",(True, "Chinese Numbers \x25cb\x4e00\x4e8c\x4e09\x56db\x4e94\x516d\x4e03\x516b\x4e5d")))
 
         self.assertEqual(u"    if not (True):\n        out.write(u'Chinese Numbers \\u25cb\\u4e00\\u4e8c\\u4e09\\u56db\\u4e94\\u516d\\u4e03\\u516b\\u4e5d')\n",
                          template_compiler.genRepeatTextHide("out","123","    ",(True, u"Chinese Numbers \u25cb\u4e00\u4e8c\u4e09\u56db\u4e94\u516d\u4e03\u516b\u4e5d")))
 
         # genHideSection
-        self.assertRaises(TemplateUnicodeError,
+        self.assertRaises(util.DemocracyUnicodeError,
                           lambda : template_compiler.genHideSection("out","123","    ",(False,[(lambda a,b,c,d: "boo","ignored")])))
-        self.assertRaises(TemplateUnicodeError,
+        self.assertRaises(util.DemocracyUnicodeError,
                           lambda : template_compiler.genHideSection("out","123","    ",(False,[(lambda a,b,c,d: "Chinese Numbers \x25cb\x4e00\x4e8c\x4e09\x56db\x4e94\x516d\x4e03\x516b\x4e5d","ignored")])))
         self.assertEqual(u'    if not (False):\nChinese Numbers \u25cb\u4e00\u4e8c\u4e09\u56db\u4e94\u516d\u4e03\u516b\u4e5d',
                           template_compiler.genHideSection("out","123","    ",(False,[(lambda a,b,c,d: u"Chinese Numbers \u25cb\u4e00\u4e8c\u4e09\u56db\u4e94\u516d\u4e03\u516b\u4e5d","ignored")])))
 
         # genQuoteAndFillAttr
-        self.assertRaises(TemplateUnicodeError,
+        self.assertRaises(util.DemocracyUnicodeError,
                           lambda:template_compiler.genQuoteAndFillAttr("out","123","    ","boo"))
-        self.assertRaises(TemplateUnicodeError,
+        self.assertRaises(util.DemocracyUnicodeError,
                           lambda:template_compiler.genQuoteAndFillAttr("out","123","    ","Chinese Numbers \x25cb\x4e00\x4e8c\x4e09\x56db\x4e94\x516d\x4e03\x516b\x4e5d"))
         self.assertEqual(u"    out.write(quoteAndFillAttr(u'Chinese Numbers \\u25cb\\u4e00\\u4e8c\\u4e09\\u56db\\u4e94\\u516d\\u4e03\\u516b\\u4e5d',locals()))\n",
                          template_compiler.genQuoteAndFillAttr("out","123","    ",u"Chinese Numbers \u25cb\u4e00\u4e8c\u4e09\u56db\u4e94\u516d\u4e03\u516b\u4e5d"))
