@@ -14,8 +14,20 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+#
+# In addition, as a special exception, the copyright holders give
+# permission to link the code of portions of this program with the OpenSSL
+# library.
+#
+# You must obey the GNU General Public License in all respects for all of
+# the code used other than OpenSSL. If you modify file(s) with this
+# exception, you may extend this exception to your version of the file(s),
+# but you are not obligated to do so. If you do not wish to do so, delete
+# this exception statement from your version. If you delete this exception
+# statement from all source files in the program, then also delete it here.
 
 import os
+import random
 import re
 import sys
 import sha
@@ -25,13 +37,13 @@ import urllib
 import socket
 import logging
 import filetypes
+import tempfile
 import threading
 import traceback
 import subprocess
 
 from clock import clock
 from types import UnicodeType, StringType
-from BitTorrent.bencode import bdecode, bencode
 
 # Should we print out warning messages.  Turn off in the unit tests.
 chatter = True
@@ -39,6 +51,8 @@ chatter = True
 inDownloader = False
 # this gets set to True when we're in the download process.
 
+ignoreErrors = False
+
 # Perform escapes needed for Javascript string contents.
 def quoteJS(x):
     x = x.replace("\\", "\\\\") # \       -> \\
@@ -111,9 +125,9 @@ def writeSimpleConfigFile(path, data):
 # this checkout. Going to fail without Cygwin. Yeah, oh well. Pass the
 # file or directory you want to use as a reference point. Returns an
 # integer on success or None on failure.
-def queryRevision(file):
+def queryRevision(f):
     try:
-        p = subprocess.Popen(["svn", "info", file], stdout=subprocess.PIPE) 
+        p = subprocess.Popen(["svn", "info", f], stdout=subprocess.PIPE) 
         info = p.stdout.read()
         p.stdout.close()
         url = re.search("URL: (.*)", info).group(1)
@@ -135,13 +149,7 @@ def absolutePathToFileURL(path):
         path = path.encode("utf-8")
     parts = string.split(path, os.sep)
     parts = [urllib.quote(x, ':') for x in parts]
-    if len(parts) > 0 and parts[0] == '':
-        # Don't let "/foo/bar" become "file:////foo/bar", but leave
-        # "c:/foo/bar" becoming "file://c:/foo/bar" -- technically :
-        # should become | (but only in a drive name?) but most
-        # consumers will let us get by with that.
-        parts = parts[1:]
-    return "file:///" + '/'.join(parts)
+    return "file://" + '/'.join(parts)
 
 
 # Shortcut for 'failed' with the exception flag.
@@ -165,11 +173,13 @@ def failed(when, withExn = False, details = None):
         header += "App:        %s\n" % config.get(prefs.LONG_APP_NAME)
         header += "Publisher:  %s\n" % config.get(prefs.PUBLISHER)
         header += "Platform:   %s\n" % config.get(prefs.APP_PLATFORM)
-        header += "Python:     %s\n" % sys.version
+        header += "Python:     %s\n" % sys.version.replace("\r\n"," ").replace("\n"," ").replace("\r"," ")
         header += "Py Path:    %s\n" % repr(sys.path)
         header += "Version:    %s\n" % config.get(prefs.APP_VERSION)
         header += "Serial:     %s\n" % config.get(prefs.APP_SERIAL)
         header += "Revision:   %s\n" % config.get(prefs.APP_REVISION)
+        header += "Builder:    %s\n" % config.get(prefs.BUILD_MACHINE)
+        header += "Build Time: %s\n" % config.get(prefs.BUILD_TIME)
     except KeyboardInterrupt:
         raise
     except:
@@ -233,7 +243,7 @@ def failed(when, withExn = False, details = None):
             logContents = readLog(downloaderLogFile)
 
     if logContents is not None:
-        report += "{{{\n%s}}}\n" % logContents
+        report += "{{{\n%s}}}\n" % stringify(logContents)
 
     # Dump the header for the report we just generated to the log, in
     # case there are multiple failures or the user sends in the log
@@ -246,9 +256,11 @@ def failed(when, withExn = False, details = None):
 
     if not inDownloader:
         try:
-            import app
-            app.delegate. \
-                notifyUnkownErrorOccurence(when, log = report)
+            import dialogs
+            from gtcache import gettext as _
+            if not ignoreErrors:
+                chkboxdialog = dialogs.CheckboxTextboxDialog(_("Internal Error"),_("Miro has encountered an internal error. You can help us track down this problem and fix it by submitting an error report."), _("Include entire program database including all video and channel metadata with crash report"), False, _("Describe what you were doing that caused this error"), dialogs.BUTTON_SUBMIT_REPORT, dialogs.BUTTON_IGNORE)
+                chkboxdialog.run(lambda x: _sendReport(report, x))
         except Exception, e:
             logging.exception ("Execption when reporting errror..")
     else:
@@ -256,6 +268,54 @@ def failed(when, withExn = False, details = None):
         c = command.DownloaderErrorCommand(daemon.lastDaemon, report)
         c.send()
 
+def _sendReport(report, dialog):
+    def callback(result):
+        app.controller.sendingCrashReport -= 1
+        if result['status'] != 200 or result['body'] != 'OK':
+            logging.warning(u"Failed to submit crash report. Server returned %r" % result)
+        else:
+            logging.info(u"Crash report submitted successfully")
+    def errback(error):
+        app.controller.sendingCrashReport -= 1
+        logging.warning(u"Failed to submit crash report %r" % error)
+
+    import dialogs
+    import httpclient
+    import config
+    import prefs
+    import app
+
+    global ignoreErrors
+    if dialog.choice == dialogs.BUTTON_IGNORE:
+        ignoreErrors = True
+        return
+
+    backupfile = None
+    if hasattr(dialog,"checkbox_value") and dialog.checkbox_value:
+        try:
+            logging.info("Sending entire database")
+            import database
+            backupfile = database.defaultDatabase.liveStorage.backupDatabase()
+        except:
+            traceback.print_exc()
+            logging.warning(u"Failed to backup database")
+
+
+    description = u"Description text not implemented"
+    if hasattr(dialog,"textbox_value"):
+        description = dialog.textbox_value
+
+    description = description.encode("utf-8")
+    postVars = {"description":description,
+                "app_name": config.get(prefs.LONG_APP_NAME),
+                "log": report}
+    if backupfile:
+        postFiles = {"databasebackup": {"filename":"databasebackup.zip", "mimetype":"application/octet-stream", "handle":open(backupfile, "rb")}}
+    else:
+        postFiles = None
+    app.controller.sendingCrashReport += 1
+    httpclient.grabURL("http://participatoryculture.org/bogondeflector/index.php", callback, errback, method="POST", postVariables = postVars, postFiles = postFiles)
+
 class AutoflushingStream:
     """Converts a stream to an auto-flushing one.  It behaves in exactly the
     same way, except all write() calls are automatically followed by a
@@ -319,7 +379,7 @@ def timeTrapCall(when, function, *args, **kwargs):
     end = clock()
     if cancel:
         return retval
-    if end-start > 0.5:
+    if end-start > 1.0:
         logging.timing ("WARNING: %s too slow (%.3f secs)",
             when, end-start)
     if TRACK_CUMULATIVE:
@@ -340,11 +400,12 @@ def timeTrapCall(when, function, *args, **kwargs):
     return retval
 
 def getTorrentInfoHash(path):
+    import libtorrent as lt
     f = open(path, 'rb')
     try:
         data = f.read()
-        metainfo = bdecode(data)
-        infohash = sha.sha(bencode(metainfo['info'])).digest()
+        metainfo = lt.bdecode(data)
+        infohash = sha.sha(lt.bencode(metainfo['info'])).digest()
         return infohash
     finally:
         f.close()
@@ -434,10 +495,14 @@ def formatTimeForUser(seconds, sign=1):
     display a '-' sign.
     """
     _, _, _, h, m, s, _, _, _ = time.gmtime(seconds)
-    if int(seconds) in range(0, 3600):
-        return "%d:%02u" % (m*sign, s)
+    if sign < 0:
+        sign = '-'
     else:
-        return "%d:%02u:%02u" % (h*sign, m, s)
+        sign = ''
+    if int(seconds) in range(0, 3600):
+        return "%s%d:%02u" % (sign, m, s)
+    else:
+        return "%s%d:%02u:%02u" % (sign, h, m, s)
 
 def makeAnchor(label, href):
     return '<a href="%s">%s</a>' % (href, label)
@@ -503,6 +568,8 @@ class ThreadSafeCounter:
 def setupLogging():
     logging.addLevelName(25, "TIMING")
     logging.timing = lambda msg, *args, **kargs: logging.log(25, msg, *args, **kargs)
+    logging.addLevelName(26, "JSALERT")
+    logging.jsalert = lambda msg, *args, **kargs: logging.log(26, msg, *args, **kargs)
 
 
 # Returned when input to a template function isn't unicode
@@ -565,6 +632,7 @@ def checkF(text):
     if text is not None and type(text) != FilenameType:
         raise DemocracyUnicodeError, (u"text \"%s\" is not a valid filename type" %
                                      text)
+
 # Decorator that raised an exception if the function doesn't return a filename
 def returnsFilename(func):
     def checkFunc(*args, **kwargs):
@@ -574,8 +642,9 @@ def returnsFilename(func):
         return result
     return checkFunc
 
-# Turns all strings in data structure to unicode
 def unicodify(d):
+    """Turns all strings in data structure to unicode.
+    """
     if isinstance(d, dict):
         for key in d.keys():
             d[key] = unicodify(d[key])
@@ -586,9 +655,29 @@ def unicodify(d):
         d = d.decode('ascii','replace')
     return d
 
-# Quote international characters contained in a URL according to w3c, see:
-# <http://www.w3.org/International/O-URL-code.html>
+def stringify(u, handleerror="xmlcharrefreplace"):
+    """Takes a possibly unicode string and converts it to a string string.
+    This is required for some logging especially where the things being
+    logged are filenames which can be Unicode in the Windows platform.
+
+    Note that this is not the inverse of unicodify.
+
+    You can pass in a handleerror argument which defaults to "xmlcharrefreplace".
+    This will increase the string size as it converts unicode characters that
+    don't have ascii equivalents into escape sequences.  If you don't want to
+    increase the string length, use "replace" which will use ? for unicode
+    characters that don't have ascii equivalents.
+    """
+    if isinstance(u, unicode):
+        return u.encode("ascii", handleerror)
+    if not isinstance(u, str):
+        return str(u)
+    return u
+
 def quoteUnicodeURL(url):
+    """Quote international characters contained in a URL according to w3c, see:
+    <http://www.w3.org/International/O-URL-code.html>
+    """
     checkU(url)
     quotedChars = list()
     for c in url.encode('utf8'):
@@ -598,24 +687,75 @@ def quoteUnicodeURL(url):
             quotedChars.append(c)
     return u''.join(quotedChars)
 
-def call_command(*args):
-    """Call an external command.  If the command doesn't exit with status 0,
-    or if it outputs to stderr, an exception will be raised.  Returns stdout.
+def no_console_startupinfo():
+    """Returns the startupinfo argument for subprocess.Popen so that we don't
+    open a console window.  On platforms other than windows, this is just
+    None.  On windows, it's some win32 sillyness.
     """
     if subprocess.mswindows:
         startupinfo = subprocess.STARTUPINFO()
         startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
+        return startupinfo
     else:
-        startupinfo = None
+        return None
+
+def call_command(*args, **kwargs):
+    """Call an external command.  If the command doesn't exit with status 0,
+    or if it outputs to stderr, an exception will be raised.  Returns stdout.
+    """
+    ignore_stderr = kwargs.pop('ignore_stderr', False)
+    if kwargs:
+        raise TypeError('extra keyword arguments: %s' % kwargs)
+
     pipe = subprocess.Popen(args, stdout=subprocess.PIPE,
             stdin=subprocess.PIPE, stderr=subprocess.PIPE,
-            startupinfo=startupinfo)
+            startupinfo=no_console_startupinfo())
     stdout, stderr = pipe.communicate()
     if pipe.returncode != 0:
-        raise OSError("call_command with %s has return code %s" % 
-                (args, pipe.returncode))
-    elif stderr:
-        raise OSError("call_command outputed error text:\n%s" % stderr)
+        raise OSError("call_command with %s has return code %s\nstdout:%s\nstderr:%s" % 
+                (args, pipe.returncode, stdout, stderr))
+    elif stderr and not ignore_stderr:
+        raise OSError("call_command with %s outputed error text:\n%s" % 
+                (args, stderr))
     else:
         return stdout
 
+def getsize(path):
+    """Get the size of a path.  If it's a file, return the size of the file.
+    If it's a directory return the total size of all the files it contains.
+    """
+
+    if os.path.isdir(path):
+        size = 0
+        for (dirpath, dirnames, filenames) in os.walk(path):
+            for name in filenames:
+                size += os.path.getsize(os.path.join(dirpath, name))
+            size += os.path.getsize(dirpath)
+        return size
+    else:
+        return os.path.getsize(path)
+
+def partition(list, size):
+    """Partiction list into smaller lists such that none is larger than
+    size elements.
+
+    Returns a list of lists.  The lists appended together will be the original
+    list.
+    """
+    retval = []
+    for start in range(0, len(list), size):
+        retval.append(list[start:start+size])
+    return retval
+
+def directoryWritable(directory):
+    """Check if we can write to a directory."""
+    try:
+        f = tempfile.TemporaryFile(dir=directory)
+    except OSError:
+        return False
+    else:
+        f.close()
+        return True
+
+def random_string(length):
+    return ''.join(random.choice(string.ascii_letters) for i in xrange(length))
